/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef ___BTYPE_H___
#define ___BTYPE_H___

typedef enum b_type {
	bt_float = 0,
	bt_integer,
	bt_string,
	bt_object,
	bt_exception
} b_type;

typedef enum b_exception {
	be_none = 0,	/* No error. */
	be_syntax,		/* ?SYNTAX ERROR */
	be_subscript,	/* ?SUBSCRIPT OUT OF RANGE */
	be_type,		/* ?INVALID TYPE */
	be_div0,		/* ?DIVISION BY ZERO */
	be_formula,		/* ?FORMULA TOO COMPLEX */
	be_range,		/* ?OUT OF RANGE ERROR */
	be_data,		/* ?OUT OF DATA ERROR */
	be_cont,		/* ?CAN'T CONTINUE */
	be_label,		/* ?LABEL NOT FOUND */
	be_noend,		/* ?STRUCTURE WITHOUT END */
	be_else,		/* ?ELSE WITHOUT IF */
	be_endif,		/* ?ENDIF WITHOUT IF */
	be_case,        /* ?CASE WITHOUT SWITCH */
	be_wend,		/* ?WEND WITHOUT WHILE */
	be_loop,		/* ?LOOP WITHOUT DO */
	be_next,		/* ?NEXT WITHOUT FOR */
	be_return,		/* ?RETURN WITHOUT GOSUB */
	be_onerr,		/* ?ONERR WITHOUT TRY */
	be_endtry,		/* ?ENDTRY WITHOUT TRY */
	be_memory,		/* ?OUT OF MEMORY ERROR */
	be_nullptr,		/* ?NULL OBJECT ERROR */
	be_undeclared,	/* ?UNDECLARED VARIABLE */
	be_constant,    /* ?CANNOT CHANGE CONSTANT */
	be_arrtype,		/* ?CANNOT CHANGE ARRAY TYPE */
	be_cast,		/* ?ILLEGAL CAST ERROR */
	be_private,		/* ?PRIVATE MEMBER ACCESSED OUTSIDE CLASS */
	be_404,			/* ?FILE NOT FOUND */
	be_eof,
	be_xdev,
	be_perm,
	be_access,
	be_toolong,
	be_notdir,
	be_notfile,
	be_notempty,
	be_rofs,
	be_exists,
	be_mfile,
	be_mlink,
	be_cloop,
	be_nospc,
	be_filedes,
	be_inuse,
	be_noexec,
	be_toobig,
	be_io,
	be_break,		/* ?BREAK */
	
	be_max
} b_exception;

typedef struct b_object {
	char           * className;
	int              useCount;
	struct b_scope * classScope;
	struct b_scope * theObject;
} b_object;

typedef struct b_varl {
	b_type			 type; /* type */
	struct b_scope * otyp; /* object type, aka class */
	int              dyna; /* dynamically allocated */
	int              readOnly; /* constant */
	double			 fval; /* value as a floating point */
	signed long int	 ival; /* value as an integer */
	char *			 sval; /* value as a string (pointer to char) */
	b_object *		 oval; /* value as an object (pointer to b_object) */
	b_exception		 eval; /* value as an exception */
} b_varl; /* It's called varl because it could be a var[iable] or a val[ue] */

typedef struct b_array {
	b_type   type; /* type */
	int      dim ; /* number of dimensions */
	int      dim1; /* size in dimension 1 */
	int      dim2; /* size in dimension 2 */
	int      dim3; /* size in dimension 3 */
	b_varl * elem; /* elements of the array */
} b_array;

typedef struct b_usub {
	int      paramCount;
	char *   paramNames[4];
	b_type   paramTypes[4];
	char *   paramList;
	char *   code;
} b_usub;

typedef struct b_ufunc {
	b_type   returnType;
	int      paramCount;
	char *   paramNames[4];
	b_type   paramTypes[4];
	char *   paramList;
	char *   code;
} b_ufunc;

typedef struct b_scope {
	struct b_scope *   fallback;
	/* - V A R I A B L E S - */
	int                count;
	b_varl *         * vars;
	char *           * names;
	/* ---- A R R A Y S ---- */
	int                acount;
	b_array *        * arrays;
	char *           * anames;
	/* -- U S E R S U B S -- */
	int                uscount;
	b_usub *         * usubs;
	char *           * usnames;
	/* - U S E R F U N C S - */
	int                ufcount;
	b_ufunc *        * ufuncs;
	char *           * ufnames;
	/* --- C L A S S E S --- */
	int                ccount;
	struct b_scope * * classes;
	char *           * cnames;
} b_scope; /* variable scope */

typedef struct b_function {
	char * name;
	b_varl * (* fs)(b_scope *, char *);
	b_varl * (* f0)(b_scope *);
	b_varl * (* f1)(b_scope *, b_varl *);
	b_varl * (* f2)(b_scope *, b_varl *, b_varl *);
	b_varl * (* f3)(b_scope *, b_varl *, b_varl *, b_varl *);
	b_varl * (* f4)(b_scope *, b_varl *, b_varl *, b_varl *, b_varl *);
	b_varl * (* fv)(b_scope *, int, b_varl **);
	/*
		If a function is called with a certain number of parameters and
		the corresponding pointer in this structure is null, the inter-
		preter must throw a syntax error.
	*/
} b_function;

typedef struct b_command {
	char * name;
	int autoVars; /* if this is true, the interpreter will create variables which don't exist yet */
	void (* fs)(b_scope *, char *);
	void (* f0)(b_scope *);
	void (* f1)(b_scope *, b_varl *);
	void (* f2)(b_scope *, b_varl *, b_varl *);
	void (* f3)(b_scope *, b_varl *, b_varl *, b_varl *);
	void (* f4)(b_scope *, b_varl *, b_varl *, b_varl *, b_varl *);
	void (* fv)(b_scope *, int, b_varl **);
	/*
		If a command is executed with a certain number of parameters and
		the corresponding pointer in this structure is null, the inter-
		preter must throw a syntax error.
	*/
} b_command;

typedef enum b_looptype {
	bl_for,
	bl_foreach,
	bl_while,
	bl_doloop,
	bl_foreachchar,
	bl_foreachline
} b_looptype;

typedef struct b_loopstackframe {
	struct b_loopstackframe * parent;
	b_looptype type;
	char * start;
	/* while */
	char * conditionExpr;
	/* for, foreach */
	b_varl * loopvar;
	double begin;
	double end;
	double step;
	/* foreach */
	b_array * looparr;
} b_loopstackframe;

typedef struct b_trystackframe {
	struct b_trystackframe * parent;
	b_varl * catchVar;
	char * catchLine;
} b_trystackframe;

typedef struct b_substackframe {
	struct b_substackframe * parent;
	char * calledFrom;
	b_loopstackframe * lsf;
	b_trystackframe * tsf;
} b_substackframe;

#endif
