/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "btype.h"
#include "vars.h"
#include "expr.h"
#include "exceptions.h"
#include "functions.h"
#include <stdio.h>
#include <string.h>
#include "stringx.h"
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include "mathx.h"

extern void bf_lets(b_scope * s, char * c); /* for implied let */
extern void bf_dims(b_scope * s, char * c); /* for user defined funcs */

/* * * * * * * * * * */
/*   INCLUDE PATH    */
/* * * * * * * * * * */
static char bx_includepath[512] = {0};

void bx_set_include_path(char * s)
{
	strcpy(bx_includepath, s);
}

char * bx_get_include_path()
{
	return bx_includepath;
}

/* * * * * * * * * * * * * * * */
/*   QUOTED STRING FUNCTIONS   */
/* * * * * * * * * * * * * * * */
char * bx_strchrq(char * s, char ch)
{
	int quote = 0;
	if (!s) return 0;
	while (s && (*s)) {
		if (quote) {
			if ((*s) == '\\') s++;
			else if (bx_quote(*s)) quote = 0;
		} else if (bx_quote(*s)) {
			quote = 1;
		} else if ((*s) == ch) {
			return s;
		}
		s++;
	}
	return 0;
}

char * bx_strrchrq(char * ss, char ch)
{
	int quote = 0;
	if (!ss) return 0;
	char * s = ss + strlen(ss) - 1;
	while (s && (*s) && s>=ss) {
		if (quote) {
			if (bx_quote(*s)) {
				if (s==ss || (*(s-1))!='\\') quote = 0;
			}
		} else if (bx_quote(*s)) {
			quote = 1;
		} else if ((*s) == ch) {
			return s;
		}
		s--;
	}
	return 0;
}

char * bx_strstrq(char * s, char * ch)
{
	int quote = 0;
	int n = strlen(ch);
	if (!s) return 0;
	while (s && (*s)) {
		if (quote) {
			if ((*s) == '\\') s++;
			else if (bx_quote(*s)) quote = 0;
		} else if (bx_quote(*s)) {
			quote = 1;
		} else if (!strncmp(s,ch,n)) {
			return s;
		}
		s++;
	}
	return 0;
}

char * bx_strrstrq(char * ss, char * ch)
{
	int quote = 0;
	int n = strlen(ch);
	if (!ss) return 0;
	char * s = ss + strlen(ss) - 1;
	while (s && (*s) && s>=ss) {
		if (quote) {
			if (bx_quote(*s)) {
				if (s==ss || (*(s-1))!='\\') quote = 0;
			}
		} else if (bx_quote(*s)) {
			quote = 1;
		} else if (!strncmp(s,ch,n)) {
			return s;
		}
		s--;
	}
	return 0;
}
char * bx_strcasechrq(char * s, char ch)
{
	int quote = 0;
	if (!s) return 0;
	while (s && (*s)) {
		if (quote) {
			if ((*s) == '\\') s++;
			else if (bx_quote(*s)) quote = 0;
		} else if (bx_quote(*s)) {
			quote = 1;
		} else if (bx_toUpper(*s) == bx_toUpper(ch)) {
			return s;
		}
		s++;
	}
	return 0;
}

char * bx_strcaserchrq(char * ss, char ch)
{
	int quote = 0;
	if (!ss) return 0;
	char * s = ss + strlen(ss) - 1;
	while (s && (*s) && s>=ss) {
		if (quote) {
			if (bx_quote(*s)) {
				if (s==ss || (*(s-1))!='\\') quote = 0;
			}
		} else if (bx_quote(*s)) {
			quote = 1;
		} else if (bx_toUpper(*s) == bx_toUpper(ch)) {
			return s;
		}
		s--;
	}
	return 0;
}

char * bx_strcasestrq(char * s, char * ch)
{
	int quote = 0;
	int n = strlen(ch);
	if (!s) return 0;
	while (s && (*s)) {
		if (quote) {
			if ((*s) == '\\') s++;
			else if (bx_quote(*s)) quote = 0;
		} else if (bx_quote(*s)) {
			quote = 1;
		} else if (!strncasecmp(s,ch,n)) {
			return s;
		}
		s++;
	}
	return 0;
}

char * bx_strcaserstrq(char * ss, char * ch)
{
	int quote = 0;
	int n = strlen(ch);
	if (!ss) return 0;
	char * s = ss + strlen(ss) - 1;
	while (s && (*s) && s>=ss) {
		if (quote) {
			if (bx_quote(*s)) {
				if (s==ss || (*(s-1))!='\\') quote = 0;
			}
		} else if (bx_quote(*s)) {
			quote = 1;
		} else if (!strncasecmp(s,ch,n)) {
			return s;
		}
		s--;
	}
	return 0;
}

/* * * * * * * * * * * * * * * * * * */
/*   STRING FORMAT CHECKING THINGS   */
/* * * * * * * * * * * * * * * * * * */
int bx_isFloat(char * s)
{
	int pCount = 0;
	if (!s) return 0;
	if (*s == '-' || *s == '+') s++;
	while (*s) {
		if (*s == '.') {
			pCount++;
			if (pCount > 1) return 0;
		} else if (!isdigit(*s)) {
			return 0;
		}
		s++;
	}
	return 1;
}

int bx_isInt(char * s)
{
	if (!s) return 0;
	if (*s == '-' || *s == '+') s++;
	while (*s) {
		if (!isdigit(*s)) return 0;
		s++;
	}
	return 1;
}

int bx_isOctal(char * s)
{
	if (!s) return 0;
	if (*s != '0') return 0; s++;
	if (!(*s)) return 0;
	while (*s) {
		if (*s < '0' || *s > '7') return 0;
		s++;
	}
	return 1;
}

int bx_isHex(char * s)
{
	if (!s) return 0;
	if (*s != '0') return 0; s++;
	if (*s != 'x' && *s != 'X') return 0; s++;
	if (!(*s)) return 0;
	while (*s) {
		if (!isxdigit(*s)) return 0;
		s++;
	}
	return 1;
}

int bx_isBinary(char * s)
{
	if (!s) return 0;
	if (*s != '0') return 0; s++;
	if (*s != 'b' && *s != 'B') return 0; s++;
	if (!(*s)) return 0;
	while (*s) {
		if (*s != '0' && *s != '1') return 0;
		s++;
	}
	return 1;
}

/* * * * * * * * * * * * * * * * * */
/*   IDENTIFIER STRING FUNCTIONS   */
/* * * * * * * * * * * * * * * * * */
int bx_verifyIdent(char * s)
{
	int i;
	if (!bx_identStart(s[0])) return 0;
	for (i = strlen(s)-1; i>0; i--) {
		if (!bx_identChar(s[i])) return 0;
	}
	return 1;
}

char * bx_normalizeIdent(char * s)
{
	char * ws = strdup(s); /* our own working copy of the string; must be freed by calling function */
	int i;
	for (i = strlen(ws)-1; i>=0; i--) {
		if (islower(ws[i])) ws[i] -= 32;
	}
	return ws;
}

b_type bx_autoTypeIdent(char * s)
{
	char * t = s;
	if ((*t) == '#') return bt_integer;
	while (bx_identChar(*t)) t++;
	t--;
	if (t>=s) switch (*t) {
		case '#': return bt_integer; break;
		case '$': return bt_string; break;
		case '@': return bt_object; break;
		default: return bt_float; break;
	} else {
		return bt_float;
	}
}

/* * * * * * * * * * * * */
/*   PARSING FUNCTIONS   */
/* * * * * * * * * * * * */
char * bx_ffToFirstStatement(char * buf)
{
	while ((*buf) && isspace(*buf)) buf++;
	return buf;
}

char * bx_ffToEndOfLine(char * buf)
{
	while ((*buf) && !((*buf)=='\n' || (*buf)=='\r')) buf++;
	return buf;
}

char * bx_ffToEndOfStatement(char * buf)
{
	int level = 0;
	int quote = 0;
	while ((*buf) && ( level>0 || quote || !bx_statementDelim(*buf) )) {
		if (quote) {
			if ((*buf) == '\\') buf++;
			else if (bx_quote(*buf)) quote = 0;
		} else if (bx_quote(*buf)) {
			quote = 1;
		} else if (bx_levelInc(*buf)) {
			level++;
		} else if (bx_levelDec(*buf)) {
			level--;
		}
		buf++;
	}
	return buf;
}

char * bx_ffToNextStatement(char * buf)
{
	buf = bx_ffToEndOfStatement(buf);
	while ((*buf) && ( bx_statementDelim(*buf) || isspace(*buf) )) buf++;
	return buf;
}

char * bx_ffToFirstParameter(char * buf)
{
	while ((*buf) && !( isspace(*buf) || bx_statementDelim(*buf) )) buf++;
	while ((*buf) && bx_space(*buf)) buf++;
	return buf;
}

char * bx_ffToEndOfParameter(char * buf)
{
	int level = 0;
	int quote = 0;
	while ((*buf) && ( level>0 || quote || !(bx_paramDelim(*buf) || bx_statementDelim(*buf)) )) {
		if (quote) {
			if ((*buf) == '\\') buf++;
			else if (bx_quote(*buf)) quote = 0;
		} else if (bx_quote(*buf)) {
			quote = 1;
		} else if (bx_levelInc(*buf)) {
			level++;
		} else if (bx_levelDec(*buf)) {
			level--;
		}
		buf++;
	}
	return buf;
}

char * bx_ffToNextParameter(char * buf)
{
	buf = bx_ffToEndOfParameter(buf);
	if (bx_paramDelim(*buf)) buf++;
	while ((*buf) && bx_space(*buf)) buf++;
	return buf;
}

char * bx_ffToFirstFParameter(char * buf)
{
	while ((*buf) && bx_space(*buf)) buf++;
	if (bx_levelInc(*buf)) buf++;
	while ((*buf) && bx_space(*buf)) buf++;
	return buf;
}

char * bx_ffToEndOfFParameter(char * buf)
{
	int level = 0;
	int quote = 0;
	while ((*buf) && ( level>0 || quote || !(bx_paramDelim(*buf) || bx_statementDelim(*buf) || bx_levelDec(*buf)) )) {
		if (quote) {
			if ((*buf) == '\\') buf++;
			else if (bx_quote(*buf)) quote = 0;
		} else if (bx_quote(*buf)) {
			quote = 1;
		} else if (bx_levelInc(*buf)) {
			level++;
		} else if (bx_levelDec(*buf)) {
			level--;
		}
		buf++;
	}
	return buf;
}

char * bx_ffToNextFParameter(char * buf)
{
	buf = bx_ffToEndOfFParameter(buf);
	if (bx_paramDelim(*buf)) buf++;
	while ((*buf) && bx_space(*buf)) buf++;
	return buf;
}

char * bx_ffToIdentifierEnd(char * buf, int atStart)
{
	if (atStart) {
		if (bx_identStart(*buf)) buf++;
		else return buf;
	}
	while (bx_identChar(*buf)) buf++;
	return buf;
}

/* * * * * * * * * * * * * * * * * * * * */
/*   PARAMETER LIST PARSING FUNCTIONS    */
/* * * * * * * * * * * * * * * * * * * * */
static int bx_countPlist(char * p) {
	int i = 0;
	while (!bx_statementDelim(*p)) {
		i++;
		p = bx_ffToNextParameter(p);
	}
	return i;
}

static int bx_evalPlist(b_scope * sc, char * p, int autoVars, int * param, b_varl ** params, int pmax) {
	char * q;
	char * tmp;
	b_varl * tmpv = 0;
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		tmp = substr(p,q);
		tmpv = bx_eval(sc,tmp,autoVars);
		if (!tmpv) {
			be_cthrow(be_undeclared);
			free(tmp);
			break;
		} else {
			if ((*param) >= pmax) {
				be_cthrow(be_syntax);
				free(tmp);
				break;
			}
			params[*param] = tmpv;
			(*param)++;
		}
		free(tmp);
		p = bx_ffToNextParameter(p);
	}
	return (!be_checkerr());
}

static void bx_freeBvarls(int n, b_varl ** v)
{
	while (n) {
		n--;
		bv_cfreeBv(v[n]);
	}
}

static void bx_freeStrings(int n, char ** v)
{
	while (n) {
		n--;
		free(v[n]);
	}
}

/* * * * * * * * * * * * * * * * * */
/*   COMMAND & FUNCTION CALLING    */
/* * * * * * * * * * * * * * * * * */
int bx_callCommand(b_scope * sc, b_command * c, char * p)
{
	if (c) {
		if (c->fs) {
			char * stuff;
			char * q = bx_ffToEndOfStatement(p);
			stuff = substr(p,q);
			c->fs(sc, stuff);
			free(stuff);
		} else {
			int paramc = bx_countPlist(bx_ffToFirstParameter(p));
			int param = 0;
			b_varl ** params = calloc(sizeof(b_varl *), paramc);
			if (!params) {
				be_throw(be_memory);
			} else {
				if (bx_evalPlist(sc, bx_ffToFirstParameter(p), c->autoVars, &param, params, paramc)) {
					switch (param) {
					case 0:
						     if (c->f0) c->f0(sc);
						else if (c->fv) c->fv(sc,param,params);
						else            be_throw(be_syntax);
						break;
					case 1:
						     if (c->f1) c->f1(sc,params[0]);
						else if (c->fv) c->fv(sc,param,params);
						else            be_throw(be_syntax);
						break;
					case 2:
						     if (c->f2) c->f2(sc,params[0],params[1]);
						else if (c->fv) c->fv(sc,param,params);
						else            be_throw(be_syntax);
						break;
					case 3:
						     if (c->f3) c->f3(sc,params[0],params[1],params[2]);
						else if (c->fv) c->fv(sc,param,params);
						else            be_throw(be_syntax);
						break;
					case 4:
						     if (c->f4) c->f4(sc,params[0],params[1],params[2],params[3]);
						else if (c->fv) c->fv(sc,param,params);
						else            be_throw(be_syntax);
						break;
					default:
						if (c->fv) c->fv(sc,param,params);
						else be_throw(be_syntax);
						break;
					}
				}
				bx_freeBvarls(param, params);
				free(params);
			}
		}
		return 1;
	} else {
		return 0;
	}
}

int bx_callUsub(b_scope * osc, b_usub * us, char * p)
{
	if (us) {
		int param = 0;
		b_varl * params[4] = {0,0,0,0};
		if (bx_evalPlist(osc, bx_ffToFirstParameter(p), 0, &param, params, 4)) {
			if (param != us->paramCount) {
				be_throw(be_syntax);
			} else {
				b_scope * nsc = bv_newScopeChild(osc);
				int ii;
				bf_dims(nsc, us->paramList);
				for (ii=0; ii<param; ii++) {
					bv_sets(nsc, us->paramNames[ii], params[ii]);
				}
				if (!be_checkerr()) {
					bv_cfreeBv(bx_exec(nsc,us->code));
				}
				bv_freeScope(nsc,0);
			}
		}
		bx_freeBvarls(param, params);
		return 1;
	} else {
		return 0;
	}
}

b_varl * bx_callFunction(b_scope * sc, b_function * f, char * o, int autoVars)
{
	b_varl * rv = 0;
	if (f) {
		if (f->fs) {
			rv = f->fs(sc, o);
			if (!rv) be_cthrow(be_cast);
		} else {
			int paramc = bx_countPlist(bx_ffToFirstStatement(o));
			int param = 0;
			b_varl ** params = calloc(sizeof(b_varl *), paramc);
			if (!params) {
				be_throw(be_memory);
			} else {
				if (bx_evalPlist(sc, bx_ffToFirstStatement(o), autoVars, &param, params, paramc)) {
					switch (param) {
					case 0:
						     if (f->f0) { rv = f->f0(sc); if (!rv) be_cthrow(be_cast); }
						else if (f->fv) { rv = f->fv(sc,param,params); if (!rv) be_cthrow(be_cast); }
						else            be_throw(be_syntax);
						break;
					case 1:
						     if (f->f1) { rv = f->f1(sc,params[0]); if (!rv) be_cthrow(be_cast); }
						else if (f->fv) { rv = f->fv(sc,param,params); if (!rv) be_cthrow(be_cast); }
						else            be_throw(be_syntax);
						break;
					case 2:
						     if (f->f2) { rv = f->f2(sc,params[0],params[1]); if (!rv) be_cthrow(be_cast); }
						else if (f->fv) { rv = f->fv(sc,param,params); if (!rv) be_cthrow(be_cast); }
						else            be_throw(be_syntax);
						break;
					case 3:
						     if (f->f3) { rv = f->f3(sc,params[0],params[1],params[2]); if (!rv) be_cthrow(be_cast); }
						else if (f->fv) { rv = f->fv(sc,param,params); if (!rv) be_cthrow(be_cast); }
						else            be_throw(be_syntax);
						break;
					case 4:
						     if (f->f4) { rv = f->f4(sc,params[0],params[1],params[2],params[3]); if (!rv) be_cthrow(be_cast); }
						else if (f->fv) { rv = f->fv(sc,param,params); if (!rv) be_cthrow(be_cast); }
						else            be_throw(be_syntax);
						break;
					default:
						if (f->fv) { rv = f->fv(sc,param,params); if (!rv) be_cthrow(be_cast); }
						else be_throw(be_syntax);
						break;
					}
				}
				bx_freeBvarls(param, params);
				free(params);
			}
		}
		return rv;
	} else {
		return 0;
	}
}

b_varl * bx_callUfunc(b_scope * osc, b_ufunc * uf, char * o, int autoVars)
{
	b_varl * rv = 0;
	if (uf) {
		int param = 0;
		b_varl * params[4] = {0,0,0,0};
		if (bx_evalPlist(osc, bx_ffToFirstStatement(o), autoVars, &param, params, 4)) {
			if (param != uf->paramCount) {
				be_throw(be_syntax);
			} else {
				b_scope * nsc = bv_newScopeChild(osc);
				int ii;
				bf_dims(nsc, uf->paramList);
				for (ii=0; ii<param; ii++) {
					bv_sets(nsc, uf->paramNames[ii], params[ii]);
				}
				if (!be_checkerr()) {
					rv = bx_exec(nsc,uf->code);
				}
				bv_freeScope(nsc,0);
			}
		}
		bx_freeBvarls(param, params);
		return rv;
	} else {
		return 0;
	}
}

/* * * * * * * * * * * * * * */
/*   EXPRESSION EVALUATION   */
/* * * * * * * * * * * * * * */
/* Find Operator Standard Arguments for Binary Operators */
#define FOSABO "+-*/\\%^<>=&|~","*/\\%^<>=&|","NOT ",0

static char * bx_findOperator(
	char * buf, char * ch1, char * ch2, char * ch3, char * ch4, int caseSens, int rev,
	char * dontFollow, char * dontPrecede, char * dontFollowWord, char * dontPrecedeWord
) {
	int quote = 0;
	int level = 0;
	int n1 = ch1?strlen(ch1):0;
	int n2 = ch2?strlen(ch2):0;
	int n3 = ch3?strlen(ch3):0;
	int n4 = ch4?strlen(ch4):0;
	int dfwn = dontFollowWord?strlen(dontFollowWord):0;
	int dpwn = dontPrecedeWord?strlen(dontPrecedeWord):0;
	char * s;
	if (!buf) return 0;
	s = rev?(buf + strlen(buf) - 1):buf;
	while (s && (*s) && s>=buf) {
		if (quote) {
			if (rev) {
				if (bx_quote(*s)) {
					if (s==buf || (*(s-1))!='\\') quote = 0;
				}
			} else {
				if ((*s) == '\\') s++;
				else if (bx_quote(*s)) quote = 0;
			}
		} else if (bx_quote(*s)) {
			quote = 1;
		} else if (bx_levelInc(*s)) {
			level++;
		} else if (bx_levelDec(*s)) {
			level--;
		} else if (
			/* Is this hideous or what? :D */
			   (!level) /* not inside parentheses */
			&& (s>buf)  /* not at the very beginning of the string */
			&& (     !dontFollow ||     s == buf || !strchr(dontFollow, *(s-1))                ) /* isn't following a char in dontFollow */
			&& ( !dontFollowWord || (s-dfwn)<buf || strncasecmp(s-dfwn, dontFollowWord, dfwn)  ) /* isn't following the string in dontFollowWord */
			&&
			(caseSens?(
				(ch1 && !strncasecmp(s,ch1,n1) && (!dontPrecede || !strchr(dontPrecede, *(s+n1))) && (!dontPrecedeWord || strncasecmp(s+n1, dontPrecedeWord, dpwn))) ||
				(ch2 && !strncasecmp(s,ch2,n2) && (!dontPrecede || !strchr(dontPrecede, *(s+n2))) && (!dontPrecedeWord || strncasecmp(s+n2, dontPrecedeWord, dpwn))) ||
				(ch3 && !strncasecmp(s,ch3,n3) && (!dontPrecede || !strchr(dontPrecede, *(s+n3))) && (!dontPrecedeWord || strncasecmp(s+n3, dontPrecedeWord, dpwn))) ||
				(ch4 && !strncasecmp(s,ch4,n4) && (!dontPrecede || !strchr(dontPrecede, *(s+n4))) && (!dontPrecedeWord || strncasecmp(s+n4, dontPrecedeWord, dpwn)))
			):(
				(ch1 && !strncmp(s,ch1,n1) && (!dontPrecede || !strchr(dontPrecede, *(s+n1))) && (!dontPrecedeWord || strncasecmp(s+n1, dontPrecedeWord, dpwn))) ||
				(ch2 && !strncmp(s,ch2,n2) && (!dontPrecede || !strchr(dontPrecede, *(s+n2))) && (!dontPrecedeWord || strncasecmp(s+n2, dontPrecedeWord, dpwn))) ||
				(ch3 && !strncmp(s,ch3,n3) && (!dontPrecede || !strchr(dontPrecede, *(s+n3))) && (!dontPrecedeWord || strncasecmp(s+n3, dontPrecedeWord, dpwn))) ||
				(ch4 && !strncmp(s,ch4,n4) && (!dontPrecede || !strchr(dontPrecede, *(s+n4))) && (!dontPrecedeWord || strncasecmp(s+n4, dontPrecedeWord, dpwn)))
			)) /* is one of the four operators AND doesn't precede a char in dontPrecede or the string in dontPrecedeWord */
		) {
			return s;
		}
		if (rev) { s--; } else { s++; }
	}
	return 0;
}

b_varl * bx_eval(b_scope * sc, char * buff, int autoVars)
{
	char * o;
	char * buf = trim(buff);
	b_varl * l;
	b_varl * r;
	b_varl * rv;
	int tmp, tmq; double tmd;
	if (!(*buf)) {
		free(buf);
		return 0;
	} else if (( o = bx_findOperator(buf, "||", " OR ", 0, 0, 1, 0, FOSABO) )) {
		if ((*o) == '|') {
			(*o) = 0;
			o += 2;
		} else {
			(*o) = 0;
			o += 4;
		}
		l = bx_eval(sc, buf, autoVars);
		if (bv_getBoolean(l)) {
			rv = bv_intToBv(1);
		} else {
			r = bx_eval(sc, o, autoVars);
			rv = bv_intToBv(!!bv_getBoolean(r));
			bv_cfreeBv(r);
		}
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, " XOR ", 0, 0, 0, 1, 0, FOSABO) )) {
		(*o) = 0;
		o += 5;
		l = bx_eval(sc, buf, autoVars);
		r = bx_eval(sc, o, autoVars);
		rv = bv_intToBv(!bv_getBoolean(l) != !bv_getBoolean(r));
		bv_cfreeBv(r);
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, "&&", " AND ", 0, 0, 1, 0, FOSABO) )) {
		if ((*o) == '&') {
			(*o) = 0;
			o += 2;
		} else {
			(*o) = 0;
			o += 5;
		}
		l = bx_eval(sc, buf, autoVars);
		if (!bv_getBoolean(l)) {
			rv = bv_intToBv(0);
		} else {
			r = bx_eval(sc, o, autoVars);
			rv = bv_intToBv(!!bv_getBoolean(r));
			bv_cfreeBv(r);
		}
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, "|", 0, 0, 0, 0, 1, FOSABO) )) {
		(*o) = 0;
		o++;
		l = bx_eval(sc, buf, autoVars);
		r = bx_eval(sc, o, autoVars);
		rv = bv_intToBv(bv_getInt(l) | bv_getInt(r));
		bv_cfreeBv(r);
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, "^^", 0, 0, 0, 0, 1, FOSABO) )) {
		(*o) = 0;
		o += 2;
		l = bx_eval(sc, buf, autoVars);
		r = bx_eval(sc, o, autoVars);
		rv = bv_intToBv(bv_getInt(l) ^ bv_getInt(r));
		bv_cfreeBv(r);
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, "&", 0, 0, 0, 0, 1, FOSABO) )) {
		(*o) = 0;
		o++;
		l = bx_eval(sc, buf, autoVars);
		r = bx_eval(sc, o, autoVars);
		rv = bv_intToBv(bv_getInt(l) & bv_getInt(r));
		bv_cfreeBv(r);
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, "==", "!=", "<>", "=", 0, 0, FOSABO) )) {
		if ((*o) == '=') {
			tmp = 0;
		} else {
			tmp = 1;
		}
		if ((*o) == '=' && !((*(o+1)) == '=')) {
			(*o) = 0;
			o++;
		} else {
			(*o) = 0;
			o += 2;
		}
		l = bx_eval(sc, buf, autoVars);
		r = bx_eval(sc, o, autoVars);
		if (!l && !r) {
			tmp = !tmp;
		} else if (!l || !r) {
			tmp = tmp;
		} else if (l->type == bt_object && r->type == bt_object) {
			tmp = tmp?(l->oval != r->oval):(l->oval == r->oval);
		} else if (l->type == bt_object || r->type == bt_object) {
			tmp = tmp;
		} else if (l->type == bt_string || r->type == bt_string) {
			tmp = tmp?(strcmp(bv_getString(l),bv_getString(r))):(!strcmp(bv_getString(l),bv_getString(r)));
		} else if (l->type == bt_float || r->type == bt_float) {
			tmp = tmp?(bv_getDouble(l) != bv_getDouble(r)):(bv_getDouble(l) == bv_getDouble(r));
		} else {
			tmp = tmp?(bv_getInt(l) != bv_getInt(r)):(bv_getInt(l) == bv_getInt(r));
		}
		rv = bv_intToBv(tmp);
		bv_cfreeBv(r);
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, "<=", ">=", "<", ">", 0, 0, FOSABO) )) {
		if ((*(o+1)) == '=') {
			tmp = 2;
		} else {
			tmp = 0;
		}
		if ((*o) == '>') {
			tmp++;
		}
		if ((*(o+1)) == '=') {
			(*o) = 0;
			o += 2;
		} else {
			(*o) = 0;
			o++;
		}
		l = bx_eval(sc, buf, autoVars);
		r = bx_eval(sc, o, autoVars);
		if (!l && !r) {
			tmp = (tmp >= 2);
		} else if (!l || !r) {
			tmp = (tmp < 2);
		} else if (l->type == bt_object && r->type == bt_object) {
			tmp = (tmp >= 2)?(l->oval == r->oval):0;
		} else if (l->type == bt_object || r->type == bt_object) {
			tmp = (tmp < 2);
		} else if (l->type == bt_string || r->type == bt_string) {
			tmq = strcasecmp(bv_getString(l),bv_getString(r));
			if (tmp >= 2 && !tmq) {
				tmp = 1;
			} else if (tmp & 1) {
				tmp = (tmq > 0);
			} else {
				tmp = (tmq < 0);
			}
		} else if (l->type == bt_float || r->type == bt_float) {
			tmd = bv_getDouble(l) - bv_getDouble(r);
			if (tmp >= 2 && (tmd == 0)) {
				tmp = 1;
			} else if (tmp & 1) {
				tmp = (tmd > 0);
			} else {
				tmp = (tmd < 0);
			}
		} else {
			tmq = bv_getInt(l) - bv_getInt(r);
			if (tmp >= 2 && !tmq) {
				tmp = 1;
			} else if (tmp & 1) {
				tmp = (tmq > 0);
			} else {
				tmp = (tmq < 0);
			}
		}
		rv = bv_intToBv(tmp);
		bv_cfreeBv(r);
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, "<<", ">>", 0, 0, 0, 1, FOSABO) )) {
		tmp = (*o);
		(*o) = 0;
		o += 2;
		l = bx_eval(sc, buf, autoVars);
		r = bx_eval(sc, o, autoVars);
		rv = (tmp == '<')?bv_intToBv(bv_getInt(l) << bv_getInt(r)):bv_intToBv(bv_getInt(l) >> bv_getInt(r));
		bv_cfreeBv(r);
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, "+", "-", 0, 0, 0, 1, FOSABO) )) {
		tmp = (*o);
		(*o) = 0;
		o++;
		l = bx_eval(sc, buf, autoVars);
		r = bx_eval(sc, o, autoVars);
		if (tmp == '+') {
			if (!l || !r || l->type == bt_object || r->type == bt_object) {
				be_cthrow(be_cast);
				rv = 0;
			} else if (l->type == bt_string || r->type == bt_string) {
				char * ls = bv_getString(l);
				char * rs = bv_getString(r);
				int ln = strlen(ls);
				int rn = strlen(rs);
				char * b = (char *)malloc(ln + rn + 1);
				if (!b) { be_throw(be_memory); rv = 0; } else {
					(*b) = 0;
					strcat(b,ls);
					strcat(b,rs);
					rv = bv_stringToBv(b);
					free(b);
				}
			} else if (l->type == bt_float || r->type == bt_float) {
				rv = bv_doubleToBv(bv_getDouble(l) + bv_getDouble(r));
			} else {
				rv = bv_intToBv(bv_getInt(l) + bv_getInt(r));
			}
		} else {
			if (!l || !r || l->type == bt_object || r->type == bt_object || l->type == bt_string || r->type == bt_string) {
				be_cthrow(be_cast);
				rv = 0;
			} else if (l->type == bt_float || r->type == bt_float) {
				rv = bv_doubleToBv(bv_getDouble(l) - bv_getDouble(r));
			} else {
				rv = bv_intToBv(bv_getInt(l) - bv_getInt(r));
			}
		}
		bv_cfreeBv(r);
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, "*", "/", 0, 0, 0, 1, FOSABO) )) {
		tmp = (*o);
		(*o) = 0;
		o++;
		l = bx_eval(sc, buf, autoVars);
		r = bx_eval(sc, o, autoVars);
		if (tmp == '*') {
			if (!l || !r || l->type == bt_object || r->type == bt_object || (l->type == bt_string && r->type == bt_string)) {
				be_cthrow(be_cast);
				rv = 0;
			} else if (l->type == bt_string) {
				char * ls = bv_getString(l);
				int ln = strlen(ls);
				int n = bv_getInt(r);
				if (n<=0) { rv = bv_stringToBv(""); } else {
					char * b = (char *)malloc(ln * n + 1);
					if (!b) { be_throw(be_memory); rv = 0; } else {
						(*b) = 0;
						while (n) {
							strcat(b,ls);
							n--;
						}
						rv = bv_stringToBv(b);
						free(b);
					}
				}
			} else if (r->type == bt_string) {
				char * ls = bv_getString(r);
				int ln = strlen(ls);
				int n = bv_getInt(l);
				if (n<=0) { rv = bv_stringToBv(""); } else {
					char * b = (char *)malloc(ln * n + 1);
					if (!b) { be_throw(be_memory); rv = 0; } else {
						(*b) = 0;
						while (n) {
							strcat(b,ls);
							n--;
						}
						rv = bv_stringToBv(b);
						free(b);
					}
				}
			} else if (l->type == bt_float || r->type == bt_float) {
				rv = bv_doubleToBv(bv_getDouble(l) * bv_getDouble(r));
			} else {
				rv = bv_intToBv(bv_getInt(l) * bv_getInt(r));
			}
		} else {
			if (!l || !r || l->type == bt_object || r->type == bt_object || l->type == bt_string || r->type == bt_string) {
				be_cthrow(be_cast);
				rv = 0;
			} else if (l->type == bt_float || r->type == bt_float) {
				rv = bv_doubleToBv(bv_getDouble(l) / bv_getDouble(r));
			} else {
				rv = bv_intToBv(bv_getInt(l) / bv_getInt(r));
			}
		}
		bv_cfreeBv(r);
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, "\\", "%", " DIV ", " MOD ", 1, 1, FOSABO) )) {
		if ((*o) != ' ') {
			tmp = (*o);
			(*o) = 0;
			o++;
		} else {
			tmp = (*(o+1));
			(*o) = 0;
			o += 5;
		}
		l = bx_eval(sc, buf, autoVars);
		r = bx_eval(sc, o, autoVars);
		if (tmp == '\\' || tmp == 'D' || tmp == 'd') {
			if (!l || !r || l->type == bt_object || r->type == bt_object || l->type == bt_string || r->type == bt_string) {
				be_cthrow(be_cast);
				rv = 0;
			} else if (l->type == bt_float || r->type == bt_float) {
				rv = bv_doubleToBv(fdiv(bv_getDouble(l),bv_getDouble(r)));
			} else {
				rv = bv_intToBv(bv_getInt(l) / bv_getInt(r));
			}
		} else {
			if (!l || !r || l->type == bt_object || r->type == bt_object || l->type == bt_string || r->type == bt_string) {
				be_cthrow(be_cast);
				rv = 0;
			} else if (l->type == bt_float || r->type == bt_float) {
				rv = bv_doubleToBv(fmod(bv_getDouble(l),bv_getDouble(r)));
			} else {
				rv = bv_intToBv(bv_getInt(l) % bv_getInt(r));
			}
		}
		bv_cfreeBv(r);
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (( o = bx_findOperator(buf, " ISA ", " INSTANCEOF ", 0, 0, 1, 1, FOSABO) )) {
		if ((*(o+2)) == 's' || (*(o+2)) == 'S') {
			(*o) = 0;
			o += 5;
		} else {
			(*o) = 0;
			o += 12;
		}
		o = trim(o);
		tmp = 0;
		l = bx_eval(sc, buf, autoVars);
		if (l) {
			b_type t = bt_float;
			b_scope * cl = 0;
			be_cthrow(bv_resolveType(sc, o, &t, &cl));
			if (!be_checkerr()) {
				tmp = (l->type == t);
				if (tmp && t == bt_object && cl) {
					/* make sure l's class or one of its superclasses is the requested class */
					if (!l->oval) {
						tmp = 0;
					} else {
						b_scope * ocl = l->oval->classScope;
						if (!bv_scopeIsChild(ocl,cl)) tmp=0;
					}
				}
			}
		}
		rv = bv_intToBv(tmp);
		free(o);
		free(buf);
		return rv;
	} else if (!strncasecmp(buf, "NOT ", 4)) {
		l = bx_eval(sc, buf+4, autoVars);
		rv = bv_intToBv(!bv_getBoolean(l));
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if ((*buf) == '!') {
		l = bx_eval(sc, buf+1, autoVars);
		rv = bv_intToBv(!bv_getBoolean(l));
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if ((*buf) == '~') {
		l = bx_eval(sc, buf+1, autoVars);
		rv = bv_intToBv(~bv_getInt(l));
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if ((*buf) == '-') {
		l = bx_eval(sc, buf+1, autoVars);
		if (!l || l->type == bt_object || l->type == bt_string) {
			be_cthrow(be_cast);
			rv = 0;
		} else if (l->type == bt_float) {
			rv = bv_doubleToBv(-bv_getDouble(l));
		} else {
			rv = bv_intToBv(-bv_getInt(l));
		}
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if ((*buf) == '+') {
		return bx_eval(sc, buf+1, autoVars);
	} else if ((*buf) == '*') {
		l = bx_eval(sc, buf+1, autoVars);
		rv = (b_varl *)bv_getInt(l);
		bv_cfreeBv(l);
		return rv;
	} else if ((*buf) == '&') {
		return bv_intToBv((int)bx_eval(sc, buf+1, autoVars));
	} else if (*(buf + strlen(buf) - 1) == '!') {
		(*(buf + strlen(buf) - 1)) = 0;
		l = bx_eval(sc, buf, autoVars);
		rv = bv_doubleToBv(factorial(bv_getInt(l)));
		bv_cfreeBv(l);
		return rv;
	} else if (( o = bx_findOperator(buf, "^", "**", 0, 0, 0, 0, FOSABO) )) {
		if ((*o) == '^') {
			(*o) = 0;
			o++;
		} else {
			(*o) = 0;
			o += 2;
		}
		l = bx_eval(sc, buf, autoVars);
		r = bx_eval(sc, o, autoVars);
		if (!l || !r || l->type == bt_object || r->type == bt_object || l->type == bt_string || r->type == bt_string) {
			be_cthrow(be_cast);
			rv = 0;
		} else {
			rv = bv_doubleToBv(pow(bv_getDouble(l),bv_getDouble(r)));
		}
		bv_cfreeBv(r);
		bv_cfreeBv(l);
		free(buf);
		return rv;
	} else if (*(buf + strlen(buf) - 1) == ')') {
		char * fn;
		(*(buf + strlen(buf) - 1)) = 0;
		o = strchr(buf, '(');
		if (!o) {
			be_throw(be_syntax);
			free(buf);
			return 0;
		}
		(*o) = 0;
		o++;
		fn = trim(buf);
		if ((*fn) == 0) {
			rv = bx_eval(sc,o,autoVars);
		} else {
			/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
			/*        This is the part where functions are called!       */
			/* Since the code is so tiny it is very easy to skip over it */
			/*             while scrolling through this file!            */
			/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
			rv = 0;
			b_function * f;
			b_ufunc * uf;
			if (( f = bf_FindFunction(fn) )) {
				rv = bx_callFunction(sc,f,o,autoVars);
			} else if (( uf = bv_finduf(sc,fn) )) {
				rv = bx_callUfunc(bv_finduscope(sc,fn),uf,o,autoVars);
			} else {
				be_cthrow(be_syntax);
			}
		}
		free(fn);
		free(buf);
		return rv;
	} else if (*(buf + strlen(buf) - 1) == ']') {
		char * fn;
		(*(buf + strlen(buf) - 1)) = 0;
		o = strchr(buf, '[');
		if (!o) {
			be_throw(be_syntax);
			free(buf);
			return 0;
		}
		(*o) = 0;
		o++;
		fn = trim(buf);
		if ((*fn) == 0) {
			rv = bx_eval(sc,o,autoVars);
		} else {
			/* array reference */
			b_array * a = bv_finda(sc, fn);
			rv = 0;
			if (a) {
				int param = 0;
				char * p;
				char * q;
				char * tmp;
				b_varl * tmpv = 0;
				int params[3] = {0,0,0};
				int dims[3] = {a->dim1, a->dim2, a->dim3};
				p = bx_ffToFirstStatement(o);
				while (!bx_statementDelim(*p)) {
					q = bx_ffToEndOfParameter(p);
					tmp = substr(p,q);
					tmpv = bx_eval(sc,tmp,autoVars);
					if (!tmpv) {
						be_cthrow(be_undeclared);
						free(tmp);
						break;
					} else if (!bv_assertNumeric(tmpv)) {
						be_cthrow(be_cast);
						bv_cfreeBv(tmpv);
						free(tmp);
						break;
					} else {
						if (param >= 3) {
							be_cthrow(be_subscript);
							free(tmp);
							break;
						}
						params[param] = bv_getInt(tmpv);
						if (params[param] < 0 || params[param] >= dims[param]) {
							be_cthrow(be_subscript);
							bv_cfreeBv(tmpv);
							free(tmp);
							break;
						}
						param++;
						bv_cfreeBv(tmpv);
					}
					free(tmp);
					p = bx_ffToNextParameter(p);
				}
				if (!be_checkerr()) {
					if (param != a->dim) {
						be_throw(be_subscript);
					}
					rv = &a->elem[(params[0] * dims[1] * dims[2]) + (params[1] * dims[2]) + params[2]];
				}
			} else {
				be_throw(be_undeclared);
			}
		}
		free(fn);
		free(buf);
		return rv;
	} else if (bx_quote(*buf) && bx_quote(*(buf + strlen(buf) - 1))) {
		/* string */
		char * tmp = (char *)malloc(strlen(buf));
		int p = 0;
		int i;
		(*tmp) = 0;
		for (i=1; i<strlen(buf)-1; i++) {
			if (buf[i] == '\\') {
				i++;
				switch (buf[i]) {
					case '\'': tmp[p] = '\''; p++; tmp[p] = 0; break;
					case '\"': tmp[p] = '\"'; p++; tmp[p] = 0; break;
					case '\\': tmp[p] = '\\'; p++; tmp[p] = 0; break;
					case 'n': tmp[p] = '\n'; p++; tmp[p] = 0; break;
					case 't': tmp[p] = '\t'; p++; tmp[p] = 0; break;
					case 'r': tmp[p] = '\r'; p++; tmp[p] = 0; break;
					case 'f': tmp[p] = '\f'; p++; tmp[p] = 0; break;
					case 'v': tmp[p] = '\v'; p++; tmp[p] = 0; break;
					case 'a': tmp[p] = '\a'; p++; tmp[p] = 0; break;
					case 'b': tmp[p] = '\b'; p++; tmp[p] = 0; break;
					case ' ': tmp[p] = ' '; p++; tmp[p] = 0; break;
					case 'e': tmp[p] =  27; p++; tmp[p] = 0; break;
					case 'd': tmp[p] = 127; p++; tmp[p] = 0; break;
					default: tmp[p] = buf[i]; p++; tmp[p] = 0; break;
				}
			} else {
				tmp[p] = buf[i]; p++;
				tmp[p] = 0;
			}
		}
		rv = bv_stringToBv(tmp);
		free(tmp);
		free(buf);
		return rv;
	} else if (bx_verifyIdent(buf)) {
		/* variable */
		b_varl * retVal = bv_find(sc, buf);
		if (autoVars && !retVal) {
			b_type t = bx_autoTypeIdent(buf);
			switch (t) {
				case bt_integer: bv_add(sc, buf, bv_intToBv(0)); break;
				case bt_float: bv_add(sc, buf, bv_doubleToBv(0.0)); break;
				case bt_string: bv_add(sc, buf, bv_stringToBv("")); break;
				case bt_object: bv_add(sc, buf, bv_objectToBv(NULL,0)); break;
				case bt_exception: bv_add(sc, buf, bv_exceptionToBv(be_none)); break;
			}
			be_clearerr();
			retVal = bv_find(sc, buf);
		}
		free(buf);
		return retVal;
	} else if (bx_isBinary(buf)) {
		unsigned long int i = 0;
		char * b = buf + 2;
		while (*b) {
			i <<= 1;
			i += (*b - '0');
			b++;
		}
		free(buf);
		return bv_intToBv(i);
	} else if (bx_isHex(buf)) {
		unsigned long int i = 0;
		char * b = buf + 2;
		while (*b) {
			i <<= 4;
			if (islower(*b)) i += (*b - 'a' + 10);
			if (isupper(*b)) i += (*b - 'A' + 10);
			if (isdigit(*b)) i += (*b - '0');
			b++;
		}
		free(buf);
		return bv_intToBv(i);
	} else if (bx_isOctal(buf)) {
		unsigned long int i = 0;
		char * b = buf;
		while (*b) {
			i <<= 3;
			i += (*b - '0');
			b++;
		}
		free(buf);
		return bv_intToBv(i);
	} else if (bx_isInt(buf)) {
		/* integer */
		unsigned long int i;
		sscanf(buf, "%ld", &i);
		free(buf);
		return bv_intToBv(i);
	} else if (bx_isFloat(buf)) {
		/* float */
		double d;
		sscanf(buf, "%lf", &d);
		free(buf);
		return bv_doubleToBv(d);
	} else {
		be_throw(be_syntax);
		return 0;
	}
}

/* * * * * * * * * * * * */
/*   PROGRAM EXECUTION   */
/* * * * * * * * * * * * */
static char * bx_findLabel(char * o, char * l)
{
	char * nbuf = bx_ffToFirstStatement(o);
	if (bx_isInt(l)) {
		while (nbuf && (*nbuf)) {
			if (isdigit(*nbuf)) {
				char * nls = nbuf;
				char * nle = nbuf;
				char * nl;
				while (isdigit(*nle)) nle++;
				nl = substrim(nls,nle);
				if (!strcmp(nl,l)) {
					free(nl);
					return nbuf;
				}
				free(nl);
			}
			nbuf = bx_ffToNextStatement(nbuf);
		}
	} else {
		while (nbuf && (*nbuf)) {
			char * np;
			char * ncmd;
			if (isdigit(*nbuf)) {
				/* Line number! Awesome! ^^ */
				while (isdigit(*nbuf)) nbuf++;
				nbuf = bx_ffToFirstStatement(nbuf);
			}
			np = bx_ffToIdentifierEnd(nbuf,1);
			ncmd = substr(nbuf,np);
			if (!strcasecmp(ncmd,"LABEL")) {
				char * nls = bx_ffToFirstParameter(np);
				char * nle = bx_ffToEndOfStatement(np);
				char * nl  = substrim(nls,nle);
				if (!strcasecmp(nl,l)) {
					/* found the label! yay! */
					free(nl);
					free(ncmd);
					return nbuf;
				}
				free(nl);
			}
			free(ncmd);
			nbuf = bx_ffToNextStatement(nbuf);
		}
	}
	return 0;
}

static char * bx_findStructKeyword(char * o, char * beginner, char * midder, char * midder2, char * ender)
{
	int levels = 0;
	char * nbuf = bx_ffToNextStatement(o);
	while (nbuf && (*nbuf)) {
		char * np;
		char * ncmd;
		if (isdigit(*nbuf)) {
			/* Line number! Awesome! ^^ */
			while (isdigit(*nbuf)) nbuf++;
			nbuf = bx_ffToFirstStatement(nbuf);
		}
		np = bx_ffToIdentifierEnd(nbuf,1);
		ncmd = substr(nbuf,np);
		if (beginner && !strcasecmp(ncmd,beginner)) {
			levels++;
		} else if (!strcasecmp(ncmd,"END")) {
			char * nls = bx_ffToFirstParameter(np);
			char * nle = bx_ffToEndOfStatement(np);
			char * nl  = substrim(nls,nle);
			if (beginner && !strcasecmp(nl,beginner)) {
				levels--;
			}
			free(nl);
		} else if (ender && !strcasecmp(ncmd,ender)) {
			levels--;
		} else if (midder && (levels==0) && !strcasecmp(ncmd,midder)) {
			levels--;
		} else if (midder2 && (levels==0) && !strcasecmp(ncmd,midder2)) {
			levels--;
		}
		free(ncmd);
		if (levels<0) return nbuf;
		nbuf = bx_ffToNextStatement(nbuf);
	}
	return 0;
}

static b_loopstackframe * bx_new_for_loop(b_loopstackframe * parent, char * start, b_varl * loopvar, double begin, double end, double step)
{
	b_loopstackframe * fr = (b_loopstackframe *)malloc(sizeof(b_loopstackframe));
	if (!fr) return 0;
	fr->parent = parent;
	fr->type = bl_for;
	fr->start = start;
	fr->conditionExpr = 0;
	fr->loopvar = loopvar;
	fr->begin = begin;
	fr->end = end;
	fr->step = step;
	fr->looparr = 0;
	return fr;
}

static b_loopstackframe * bx_new_foreach_loop(b_loopstackframe * parent, char * start, b_varl * loopvar, b_array * looparr)
{
	b_loopstackframe * fr = (b_loopstackframe *)malloc(sizeof(b_loopstackframe));
	if (!fr) return 0;
	fr->parent = parent;
	fr->type = bl_foreach;
	fr->start = start;
	fr->conditionExpr = 0;
	fr->loopvar = loopvar;
	fr->begin = 0;
	fr->end = looparr->dim1 * looparr->dim2 * looparr->dim3 - 1;
	fr->step = 1;
	fr->looparr = looparr;
	return fr;
}

static b_loopstackframe * bx_new_while_loop(b_loopstackframe * parent, char * start, char * condition)
{
	b_loopstackframe * fr = (b_loopstackframe *)malloc(sizeof(b_loopstackframe));
	if (!fr) return 0;
	fr->parent = parent;
	fr->type = bl_while;
	fr->start = start;
	fr->conditionExpr = strdup(condition);
	fr->loopvar = 0;
	fr->begin = 0;
	fr->end = 0;
	fr->step = 0;
	fr->looparr = 0;
	return fr;
}

static b_loopstackframe * bx_new_do_loop(b_loopstackframe * parent, char * start)
{
	b_loopstackframe * fr = (b_loopstackframe *)malloc(sizeof(b_loopstackframe));
	if (!fr) return 0;
	fr->parent = parent;
	fr->type = bl_doloop;
	fr->start = start;
	fr->conditionExpr = 0;
	fr->loopvar = 0;
	fr->begin = 0;
	fr->end = 0;
	fr->step = 0;
	fr->looparr = 0;
	return fr;
}

static b_loopstackframe * bx_exit_loop(b_loopstackframe * fr)
{
	b_loopstackframe * parent;
	if (!fr) return 0;
	parent = fr->parent;
	if (fr->conditionExpr) free(fr->conditionExpr);
	if (fr->loopvar) bv_cfreeBv(fr->loopvar);
	free(fr);
	return parent;
}

static b_loopstackframe * bx_exit_all(b_loopstackframe * fr)
{
	while (fr) {
		fr = bx_exit_loop(fr);
	}
	return 0;
}

static b_trystackframe * bx_new_try(b_trystackframe * parent, b_varl * catchVar, char * catchLine)
{
	b_trystackframe * fr = (b_trystackframe *)malloc(sizeof(b_trystackframe));
	if (!fr) return 0;
	fr->parent = parent;
	fr->catchVar = catchVar;
	fr->catchLine = catchLine;
	return fr;
}

static b_trystackframe * bx_return_from_try(b_trystackframe * fr)
{
	b_trystackframe * parent;
	if (!fr) return 0;
	parent = fr->parent;
	free(fr);
	return parent;
}

static b_trystackframe * bx_return_from_alltries(b_trystackframe * fr)
{
	while (fr) {
		fr = bx_return_from_try(fr);
	}
	return 0;
}

static b_substackframe * bx_new_subcall(b_substackframe * parent, char * start, b_loopstackframe * lsf, b_trystackframe * tsf)
{
	b_substackframe * fr = (b_substackframe *)malloc(sizeof(b_substackframe));
	if (!fr) return 0;
	fr->parent = parent;
	fr->calledFrom = start;
	fr->lsf = lsf;
	fr->tsf = tsf;
	return fr;
}

static b_substackframe * bx_return_from_subcall(b_substackframe * fr)
{
	b_substackframe * parent;
	if (!fr) return 0;
	parent = fr->parent;
	free(fr);
	return parent;
}

static b_substackframe * bx_return_from_all(b_substackframe * fr)
{
	while (fr) {
		if (fr->lsf) bx_exit_all(fr->lsf);
		if (fr->tsf) bx_return_from_alltries(fr->tsf);
		fr = bx_return_from_subcall(fr);
	}
	return 0;
}

static int bx_parsePlist(b_scope * s, char * c, int * pcount, b_type * ptypes, char ** pnames, b_scope ** pclasses, int pmax)
{
	char * pp = bx_ffToFirstStatement(c);
	char * qq;
	while (!bx_statementDelim(*pp) && !be_checkerr()) {
		qq = bx_ffToEndOfParameter(pp);
		char * line = substrim(pp,qq);
		char * ts = 0;
		b_type t = bx_autoTypeIdent(line);
		b_scope * cl = 0;
		char * p = strcasestr(line, " AS ");
		if (p) {
			char * tmp;
			ts = trim(p+4);
			be_cthrow(bv_resolveType(s, ts, &t, &cl));
			tmp = substrim(line,p);
			free(line);
			line = tmp;
		}
		if (be_checkerr()) {
			if (ts) free(ts);
			free(line);
			break;
		}
		p = bx_ffToIdentifierEnd(line, 1);
		if (*p) {
			be_throw(be_syntax);
		} else {
			if (!bx_verifyIdent(line)) { be_throw(be_syntax); } else {
				ptypes[*pcount] = t;
				pnames[*pcount] = strdup(line);
				pclasses[*pcount] = cl;
				(*pcount)++;
			}
		}
		if (ts) free(ts);
		free(line);
		pp = bx_ffToNextParameter(pp);
	}
	return !be_checkerr();
}

b_varl * bx_exec(b_scope * sc, char * buf)
{
	char * o = buf;
	char * onerr = 0;
	/* int onerrcalled = 0; */ /* commented iff error handling option 2 is commented */
	char * tbuf = 0;
	char * ttbuf = 0;
	int repeatLine = 0;
	char * dataPtr = 0;
	b_loopstackframe * loopstack = 0;
	b_trystackframe * trystack = 0;
	b_substackframe * substack = 0;
	if (!(buf && (*buf))) return 0;
	buf = bx_ffToFirstStatement(buf);
	while ((buf && (*buf)) || (tbuf && (*tbuf))) {
		char * p = 0;
		char * q = 0;
		char * cmd = 0;
		if (tbuf) {
			ttbuf = buf;
			buf = tbuf;
		}
		if (isdigit(*buf)) {
			/* Line number! Awesome! ^^ */
			while (isdigit(*buf)) buf++;
			buf = bx_ffToFirstStatement(buf);
		}
		if ( (*buf)=='\'' || (*buf)=='/' || (*buf)=='#' ) {
			/* hit a comment */
			buf = bx_ffToEndOfLine(buf);
			goto commandFinished; /* goto is evil! but sometimes evils are necessary. */
		} else if (bx_identStart(*buf)) {
			p = bx_ffToIdentifierEnd(buf,1);
			cmd = substr(buf,p);
		} else {
			be_throw(be_syntax);
			goto commandFinished; /* except for george bush. */
		}
		if (p == buf) {
			/* blank line */
		} else if (!strcasecmp(cmd,"REM")) {
			/* hit a comment */
			buf = bx_ffToEndOfLine(buf);
		} else if ( (*(bx_ffToFirstStatement(p)))=='=' ) {
			/* implicit let */
			char * stuff;
			q = bx_ffToEndOfStatement(p);
			stuff = substr(buf,q);
			bf_lets(sc, stuff);
			free(stuff);
		} else if (!strcasecmp(cmd,"ONERR")) {
			q = bx_ffToEndOfStatement(p);
			if (onerr) free(onerr);
			onerr = substr(p,q);
		} else if (!strcasecmp(cmd,"END")) {
			char * stuff;
			q = bx_ffToEndOfStatement(p);
			stuff = substrim(p,q);
			if (!(*stuff)) {
				be_close();
			} else if (!strcasecmp(stuff,"IF")) {
				/* end if - ignore */
			} else if (!strcasecmp(stuff,"SELECT") || !strcasecmp(stuff,"SWITCH")) {
				/* end select/switch - ignore */
			} else if (!strcasecmp(stuff,"TRY")) {
				if (!trystack) {
					be_throw(be_endtry);
				} else {
					trystack = bx_return_from_try(trystack);
				}
			} else if (!strcasecmp(stuff,"FOR")) {
				if (!loopstack || loopstack->type != bl_for) {
					be_throw(be_next);
				} else {
					double nv = bv_getDouble(loopstack->loopvar);
					nv += loopstack->step;
					if (nv > loopstack->end) {
						loopstack = bx_exit_loop(loopstack);
					} else {
						bv_setDouble(loopstack->loopvar, nv);
						buf = loopstack->start;
					}
				}
			} else if (!strcasecmp(stuff,"FOREACH")) {
				if (!loopstack || loopstack->type != bl_foreach) {
					be_throw(be_next);
				} else {
					loopstack->begin++;
					if (loopstack->begin > loopstack->end) {
						loopstack = bx_exit_loop(loopstack);
					} else {
						bv_setsp(loopstack->loopvar, &loopstack->looparr->elem[(int)loopstack->begin]);
						buf = loopstack->start;
					}
				}
			} else if (!strcasecmp(stuff,"WHILE")) {
				if (!loopstack || loopstack->type != bl_while) {
					be_throw(be_wend);
				} else {
					b_varl * v;
					v = bx_eval(sc, loopstack->conditionExpr, 0);
					if (bv_getBoolean(v)) {
						buf = loopstack->start;
					} else {
						loopstack = bx_exit_loop(loopstack);
					}
					bv_cfreeBv(v);
				}
			} else {
				be_throw(be_syntax);
			}
			free(stuff);
		} else if (!strcasecmp(cmd,"RETURN")) {
			if (!substack) {
				char * stuff;
				b_varl * rv;
				q = bx_ffToEndOfStatement(p);
				stuff = substr(p,q);
				rv = bx_eval(sc, stuff, 0);
				free(stuff);
				if (onerr) { free(onerr); onerr = 0; }
				bx_exit_all(loopstack);
				bx_return_from_alltries(trystack);
				return rv;
			} else {
				bx_exit_all(loopstack);
				bx_return_from_alltries(trystack);
				loopstack = substack->lsf;
				trystack = substack->tsf;
				buf = substack->calledFrom;
				if (tbuf) free(tbuf);
				tbuf = 0;
				ttbuf = 0;
				substack = bx_return_from_subcall(substack);
			}
		} else if (!strcasecmp(cmd,"LABEL")) {
			/* hit a label */
			char * ls = bx_ffToFirstParameter(p);
			char * le = bx_ffToEndOfStatement(p);
			char * l  = substrim(ls,le);
			if (!bx_verifyIdent(l)) {
				be_throw(be_syntax);
			}
			free(l);
		} else if (!strcasecmp(cmd,"GOTO")) {
			char * ls = bx_ffToFirstParameter(p);
			char * le = bx_ffToEndOfStatement(p);
			char * l  = substrim(ls,le);
			if (!( bx_verifyIdent(l) || bx_isInt(l) )) {
				be_throw(be_syntax);
			} else {
				char * n = bx_findLabel(o,l);
				if (n) {
					buf = n;
					if (tbuf) free(tbuf);
					tbuf = 0;
					ttbuf = 0;
					repeatLine = 1;
				} else {
					be_throw(be_label);
				}
			}
			free(l);
		} else if (!strcasecmp(cmd,"GOSUB")) {
			char * ls = bx_ffToFirstParameter(p);
			char * le = bx_ffToEndOfStatement(p);
			char * l  = substrim(ls,le);
			b_substackframe * ssf;
			if (!( bx_verifyIdent(l) || bx_isInt(l) )) {
				be_throw(be_syntax);
			} else {
				char * n = bx_findLabel(o,l);
				if (n) {
					if ((ssf = bx_new_subcall(substack, buf, loopstack, trystack))) {
						loopstack = 0;
						trystack = 0;
						substack = ssf;
						buf = n;
						if (tbuf) free(tbuf);
						tbuf = 0;
						ttbuf = 0;
						repeatLine = 1;
					} else {
						be_throw(be_memory);
					}
				} else {
					be_throw(be_label);
				}
			}
			free(l);
		} else if (!strcasecmp(cmd,"POP")) {
			if (!substack) {
				be_throw(be_return);
			} else {
				bx_exit_all(loopstack);
				bx_return_from_alltries(trystack);
				loopstack = substack->lsf;
				trystack = substack->tsf;
				substack = bx_return_from_subcall(substack);
			}
		} else if (!strcasecmp(cmd,"ON")) {
			char * stuff;
			char * g;
			b_varl * v;
			int whichOne;
			b_substackframe * ssf;
			q = bx_ffToEndOfStatement(p);
			stuff = substrim(p,q);
			if ((g = bx_strcasestrq(stuff," GOSUB "))) {
				(*g) = 0;
				g += 7;
				v = bx_eval(sc, stuff, 0);
				if (!v) {
					be_cthrow(be_undeclared);
				} else if (be_checkerr()) {
				} else if (!bv_assertNumeric(v)) {
					be_throw(be_cast);
				} else {
					whichOne = bv_getInt(v);
					g = bx_ffToFirstStatement(g);
					while (!bx_statementDelim(*g) && !be_checkerr()) {
						if (whichOne == 1) {
							q = bx_ffToEndOfParameter(g);
							char * l = substrim(g,q);
							char * n = bx_findLabel(o,l);
							if (n) {
								if ((ssf = bx_new_subcall(substack, buf, loopstack, trystack))) {
									loopstack = 0;
									trystack = 0;
									substack = ssf;
									buf = n;
									if (tbuf) free(tbuf);
									tbuf = 0;
									ttbuf = 0;
									repeatLine = 1;
								} else {
									be_throw(be_memory);
								}
							} else {
								be_throw(be_label);
							}
							free(l);
						}
						whichOne--;
						g = bx_ffToNextParameter(g);
					}
				}
			} else if ((g = bx_strcasestrq(stuff," GOTO "))) {
				(*g) = 0;
				g += 6;
				v = bx_eval(sc, stuff, 0);
				if (!v) {
					be_cthrow(be_undeclared);
				} else if (be_checkerr()) {
				} else if (!bv_assertNumeric(v)) {
					be_throw(be_cast);
				} else {
					whichOne = bv_getInt(v);
					g = bx_ffToFirstStatement(g);
					while (!bx_statementDelim(*g) && !be_checkerr()) {
						if (whichOne == 1) {
							q = bx_ffToEndOfParameter(g);
							char * l = substrim(g,q);
							char * n = bx_findLabel(o,l);
							if (n) {
								buf = n;
								if (tbuf) free(tbuf);
								tbuf = 0;
								ttbuf = 0;
								repeatLine = 1;
							} else {
								be_throw(be_label);
							}
							free(l);
						}
						whichOne--;
						g = bx_ffToNextParameter(g);
					}
				}
			} else {
				be_throw(be_syntax);
			}
			free(stuff);
		} else if (!strcasecmp(cmd,"IF")) {
			char * stuff;
			char * th;
			b_varl * v;
			q = bx_ffToEndOfStatement(p);
			stuff = substrim(p,q);
			if ((th = bx_strcasestrq(stuff," THEN "))) {
				/* single-line if */
				/* this is broken and needs to be fixed */
				char * el;
				(*th) = 0;
				th += 6;
				el = bx_strcasestrq(th," ELSE ");
				if (el) {
					(*el) = 0;
					el += 6;
				}
				v = bx_eval(sc, stuff, 0);
				if (!be_checkerr()) {
					if (bv_getBoolean(v)) {
						tbuf = trim(th);
					} else if (el) {
						tbuf = trim(el);
					}
				}
				bv_cfreeBv(v);
			} else if ((th = bx_strcasestrq(stuff," THEN"))) {
				(*th) = 0;
				v = bx_eval(sc, stuff, 0);
				while (1) {
					if (be_checkerr() || bv_getBoolean(v)) {
						break;
					} else {
						p = bx_findStructKeyword(buf, "IF", "ELSEIF", "ELSE", "ENDIF");
						if (!p) {
							be_throw(be_noend);
							break;
						} else {
							char * stuff2;
							buf = p;
							p = bx_ffToIdentifierEnd(buf,1);
							free(cmd);
							cmd = substr(buf,p);
							q = bx_ffToEndOfStatement(p);
							stuff2 = substrim(p,q);
							if (!strcasecmp(cmd,"END")) {
								if (strcasecmp(stuff2,"IF")) be_throw(be_syntax);
								free(stuff2);
								break;
							} else if (!strcasecmp(cmd,"ENDIF")) {
								if (*stuff2) be_throw(be_syntax);
								free(stuff2);
								break;
							} else if (!strcasecmp(cmd,"ELSEIF")) {
								char * th2 = bx_strcasestrq(stuff2," THEN");
								if (!th2) {
									be_throw(be_syntax);
									free(stuff2);
									break;
								} else {
									(*th2) = 0;
									bv_cfreeBv(v);
									v = bx_eval(sc, stuff2, 0);
								}
							} else if (!strcasecmp(cmd,"ELSE")) {
								if (!(*stuff2)) {
									free(stuff2);
									break;
								} else {
									char * p2 = bx_ffToIdentifierEnd(stuff2,1);
									/* char * q2 = bx_ffToEndOfStatement(p2); */
									char * cmd2 = substr(stuff2,p2);
									char * th2;
									if (strcasecmp(cmd2,"IF")) {
										be_throw(be_syntax);
										free(cmd2);
										free(stuff2);
										break;
									}
									free(cmd2);
									th2 = bx_strcasestrq(stuff2," THEN");
									if (!th2) {
										be_throw(be_syntax);
										free(stuff2);
										break;
									} else {
										(*th2) = 0;
										bv_cfreeBv(v);
										v = bx_eval(sc, stuff2+2, 0);
									}
								}
							} else {
								be_throw(be_syntax);
								free(stuff2);
								break;
							}
							free(stuff2);
						}
					}
				}
				bv_cfreeBv(v);
			} else {
				be_throw(be_syntax);
			}
			free(stuff);
		} else if (!strcasecmp(cmd,"ELSE") || !strcasecmp(cmd,"ELSEIF")) {
			char * n = bx_findStructKeyword(buf, "IF", 0, 0, "ENDIF");
			if (!n) {
				be_throw(be_else);
			} else {
				buf = n;
			}
		} else if (!strcasecmp(cmd,"ENDIF")) {
			/* end if - ignore */
		} else if (!strcasecmp(cmd,"SELECT") || !strcasecmp(cmd,"SWITCH")) {
			char * stuff;
			b_varl * v;
			b_varl * cv;
			q = bx_ffToEndOfStatement(p);
			stuff = substrim(p,q);
			v = bx_eval(sc, stuff, 0);
			if (!v) {
				be_cthrow(be_undeclared);
			} else { while (1) {
				p = bx_findStructKeyword(buf, "SELECT", "CASE", "DEFAULT", "ENDSELECT");
				q = bx_findStructKeyword(buf, "SWITCH", "CASE", "DEFAULT", "ENDSWITCH");
				if (!p || (q && q<p)) p=q;
				if (!p) {
					be_throw(be_noend);
					break;
				} else {
					char * stuff2;
					buf = p;
					p = bx_ffToIdentifierEnd(buf,1);
					free(cmd);
					cmd = substr(buf,p);
					q = bx_ffToEndOfStatement(p);
					stuff2 = substrim(p,q);
					if (!strcasecmp(cmd,"END")) {
						if (strcasecmp(stuff2,"SELECT") && strcasecmp(stuff2,"SWITCH")) be_throw(be_syntax);
						free(stuff2);
						break;
					} else if (!strcasecmp(cmd,"ENDSELECT") || !strcasecmp(cmd,"ENDSWITCH")) {
						if (*stuff2) be_throw(be_syntax);
						free(stuff2);
						break;
					} else if (!strcasecmp(cmd,"CASE")) {
						cv = bx_eval(sc, stuff2, 0);
						if (bv_equal(v,cv)) {
							bv_cfreeBv(cv);
							free(stuff2);
							break;
						}
						bv_cfreeBv(cv);
					} else if (!strcasecmp(cmd,"DEFAULT")) {
						if (*stuff2) {
							be_throw(be_syntax);
						}
						free(stuff2);
						break;
					} else {
						be_throw(be_syntax);
						free(stuff2);
						break;
					}
					free(stuff2);
				}
			}}
			bv_cfreeBv(v);
			free(stuff);
		} else if (!strcasecmp(cmd,"CASE") || !strcasecmp(cmd,"DEFAULT")) {
			char * n = bx_findStructKeyword(buf, "SELECT", 0, 0, "ENDSELECT");
			char * m = bx_findStructKeyword(buf, "SWITCH", 0, 0, "ENDSWITCH");
			if (!n && !m) {
				be_throw(be_case);
			} else if (!m || n<m) {
				buf = n;
			} else if (!n || m<n) {
				buf = m;
			} else {
				be_throw(be_case);
			}
		} else if (!strcasecmp(cmd,"ENDSELECT") || !strcasecmp(cmd,"ENDSWITCH")) {
			/* end select/switch - ignore */
		} else if (!strcasecmp(cmd,"TRY")) {
			char * n = bx_findStructKeyword(buf, "TRY", "CATCH", 0, "ENDTRY");
			char * evn = 0;
			b_varl * ev = 0;
			b_trystackframe * tsf;
			if (!n) {
				be_throw(be_noend);
			} else if (!strncasecmp(n,"CATCH",5)) {
				char * pp = bx_ffToIdentifierEnd(n,1);
				char * qq = bx_ffToEndOfStatement(pp);
				evn = substrim(pp,qq);
				if (bx_verifyIdent(evn)) {
					ev = bv_exceptionToBv(be_none);
					bv_seth(sc,evn,ev);
					bv_cfreeBv(ev);
					ev = bv_find(sc,evn);
				} else {
					be_throw(be_syntax);
					buf = n;
				}
				free(evn);
			}
			if (!be_checkerr()) {
				if ((tsf = bx_new_try(trystack, ev, n))) {
					trystack = tsf;
				} else {
					be_throw(be_memory);
				}
			}
		} else if (!strcasecmp(cmd,"CATCH")) {
			char * n = bx_findStructKeyword(buf, "TRY", 0, 0, "ENDTRY");
			if (!n) {
				be_throw(be_onerr);
			} else {
				buf = n;
			}
			trystack = bx_return_from_try(trystack);
		} else if (!strcasecmp(cmd,"ENDTRY")) {
			if (!trystack) {
				be_throw(be_endtry);
			} else {
				trystack = bx_return_from_try(trystack);
			}
		} else if (!strcasecmp(cmd,"FOR")) {
			char * stuff;
			char * st;
			char * to;
			char * eq;
			b_varl * var;
			b_varl * ste;
			b_varl * too;
			b_varl * equ;
			double step = 1.0;
			double tooo = 0.0;
			double equa = 0.0;
			b_loopstackframe * lsf;
			q = bx_ffToEndOfStatement(p);
			stuff = substrim(p,q);
			st = bx_strcasestrq(stuff," STEP ");
			if (st) {
				(*st) = 0;
				st += 6;
				ste = bx_eval(sc, st, 0);
				step = bv_getDouble(ste);
				bv_cfreeBv(ste);
			} else {
				step = 1.0;
			}
			to = bx_strcasestrq(stuff, " TO ");
			if (to) {
				(*to) = 0;
				to += 4;
				too = bx_eval(sc, to, 0);
				tooo = bv_getDouble(too);
				bv_cfreeBv(too);
			} else {
				be_throw(be_syntax);
			}
			bf_lets(sc, stuff);
			eq = bx_strcasechrq(stuff, '=');
			if (eq) {
				(*eq) = 0;
				eq++;
				equ = bx_eval(sc, eq, 0);
				equa = bv_getDouble(equ);
				bv_cfreeBv(equ);
			} else {
				be_throw(be_syntax);
			}
			var = bx_eval(sc, stuff, 0);
			if (!be_checkerr()) {
				if (tooo < equa) {
					char * n = bx_findStructKeyword(buf, "FOR", 0, 0, "NEXT");
					if (!n) {
						be_throw(be_noend);
					} else {
						buf = n;
					}
				} else if ((lsf = bx_new_for_loop(loopstack, buf, var, equa, tooo, step))) {
					loopstack = lsf;
				} else {
					be_throw(be_memory);
				}
			}
			free(stuff);
		} else if (!strcasecmp(cmd,"FOREACH")) {
			char * stuff;
			char * vn;
			char * an;
			char * tmp;
			b_varl * var;
			b_array * arr;
			b_loopstackframe * lsf;
			q = bx_ffToEndOfStatement(p);
			stuff = substr(p,q);
			tmp = bx_strcasestrq(stuff," IN ");
			if (!tmp) {
				be_throw(be_syntax);
			} else {
				(*tmp) = 0;
				tmp += 4;
				an = trim(tmp);
				vn = trim(stuff);
				if (!(bx_verifyIdent(vn) && bx_verifyIdent(an))) {
					be_throw(be_syntax);
				} else {
					arr = bv_finda(sc, an);
					if (!arr) {
						be_throw(be_undeclared);
					} else {
						var = bv_find(sc, vn);
						if (!var) {
							be_clearerr();
							bv_add(sc, vn, &arr->elem[0]);
							var = bv_find(sc, vn);
						}
						if (!var) {
							be_throw(be_memory);
						} else {
							bv_setsp(var, &arr->elem[0]);
							if ((lsf = bx_new_foreach_loop(loopstack, buf, var, arr))) {
								loopstack = lsf;
							} else {
								be_throw(be_memory);
							}
						}
					}
				}
				free(an);
				free(vn);
			}
			free(stuff);
		} else if (!strcasecmp(cmd,"WHILE")) {
			char * stuff;
			b_varl * v;
			b_loopstackframe * lsf;
			q = bx_ffToEndOfStatement(p);
			stuff = substr(p,q);
			v = bx_eval(sc, stuff, 0);
			if (be_checkerr()) {
			} else if (!v) {
				be_throw(be_undeclared);
			} else if (!bv_getBoolean(v)) {
				char * n = bx_findStructKeyword(buf, "WHILE", 0, 0, "WEND");
				if (!n) {
					be_throw(be_noend);
				} else {
					buf = n;
				}
			} else if ((lsf = bx_new_while_loop(loopstack, buf, stuff))) {
				loopstack = lsf;
			} else {
				be_throw(be_memory);
			}
			bv_cfreeBv(v);
			free(stuff);
		} else if (!strcasecmp(cmd,"DO")) {
			char * tmp2;
			b_loopstackframe * lsf;
			q = bx_ffToEndOfStatement(p);
			tmp2 = substrim(p,q);
			if (*tmp2) {
				b_varl * v = bx_eval(sc, tmp2, 0);
				if (!bv_assertString(v)) {
					be_throw(be_cast);
				} else {
					bv_cfreeBv(bx_exec(sc, bv_getString(v)));
				}
				bv_cfreeBv(v);
			} else if ((lsf = bx_new_do_loop(loopstack, buf))) {
				loopstack = lsf;
			} else {
				be_throw(be_memory);
			}
			free(tmp2);
		} else if (!strcasecmp(cmd,"NEXT")) {
			if (!loopstack) {
				be_throw(be_next);
			} else if (loopstack->type == bl_for) {
				double nv = bv_getDouble(loopstack->loopvar);
				nv += loopstack->step;
				if (nv > loopstack->end) {
					loopstack = bx_exit_loop(loopstack);
				} else {
					bv_setDouble(loopstack->loopvar, nv);
					buf = loopstack->start;
				}
			} else if (loopstack->type == bl_foreach) {
				loopstack->begin++;
				if (loopstack->begin > loopstack->end) {
					loopstack = bx_exit_loop(loopstack);
				} else {
					bv_setsp(loopstack->loopvar, &loopstack->looparr->elem[(int)loopstack->begin]);
					buf = loopstack->start;
				}
			} else {
				be_throw(be_next);
			}
		} else if (!strcasecmp(cmd,"WEND")) {
			if (!loopstack || loopstack->type != bl_while) {
				be_throw(be_wend);
			} else {
				b_varl * v;
				v = bx_eval(sc, loopstack->conditionExpr, 0);
				if (bv_getBoolean(v)) {
					buf = loopstack->start;
				} else {
					loopstack = bx_exit_loop(loopstack);
				}
				bv_cfreeBv(v);
			}
		} else if (!strcasecmp(cmd,"LOOP")) {
			if (!loopstack || loopstack->type != bl_doloop) {
				be_throw(be_loop);
			} else {
				char * stuff;
				char * tmp;
				b_varl * v;
				q = bx_ffToEndOfStatement(p);
				stuff = substrim(p,q);
				if (!(*stuff)) {
					buf = loopstack->start;
				} else {
					tmp = bx_ffToIdentifierEnd(stuff,1);
					if (!tmp) {
						be_throw(be_syntax);
					} else {
						(*tmp) = 0;
						tmp++;
						if (strcasecmp(stuff,"WHILE") && strcasecmp(stuff,"UNTIL")) {
							be_throw(be_syntax);
						} else {
							v = bx_eval(sc, tmp, 0);
							if (be_checkerr()) {
							} else if (!v) {
								be_throw(be_undeclared);
							} else if (bv_getBoolean(v) == !strcasecmp(stuff,"WHILE")) {
								buf = loopstack->start;
							} else {
								loopstack = bx_exit_loop(loopstack);
							}
							bv_cfreeBv(v);
						}
					}
				}
				free(stuff);
			}
		} else if (!strcasecmp(cmd,"BREAK")) {
			if (!loopstack) {
				be_throw(be_break);
			} else {
				char * n = 0;
				switch (loopstack->type) {
				case bl_for:
					n = bx_findStructKeyword(loopstack->start+1, "FOR", 0, 0, "NEXT");
					break;
				case bl_foreach:
				case bl_foreachchar:
				case bl_foreachline:
					n = bx_findStructKeyword(loopstack->start+1, "FOREACH", 0, 0, "NEXT");
					break;
				case bl_while:
					n = bx_findStructKeyword(loopstack->start+1, "WHILE", 0, 0, "WEND");
					break;
				case bl_doloop:
					n = bx_findStructKeyword(loopstack->start+1, "DO", 0, 0, "LOOP");
					break;
				}
				if (!n) {
					be_throw(be_noend);
				} else {
					buf = n;
					if (tbuf) free(tbuf);
					tbuf = 0;
					ttbuf = 0;
				}
				loopstack = bx_exit_loop(loopstack);
			}
		} else if (!strcasecmp(cmd,"STOP")) {
			be_throw(be_break);
		} else if (!strcasecmp(cmd,"SUB")) {
			char * stuff;
			char * sname;
			int    pcount = 0;
			char * pnames[4] = {0, 0, 0, 0};
			b_type ptypes[4];
			b_scope * pclasses[4];
			char * code;
			char * n;
			char * pp;
			char * qq;
			q = bx_ffToEndOfStatement(p);
			stuff = substrim(p,q);
			pp = bx_ffToIdentifierEnd(stuff,1);
			sname = substr(stuff,pp);
			if (!(*sname) || !bx_verifyIdent(sname)) {
				be_throw(be_syntax);
			} else {
				qq = strrchr(pp,')');
				pp = strchr(pp,'(');
				if (!pp || !qq) {
					be_throw(be_syntax);
				} else {
					pp = substr(pp+1,qq);
					qq = bx_ffToFirstStatement(qq+1);
					if (*qq) {
						be_throw(be_syntax);
					} else if (bx_parsePlist(sc,pp,&pcount,ptypes,pnames,pclasses,4)) {
						n = bx_findStructKeyword(buf, "SUB", 0, 0, 0);
						if (!n) {
							be_throw(be_noend);
						} else {
							code = substrim(bx_ffToNextStatement(p),n-1);
							bv_addus(sc, sname, bv_createUsub(pcount, pp, pnames[0], ptypes[0], pnames[1], ptypes[1], pnames[2], ptypes[2], pnames[3], ptypes[3], code));
							free(code);
							buf = n;
						}
					}
					bx_freeStrings(pcount, pnames);
					free(pp);
				}
			}
			free(sname);
			free(stuff); /* Yay! Free stuff! */
		} else if (!strcasecmp(cmd,"FUNCTION")) {
			char * stuff;
			char * fname;
			b_type rettype;
			b_scope * retclass;
			int    pcount = 0;
			char * pnames[4] = {0, 0, 0, 0};
			b_type ptypes[4];
			b_scope * pclasses[4];
			char * code;
			char * n;
			char * pp;
			char * qq;
			q = bx_ffToEndOfStatement(p);
			stuff = substrim(p,q);
			pp = bx_ffToIdentifierEnd(stuff,1);
			fname = substr(stuff,pp);
			if (!(*fname) || !bx_verifyIdent(fname)) {
				be_throw(be_syntax);
			} else {
				rettype = bx_autoTypeIdent(fname);
				qq = strrchr(pp,')');
				pp = strchr(pp,'(');
				if (!pp || !qq) {
					be_throw(be_syntax);
				} else {
					pp = substr(pp+1,qq);
					qq = bx_ffToFirstStatement(qq+1);
					if (*qq && strncasecmp(qq,"AS ",3)) {
						be_throw(be_syntax);
					} else {
						if (*qq) {
							(*qq) = 0;
							qq = trim(qq+3);
							be_cthrow(bv_resolveType(sc, qq, &rettype, &retclass));
							free(qq);
						}
						if (!be_checkerr() && bx_parsePlist(sc,pp,&pcount,ptypes,pnames,pclasses,4)) {
							n = bx_findStructKeyword(buf, "FUNCTION", 0, 0, 0);
							if (!n) {
								be_throw(be_noend);
							} else {
								code = substrim(bx_ffToNextStatement(p),n-1);
								bv_adduf(sc, fname, bv_createUfunc(rettype, pcount, pp, pnames[0], ptypes[0], pnames[1], ptypes[1], pnames[2], ptypes[2], pnames[3], ptypes[3], code));
								free(code);
								buf = n;
							}
						}
					}
					bx_freeStrings(pcount, pnames);
					free(pp);
				}
			}
			free(fname);
			free(stuff); /* Yay! Free stuff! */
		} else if (!strcasecmp(cmd,"DEF")) {
			while ((*p)==' ') p++;
			if (strncasecmp(p,"FN ",3)) {
				be_throw(be_syntax);
			} else {
				char * stuff;
				char * eq;
				char * fname;
				char * fparam;
				char * plist;
				char * code;
				p = bx_ffToIdentifierEnd(p,1);
				q = bx_ffToEndOfStatement(p);
				stuff = substrim(p,q);
				eq = strchr(stuff,'=');
				if (!eq) {
					be_throw(be_syntax);
				} else {
					char * teq = eq;
					(*eq) = 0;
					while (teq>stuff && isspace(*(teq-1)))  {
						teq--;
						(*teq) = 0;
					}
					eq++;
					while (isspace(*eq)) eq++;
					
					q = bx_ffToIdentifierEnd(stuff,1);
					fname = substr(stuff,q);
					while ((*q)==' ') q++;
					if (*q != '(' || *(teq-1) != ')') {
						be_throw(be_syntax);
					} else {
						fparam = substr(q+1,teq-1);
						code = malloc(strlen(eq) + 8);
						if (!code) {
							be_throw(be_memory);
						} else {
							plist = malloc(strlen(fparam)+14);
							if (!plist) {
								be_throw(be_memory);
							} else {
								strcpy(plist, fparam);
								strcat(plist, " AS ");
								switch (bx_autoTypeIdent(fparam)) {
									case bt_integer: strcat(plist, "integer"); break;
									case bt_float: strcat(plist, "float"); break;
									case bt_string: strcat(plist, "string"); break;
									case bt_object: strcat(plist, "object"); break;
									case bt_exception: strcat(plist, "exception"); break;
								}
								strcpy(code, "RETURN ");
								strcat(code, eq);
								bv_adduf(sc, fname, bv_createUfunc(bx_autoTypeIdent(fname), 1, plist, fparam, bx_autoTypeIdent(fparam), 0, 0, 0, 0, 0, 0, code));
							}
							free(code);
						}
						free(fparam);
					}
					free(fname);
				}
				free(stuff);
			}
		} else if (!strcasecmp(cmd,"CLASS")) {
			char * stuff;
			char * ex;
			q = bx_ffToEndOfStatement(p);
			stuff = substrim(p,q);
			if ((ex = bx_strcasestrq(stuff," EXTENDS "))) {
				char * tex = ex;
				(*ex) = 0;
				while (tex>stuff && isspace(*(tex-1))) {
					tex--;
					(*tex) = 0;
				}
				ex += 9;
				while (isspace(*ex)) ex++;
			}
			if ( !bx_verifyIdent(stuff) || (ex && !bx_verifyIdent(ex)) ) {
				be_throw(be_syntax);
			} else {
				char * n = bx_findStructKeyword(buf, "CLASS", 0, 0, 0);
				if (!n) {
					be_throw(be_noend);
				} else {
					b_scope * clas = 0;
					b_scope * supr = 0;
					char * b = bx_ffToNextStatement(p);
					char * e = n-1;
					char * code = ((b<=e)?(substrim(b,e)):strdup(""));
					if (!ex) {
						clas = bv_newScope();
					} else if (( supr = bv_findc(sc,ex) )) {
						clas = bv_newScopeChild(supr);
					}
					if (clas) {
						bv_cfreeBv(bx_exec(clas, code));
						if (be_checkerr()) {
							if (be_lastline()) {
								be_set_lineNum(be_lastline() + charCount2(o,p,'\n','\r') + 1);
							}
							bv_freeScope(clas,0);
						} else {
							bv_addc(sc, stuff, clas);
						}
					}
					free(code);
					buf = n;
				}
			}
			free(stuff);
		} else if (!strcasecmp(cmd,"DATA")) {
			/* data - skip */
		} else if (!strcasecmp(cmd,"READ") && (*bx_ffToFirstParameter(p))!='#') {
			while (!bx_statementDelim(*p) && !be_checkerr()) {
				char * stuff;
				b_varl * v;
				b_varl * nv;
				q = bx_ffToEndOfParameter(p);
				stuff = substrim(p,q);
				v = bx_eval(sc,stuff,1);
				free(stuff);
				if (!v) {
					be_cthrow(be_undeclared);
				} else {
					/* find the next data item */
					if (!dataPtr) {
						dataPtr = bx_findStructKeyword(o, 0, "DATA", 0, 0);
					}
					if (dataPtr) {
						if (!(  !strncasecmp(dataPtr,"DATA",4) && (isspace(dataPtr[4]) || bx_statementDelim(dataPtr[4]))  )) {
							dataPtr = bx_ffToNextParameter(dataPtr);
							if (bx_statementDelim(*dataPtr)) {
								dataPtr = bx_findStructKeyword(dataPtr, 0, "DATA", 0, 0);
							}
						}
						while (  dataPtr && !strncasecmp(dataPtr,"DATA",4) && (isspace(dataPtr[4]) || bx_statementDelim(dataPtr[4]))  ) {
							dataPtr = bx_ffToFirstParameter(dataPtr);
							if (bx_statementDelim(*dataPtr)) {
								dataPtr = bx_findStructKeyword(dataPtr, 0, "DATA", 0, 0);
							}
						}
					}
					/* now read it */
					if (!dataPtr) {
						be_throw(be_data);
					} else {
						q = bx_ffToEndOfParameter(dataPtr);
						stuff = substrim(dataPtr,q);
						if (bx_verifyIdent(stuff)) {
							nv = bv_stringToBv(stuff);
						} else {
							nv = bx_eval(sc,stuff,0);
						}
						free(stuff);
						if (!nv) {
							be_cthrow(be_undeclared);
						} else {
							bv_setsp(v,nv);
							bv_cfreeBv(nv);
						}
					}
					bv_cfreeBv(v);
				}
				p = bx_ffToNextParameter(p);
			}
		} else if (!strcasecmp(cmd,"RESTORE")) {
			dataPtr = 0;
		} else {
			/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
			/*        This is the part where commands are called!        */
			/* Since the code is so tiny it is very easy to skip over it */
			/*             while scrolling through this file!            */
			/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
			b_command * c;
			b_usub * us;
			if (( c = bf_FindCommand(cmd) )) {
				bx_callCommand(sc,c,p);
			} else if (( us = bv_findus(sc,cmd) )) {
				bx_callUsub(bv_finduscope(sc,cmd),us,p);
			} else {
				be_cthrow(be_syntax);
			}
		}
		free(cmd);
		commandFinished:
		if (be_checkerr()) {
			if (trystack) {
				if (trystack->catchVar) {
					b_varl * e = bv_exceptionToBv(be_lasterr());
					bv_setsp(trystack->catchVar, e);
					bv_cfreeBv(e);
				}
				be_clearerr();
				buf = trystack->catchLine;
				if (tbuf) free(tbuf);
				tbuf = 0;
				ttbuf = 0;
				if (!strncasecmp(buf,"END",3)) {
					repeatLine = 1;
				}
			} else {
				/* Error handling option 1: call the onerr handler like a subroutine */
				if (onerr) {
					char * tmp = (char *)malloc(strlen(o) + strlen(onerr) + 10);
					if (tmp) {
						tmp[0] = 0;
						strcat(tmp, onerr);
						strcat(tmp, "\nRETURN\n");
						strcat(tmp, o);
						be_clearerr();
						bv_cfreeBv(bx_exec(sc,tmp));
						if (!be_lastfile() && be_lastline()) {
							be_set_lineNum(be_lastline()-2);
						}
						free(tmp);
					}
				}
				if (be_checkerr()) {
					if (!be_lastline()) be_set_lineNum(charCount2(o,p,'\n','\r')+1);
					if (tbuf) free(tbuf);
					break;
				}
				/* Error handling option 2: handle the onerr handler like an if */
				/*
				if (onerr && !onerrcalled) {
					be_clearerr();
					tbuf = trim(onerr);
					onerrcalled = 1;
				} else {
					if (!be_lastline()) be_set_lineNum(charCount2(o,p,'\n','\r')+1);
					if (tbuf) free(tbuf);
					break;
				}
				*/
			}
		}
		if (be_closed()) {
			if (tbuf) free(tbuf);
			break;
		}
		if (ttbuf) {
			if (tbuf) free(tbuf);
			buf = ttbuf;
			tbuf = 0;
			ttbuf = 0;
		} else if (repeatLine) {
			repeatLine = 0;
			buf = bx_ffToFirstStatement(buf);
		} else {
			buf = bx_ffToNextStatement(buf);
		}
	}
	if (onerr) { free(onerr); onerr = 0; }
	bx_exit_all(loopstack);
	bx_return_from_alltries(trystack);
	bx_return_from_all(substack);
	return 0;
}


