/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "btype.h"
#include "vars.h"
#include "exceptions.h"
#include "functions.h"
#include "expr.h"
#include <stdio.h>
#include <string.h>
#include "stringx.h"
#include <stdlib.h>

static char * bf_loadedprog = 0;

void bf_includepath1(b_scope * s, b_varl * a)
{
	if (!a) { be_throw(be_undeclared); } else if (!bv_assertString(a)) { be_throw(be_type); } else {
		bx_set_include_path(bv_getString(a));
	}
}

b_varl * bf_includepath0(b_scope * s)
{
	return bv_stringToBv(bx_get_include_path());
}

void bf_new0(b_scope * s)
{
	if (bf_loadedprog) {
		free(bf_loadedprog);
		bf_loadedprog = 0;
	}
}

void bf_load1(b_scope * s, b_varl * a)
{
	/* load program */
	if (!a) { be_throw(be_undeclared); } else if (!bv_assertString(a)) { be_throw(be_type); } else {
		FILE * fp = fopen(bv_getString(a), "r");
		if (!fp) {
			be_throw(be_fileerr());
		} else {
			int l;
			if (bf_loadedprog) {
				free(bf_loadedprog);
				bf_loadedprog = 0;
			}
			fseek(fp, 0, SEEK_END);
			l = ftell(fp);
			fseek(fp, 0, SEEK_SET);
			bf_loadedprog = (char *)malloc(l+1);
			if (bf_loadedprog) {
				fread(bf_loadedprog, 1, l, fp);
				bf_loadedprog[l] = 0;
			} else {
				be_throw(be_memory);
			}
			fclose(fp);
		}
	}
}

void bf_save1(b_scope * s, b_varl * a)
{
	/* save program */
	if (!a) { be_throw(be_undeclared); } else if (!bv_assertString(a)) { be_throw(be_type); } else {
		FILE * fp = fopen(bv_getString(a), "w");
		if (!fp) {
			be_throw(be_fileerr());
		} else {
			if (bf_loadedprog) {
				fwrite(bf_loadedprog, 1, strlen(bf_loadedprog), fp);
			}
			fclose(fp);
		}
	}
}

void bf_run0(b_scope * s)
{
	if (bf_loadedprog) {
		b_scope * ns = bv_newScope();
		bv_cfreeBv(bx_exec(ns, bf_loadedprog));
		bv_freeScope(ns,0);
	}
}

void bf_run1(b_scope * s, b_varl * a)
{
	/* runs in an entirely new variable scope */
	if (!a) { be_throw(be_undeclared); } else if (!bv_assertString(a)) { be_throw(be_type); } else {
		FILE * fp = fopen(bv_getString(a), "r");
		if (!fp) {
			be_throw(be_fileerr());
		} else {
			int l;
			char * buf;
			fseek(fp, 0, SEEK_END);
			l = ftell(fp);
			fseek(fp, 0, SEEK_SET);
			buf = (char *)malloc(l+1);
			if (buf) {
				b_scope * ns = bv_newScope();
				fread(buf, 1, l, fp);
				buf[l] = 0;
				bv_cfreeBv(bx_exec(ns, buf));
				if (be_checkerr()) {
					be_set_fileName(bv_getString(a));
				}
				bv_freeScope(ns,0);
				free(buf);
			}
			fclose(fp);
		}
	}
}

void bf_subrun0(b_scope * s)
{
	if (bf_loadedprog) {
		b_scope * ns = bv_newScopeChild(s);
		bv_cfreeBv(bx_exec(ns, bf_loadedprog));
		bv_freeScope(ns,0);
	}
}

void bf_subrun1(b_scope * s, b_varl * a)
{
	/* runs in a child of the current variable scope */
	/* variables can be get and set, but not added or deleted from the current variable scope */
	if (!a) { be_throw(be_undeclared); } else if (!bv_assertString(a)) { be_throw(be_type); } else {
		FILE * fp = fopen(bv_getString(a), "r");
		if (!fp) {
			be_throw(be_fileerr());
		} else {
			int l;
			char * buf;
			fseek(fp, 0, SEEK_END);
			l = ftell(fp);
			fseek(fp, 0, SEEK_SET);
			buf = (char *)malloc(l+1);
			if (buf) {
				b_scope * ns = bv_newScopeChild(s);
				fread(buf, 1, l, fp);
				buf[l] = 0;
				bv_cfreeBv(bx_exec(ns, buf));
				if (be_checkerr()) {
					be_set_fileName(bv_getString(a));
				}
				bv_freeScope(ns,0);
				free(buf);
			}
			fclose(fp);
		}
	}
}

void bf_chain0(b_scope * s)
{
	if (bf_loadedprog) {
		bv_cfreeBv(bx_exec(s, bf_loadedprog));
	}
}

void bf_chain1(b_scope * s, b_varl * a)
{
	/* runs in the current variable scope */
	if (!a) { be_throw(be_undeclared); } else if (!bv_assertString(a)) { be_throw(be_type); } else {
		FILE * fp = fopen(bv_getString(a), "r");
		if (!fp) {
			be_throw(be_fileerr());
		} else {
			int l;
			char * buf;
			fseek(fp, 0, SEEK_END);
			l = ftell(fp);
			fseek(fp, 0, SEEK_SET);
			buf = (char *)malloc(l+1);
			if (buf) {
				fread(buf, 1, l, fp);
				buf[l] = 0;
				bv_cfreeBv(bx_exec(s, buf));
				if (be_checkerr()) {
					be_set_fileName(bv_getString(a));
				}
				free(buf);
			}
			fclose(fp);
		}
	}
}

void bf_include0(b_scope * s)
{
	if (bf_loadedprog) {
		b_scope * ns = s;
		while (ns->fallback) ns = ns->fallback;
		bv_cfreeBv(bx_exec(ns, bf_loadedprog));
	}
}

void bf_include1(b_scope * s, b_varl * a)
{
	/* runs in the parent of all current variable scopes */
	if (!a) { be_throw(be_undeclared); } else if (!bv_assertString(a)) { be_throw(be_type); } else {
		char filename[1024];
		FILE * fp;
		strcpy(filename, bv_getString(a));
		fp = fopen(filename, "r");
		if (!fp) {
			strcat(filename, ".bi");
			fp = fopen(filename, "r");
		}
		if (bx_get_include_path()[0]) {
			if (!fp) {
				strcpy(filename, bx_get_include_path());
				if (filename[strlen(filename)-1] != '/') strcat(filename, "/");
				strcat(filename, bv_getString(a));
				fp = fopen(filename, "r");
			}
			if (!fp) {
				strcat(filename, ".bi");
				fp = fopen(filename, "r");
			}
		}
		if (!fp) {
			be_throw(be_fileerr());
		} else {
			int l;
			char * buf;
			fseek(fp, 0, SEEK_END);
			l = ftell(fp);
			fseek(fp, 0, SEEK_SET);
			buf = (char *)malloc(l+1);
			if (buf) {
				b_scope * ns = s;
				while (ns->fallback) ns = ns->fallback;
				fread(buf, 1, l, fp);
				buf[l] = 0;
				bv_cfreeBv(bx_exec(ns, buf));
				if (be_checkerr()) {
					be_set_fileName(bv_getString(a));
				}
				free(buf);
			}
			fclose(fp);
		}
	}
}

void bf_list0(b_scope * s)
{
	if (bf_loadedprog) {
		printf("%s\n",bf_loadedprog);
	}
}

void bf_list1(b_scope * s, b_varl * a)
{
	/* just display contents */
	if (!a) { be_throw(be_undeclared); } else if (!bv_assertString(a)) { be_throw(be_type); } else {
		FILE * fp = fopen(bv_getString(a), "r");
		if (!fp) {
			be_throw(be_fileerr());
		} else {
			int l;
			char * buf;
			fseek(fp, 0, SEEK_END);
			l = ftell(fp);
			fseek(fp, 0, SEEK_SET);
			buf = (char *)malloc(l+1);
			if (buf) {
				fread(buf, 1, l, fp);
				buf[l] = 0;
				printf("%s\n",buf);
				free(buf);
			}
			fclose(fp);
		}
	}
}

void bf_lets(b_scope * s, char * c)
{
	char * pp = bx_ffToFirstStatement(c);
	char * qq;
	while (!bx_statementDelim(*pp) && !be_checkerr()) {
		qq = bx_ffToEndOfParameter(pp);
		char * line = substrim(pp,qq);
		char * splt = strchr(line, '=');
		char * lval;
		char * rval;
		b_varl * lvarl;
		b_varl * rvarl;
		if (!splt) {
			be_throw(be_syntax);
			free(line);
			return;
		}
		(*splt) = 0;
		splt++;
		lval = trim(line);
		rval = trim(splt);
		if (!( (*lval) && (*rval) )) {
			be_throw(be_syntax);
			free(rval);
			free(lval);
			free(line);
			return;
		}
		rvarl = bx_eval(s,rval,0);
		if (!rvarl) {
			be_cthrow(be_undeclared);
			free(rval);
			free(lval);
			free(line);
			return;
		}
		lvarl = bx_eval(s,lval,0);
		if (lvarl) {
			bv_setsp(lvarl,rvarl);
			bv_cfreeBv(lvarl);
		} else {
			if (bx_verifyIdent(lval)) {
				be_clearerr();
				bv_add(s, lval, rvarl);
			} else {
				be_cthrow(be_syntax);
			}
		}
		bv_cfreeBv(rvarl);
		free(rval);
		free(lval);
		free(line);
		pp = bx_ffToNextParameter(pp);
	}
}

void bf_consts(b_scope * s, char * c)
{
	char * pp = bx_ffToFirstStatement(c);
	char * qq;
	while (!bx_statementDelim(*pp) && !be_checkerr()) {
		qq = bx_ffToEndOfParameter(pp);
		char * line = substrim(pp,qq);
		char * splt = strchr(line, '=');
		char * lval;
		char * rval;
		b_varl * lvarl;
		b_varl * rvarl;
		if (!splt) {
			be_throw(be_syntax);
			free(line);
			return;
		}
		(*splt) = 0;
		splt++;
		lval = trim(line);
		rval = trim(splt);
		if (!( (*lval) && (*rval) )) {
			be_throw(be_syntax);
			free(rval);
			free(lval);
			free(line);
			return;
		}
		rvarl = bx_eval(s,rval,0);
		if (!rvarl) {
			be_cthrow(be_undeclared);
			free(rval);
			free(lval);
			free(line);
			return;
		}
		lvarl = bx_eval(s,lval,0);
		if (lvarl) {
			lvarl->readOnly = 0;
			bv_setsp(lvarl,rvarl);
			lvarl->readOnly = 1;
			bv_cfreeBv(lvarl);
		} else {
			if (bx_verifyIdent(lval)) {
				be_clearerr();
				bv_add(s, lval, rvarl);
				(bv_find(s, lval))->readOnly = 1;
			} else {
				be_cthrow(be_syntax);
			}
		}
		bv_cfreeBv(rvarl);
		free(rval);
		free(lval);
		free(line);
		pp = bx_ffToNextParameter(pp);
	}
}

void bf_dels(b_scope * s, char * c)
{
	char * pp = bx_ffToFirstStatement(c);
	char * qq;
	while (!bx_statementDelim(*pp) && !be_checkerr()) {
		qq = bx_ffToEndOfParameter(pp);
		char * line = substrim(pp,qq);
		if (strstr(c,"[]")) {
			char * line2 = left(line, strlen(line)-2);
			char * line3 = trim(line2);
			if (!bx_verifyIdent(line3)) { be_throw(be_syntax); } else { bv_dela(s, line3); }
			free(line3);
			free(line2);
		} else {
			if (!bx_verifyIdent(line)) { be_throw(be_syntax); } else { bv_del(s, line); }
		}
		free(line);
		pp = bx_ffToNextParameter(pp);
	}
}

void bf_clear0(b_scope * s)
{
	bv_freeVA(s);
}

void bf_dims(b_scope * s, char * c)
{
	char * pp = bx_ffToFirstStatement(c);
	char * qq;
	while (!bx_statementDelim(*pp) && !be_checkerr()) {
		qq = bx_ffToEndOfParameter(pp);
		char * line = substrim(pp,qq);
		char * ts = 0;
		b_type t = bx_autoTypeIdent(line);
		b_scope * cl = 0;
		char * p = strcasestr(line, " AS ");
		if (p) {
			char * tmp;
			ts = trim(p+4);
			be_cthrow(bv_resolveType(s, ts, &t, &cl));
			tmp = substrim(line,p);
			free(line);
			line = tmp;
		}
		if (be_checkerr()) {
			if (ts) free(ts);
			free(line);
			break;
		}
		p = bx_ffToIdentifierEnd(line, 1);
		if (*p) {
			char * name = substr(line,p);
			int dim = 0;
			int dims[3] = {1, 1, 1};
			p = bx_ffToFirstFParameter(p);
			if (bx_statementDelim(*p) || bx_levelDec(*p)) { be_throw(be_syntax); } else {
				char * q;
				char * d;
				char * e;
				b_varl * v;
				do {
					q = bx_ffToEndOfFParameter(p);
					d = substr(p,q);
					e = trim(d);
					free(d);
					v = bx_eval(s,e,0);
					if (!v) {
						be_throw(be_undeclared);
					} else if (!bv_assertNumeric(v)) {
						be_throw(be_type);
					} else {
						if (dim >= 3) {
							be_throw(be_subscript);
						} else {
							dims[dim] = bv_getInt(v);
							dim++;
						}
					}
					bv_cfreeBv(v);
					free(e);
					p = bx_ffToNextFParameter(p);
				} while (!(bx_statementDelim(*p) || bx_levelDec(*p)));
				bv_aadd(s, name, t, dim, dims[0], dims[1], dims[2]);
			}
			free(name);
		} else {
			if (!bx_verifyIdent(line)) { be_throw(be_syntax); } else {
				switch (t) {
					case bt_integer: bv_add(s, line, bv_intToBv(0)); break;
					case bt_float: bv_add(s, line, bv_doubleToBv(0.0)); break;
					case bt_string: bv_add(s, line, bv_stringToBv("")); break;
					case bt_object: bv_add(s, line, bv_objectToBv(NULL, cl)); break;
					case bt_exception: bv_add(s, line, bv_exceptionToBv(be_none)); break;
					default: be_throw(be_type); break;
				}
			}
		}
		if (ts) free(ts);
		free(line);
		pp = bx_ffToNextParameter(pp);
	}
}

void bf_redims(b_scope * s, char * c)
{
	char * pp = bx_ffToFirstStatement(c);
	char * qq;
	b_array * arr;
	while (!bx_statementDelim(*pp) && !be_checkerr()) {
		qq = bx_ffToEndOfParameter(pp);
		char * line = substrim(pp,qq);
		char * p = strcasestr(line, " AS ");
		if (p) {
			be_throw(be_arrtype);
			free(line);
			break;
		}
		p = bx_ffToIdentifierEnd(line, 1);
		if (*p) {
			char * name = substr(line,p);
			int dim = 0;
			int dims[3] = {1, 1, 1};
			p = bx_ffToFirstFParameter(p);
			if (bx_statementDelim(*p) || bx_levelDec(*p)) { be_throw(be_syntax); } else {
				char * q;
				char * d;
				char * e;
				b_varl * v;
				do {
					q = bx_ffToEndOfFParameter(p);
					d = substr(p,q);
					e = trim(d);
					free(d);
					v = bx_eval(s,e,0);
					if (!v) {
						be_throw(be_undeclared);
					} else if (!bv_assertNumeric(v)) {
						be_throw(be_type);
					} else {
						if (dim >= 3) {
							be_throw(be_subscript);
						} else {
							dims[dim] = bv_getInt(v);
							dim++;
						}
					}
					bv_cfreeBv(v);
					free(e);
					p = bx_ffToNextFParameter(p);
				} while (!(bx_statementDelim(*p) || bx_levelDec(*p)));
				arr = bv_finda(s, name);
				if (!arr) {
					be_throw(be_undeclared);
				} else if (!bv_redimArray(arr, dim, dims[0], dims[1], dims[2])) {
					be_throw(be_memory);
				}
			}
			free(name);
		} else {
			be_throw(be_cast);
			free(line);
			break;
		}
		free(line);
		pp = bx_ffToNextParameter(pp);
	}
}

void bf_swap2(b_scope * s, b_varl * a, b_varl * b)
{
	if (!a || !b) {
		be_throw(be_undeclared);
	} else if (a->readOnly || b->readOnly) {
		be_throw(be_constant);
	} else {
		b_varl c;
		c = (*b);
		(*b) = (*a);
		(*a) = c;
	}
}

b_varl * bf_ifs(b_scope * s, char * c)
{
	char * cond;
	b_varl * condv;
	char * ret;
	b_varl * retv;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		cond = substr(p,q);
		condv = bx_eval(s,cond,0);
		if (!condv) {
			free(cond);
			return 0;
		}
		p = bx_ffToNextParameter(p);
		if (bx_statementDelim(*p)) {
			free(cond);
			return condv;
		}
		q = bx_ffToEndOfParameter(p);
		ret = substr(p,q);
		if (bv_getBoolean(condv)) {
			free(cond);
			bv_cfreeBv(condv);
			retv = bx_eval(s,ret,0);
			free(ret);
			return retv;
		}
		free(cond);
		bv_cfreeBv(condv);
		p = bx_ffToNextParameter(p);
	}
	be_throw(be_syntax);
	return 0;
}

b_varl * bf_ubounds(b_scope * s, char * c)
{
	char * n = trim(c);
	char * i = bx_ffToEndOfParameter(n);
	b_varl * v = 0;
	int ii = -1;
	if (i >= (n+strlen(n))) {
		ii = 1;
	} else {
		(*i) = 0;
		i++;
		v = bx_eval(s,i,0);
		if (!v) {
			be_throw(be_undeclared);
		} else if (!bv_assertNumeric(v)) {
			be_throw(be_cast);
		} else {
			ii = bv_getInt(v);
			if (ii<0 || ii>3) be_throw(be_range);
		}
		bv_cfreeBv(v);
		i = n;
		n = trim(n);
		free(i);
	}
	if (ii>=0 && ii<4) {
		if (!bx_verifyIdent(n)) {
			be_throw(be_syntax);
		} else {
			b_array * a = bv_finda(s,n);
			if (!a) {
				be_throw(be_undeclared);
			} else {
				switch (ii) {
				case 0: v = bv_intToBv(a->dim); break;
				case 1: v = bv_intToBv(a->dim1-1); break;
				case 2: v = bv_intToBv(a->dim2-1); break;
				case 3: v = bv_intToBv(a->dim3-1); break;
				}
			}
		}
	}
	free(n);
	return v;
}

b_varl * bf_clone1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertObject(a) )) { return 0; }
	if (!a->oval) { be_throw(be_nullptr); return 0; }
	b_object * o = bv_copyObj(a->oval);
	b_varl * v = bv_objectToBv(o, o->classScope);
	bv_unuseObj(o);
	return v;
}

b_varl * bf_news(b_scope * sc, char * ss)
{
	char * s = trim(ss);
	b_scope * clas = bv_findc(sc,s);
	if (!clas) { be_throw(be_type); free(s); return 0; }
	b_object * o = bv_createObj(s,clas);
	b_varl * v = bv_objectToBv(o,clas);
	bv_unuseObj(o);
	free(s);
	return v;
}

b_varl * bf_errstr1(b_scope * sc, b_varl * a)
{
	return bv_stringToBv(be_errstr(bv_getException(a)));
}

void bf_throw1(b_scope * sc, b_varl * a)
{
	be_throw(bv_getException(a));
}

b_varl * bf_commands0(b_scope * s)
{
	return bv_intToBv(bf_CommandCount());
}

b_varl * bf_commands1(b_scope * s, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	int i = bv_getInt(a);
	if (i==-1) return bv_intToBv(bf_CommandCount());
	if (i==-2) return bv_intToBv(MAX_COMMANDS);
	if (i<0 || i>=bf_CommandCount()) return bv_stringToBv("");
	return bv_stringToBv((char *)bf_CommandName(i));
}

b_varl * bf_functions0(b_scope * s)
{
	return bv_intToBv(bf_FunctionCount());
}

b_varl * bf_functions1(b_scope * s, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	int i = bv_getInt(a);
	if (i==-1) return bv_intToBv(bf_FunctionCount());
	if (i==-2) return bv_intToBv(MAX_FUNCTIONS);
	if (i<0 || i>=bf_FunctionCount()) return bv_stringToBv("");
	return bv_stringToBv((char *)bf_FunctionName(i));
}

b_varl * bf_about1(b_scope * s, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	switch (bv_getInt(a)) {
		case 0: return bv_stringToBv("LET BASIC"); break;
		case 1: return bv_stringToBv("version 1.2.1"); break;
		case 2: return bv_stringToBv("9 April 2008"); break;
		case 3: return bv_stringToBv("(c) 2006-2008 Rebecca G. Bettencourt"); break;
		case 4: return bv_stringToBv("(c) 2006-2008 Kreative Software"); break;
		case 56: return bv_stringToBv("Margaret Chastain is a Child Diva!!!! <3 xox <3"); break;
		case 65: return bv_stringToBv("Mark Chastain is a Man-Child!!!"); break;
		case 77: return bv_stringToBv("I'm a pretty girl and I'm happy happy happy..."); break;
		case 99: bv_seth(s, "REMREM$", bv_stringToBv("Laura Tres")); break;
		case 100: return bv_stringToBv("It is practically impossible to teach good programming to students that have had a prior exposure to BASIC: as potential programmers they are mentally mutilated beyond hope of regeneration. -- Edsger W. Dijkstra"); break;
	}
	return bv_stringToBv("");
}

void bf_about0(b_scope * s)
{
	int i=0;
	while (1) {
		b_varl * n = bv_intToBv(i);
		b_varl * st = bf_about1(s,n);
		if (st->sval && st->sval[0]) {
			printf("%s\n", st->sval);
		} else {
			bv_cfreeBv(st);
			bv_cfreeBv(n);
			break;
		}
		bv_cfreeBv(st);
		bv_cfreeBv(n);
		i++;
	}
}
