/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "btype.h"
#include "vars.h"
#include "expr.h"
#include "exceptions.h"
#include <stdio.h>
#include <string.h>
#include "stringx.h"
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/stat.h>

void bf_open3(b_scope * sc, b_varl * mode, b_varl * fnum, b_varl * path)
{
	if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); return; }
	char * m = trim(bv_getString(mode));
	FILE * fp;
	int i;
	for (i=0; i<strlen(m); i++) {
		switch (m[i]) {
		case 'R': m[i]='r'; break;
		case 'W': m[i]='w'; break;
		case 'A': m[i]='a'; break;
		case 'T': m[i]='t'; break;
		case 'B': m[i]='b'; break;
		case 'I': m[i]='r'; break;
		case 'O': m[i]='w'; break;
		case 'i': m[i]='r'; break;
		case 'o': m[i]='w'; break;
		}
	}
	fp = fopen(bv_getString(path),m);
	if (!fp) {
		be_throw(be_fileerr());
	} else {
		bv_setInt(fnum,(long int)fp);
	}
	free(m);
}

void bf_close1(b_scope * sc, b_varl * fnum)
{
	if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); return; }
	fclose((FILE *)bv_getInt(fnum));
}

void bf_flush1(b_scope * sc, b_varl * fnum)
{
	if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); return; }
	fflush((FILE *)bv_getInt(fnum));
}

b_varl * bf_eof1(b_scope * sc, b_varl * fnum)
{
	if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); return 0; }
	return bv_intToBv(feof((FILE *)bv_getInt(fnum)));
}

void bf_rewind1(b_scope * sc, b_varl * fnum)
{
	if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); return; }
	rewind((FILE *)bv_getInt(fnum));
}

void bf_seek2(b_scope * sc, b_varl * fnum, b_varl * pos)
{
	if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); return; }
	fseek((FILE *)bv_getInt(fnum), bv_getInt(pos), SEEK_SET);
}

void bf_seek3(b_scope * sc, b_varl * fnum, b_varl * pos, b_varl * org)
{
	if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); return; }
	fseek((FILE *)bv_getInt(fnum), bv_getInt(pos), bv_getInt(org));
}

b_varl * bf_tell1(b_scope * sc, b_varl * fnum)
{
	if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); return 0; }
	return bv_intToBv(ftell((FILE *)bv_getInt(fnum)));
}

void bf_writes(b_scope * sc, char * c)
{
	char * tmp;
	b_varl * tmpv;
	b_varl * fnum;
	FILE * fp = 0;
	int first = 1;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	if (bx_statementDelim(*p)) be_throw(be_syntax);
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		tmp = substrim(p,q);
		if (first) {
			fnum = bx_eval(sc,tmp,0);
			if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); bv_cfreeBv(fnum); free(tmp); return; }
			fp = (FILE *)bv_getInt(fnum);
			bv_cfreeBv(fnum);
			first = 0;
		} else {
			if (*tmp) {
				tmpv = bx_eval(sc,tmp,0);
				if (!tmpv) {
					be_cthrow(be_undeclared);
				} else {
					fprintf(fp, "%s", bv_getString(tmpv));
				}
				bv_cfreeBv(tmpv);
			}
			switch (*q) {
				case ';': break;
				case ',': fprintf(fp, "\t"); break;
				case  0 : fprintf(fp, "\n"); break;
			}
		}
		free(tmp);
		p = bx_ffToNextParameter(p);
	}
}

void bf_writebs(b_scope * sc, char * c)
{
	char * tmp;
	b_varl * tmpv;
	b_varl * fnum;
	FILE * fp = 0;
	int first = 1;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	if (bx_statementDelim(*p)) be_throw(be_syntax);
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		tmp = substrim(p,q);
		if (first) {
			fnum = bx_eval(sc,tmp,0);
			if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); bv_cfreeBv(fnum); free(tmp); return; }
			fp = (FILE *)bv_getInt(fnum);
			bv_cfreeBv(fnum);
			first = 0;
		} else {
			if (*tmp) {
				tmpv = bx_eval(sc,tmp,0);
				if (!tmpv) {
					be_cthrow(be_undeclared);
				} else {
					switch (tmpv->type) {
					case bt_integer:
						fwrite(&tmpv->ival, sizeof(signed long int), 1, fp);
						break;
					case bt_float:
						fwrite(&tmpv->fval, sizeof(double), 1, fp);
						break;
					case bt_string:
						fwrite(tmpv->sval, sizeof(char), strlen(tmpv->sval)+1, fp);
						break;
					case bt_object:
						be_throw(be_cast);
						break;
					case bt_exception:
						fwrite(&tmpv->eval, sizeof(b_exception), 1, fp);
						break;
					}
				}
				bv_cfreeBv(tmpv);
			}
		}
		free(tmp);
		p = bx_ffToNextParameter(p);
	}
}

void bf_writem3(b_scope * sc, b_varl * fnum, b_varl * addr, b_varl * len)
{
	if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); return; }
	if (!( bv_assertNumeric(addr) && bv_assertNumeric(len) )) { be_throw(be_cast); return; }
	FILE * fp = (FILE *)bv_getInt(fnum);
	fwrite((void *)bv_getInt(addr), 1, bv_getInt(len), fp);
}

void bf_reads(b_scope * sc, char * c)
{
	char * tmp;
	b_varl * tmpv;
	b_varl * fnum;
	FILE * fp = 0;
	int first = 1;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	if (bx_statementDelim(*p)) be_throw(be_syntax);
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		tmp = substrim(p,q);
		if (first) {
			fnum = bx_eval(sc,tmp,0);
			if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); bv_cfreeBv(fnum); free(tmp); return; }
			fp = (FILE *)bv_getInt(fnum);
			bv_cfreeBv(fnum);
			first = 0;
		} else {
			if (*tmp) {
				tmpv = bx_eval(sc,tmp,1);
				if (!tmpv) {
					be_cthrow(be_undeclared);
				} else {
					switch (tmpv->type) {
					case bt_integer:
						fscanf(fp, "%ld", &tmpv->ival);
						break;
					case bt_float:
						fscanf(fp, "%lf", &tmpv->fval);
						break;
					case bt_string: {
						char buf[65536];
						char * p = buf;
						char last;
						do {
							fread(&last, sizeof(char), 1, fp);
							(*p) = last;
							p++;
						} while (last && last!='\n' && last!='\r' && (p < (buf+65530)));
						if (last == '\n' || last == '\r') (*(p-1)) = 0; else (*p) = 0;
						bv_setString(tmpv, buf);
						} break;
					case bt_object:
						be_throw(be_cast);
						break;
					case bt_exception: {
						int e;
						fscanf(fp, "%d", &e);
						tmpv->eval = e;
						} break;
					}
					
					fprintf(fp, "%s", bv_getString(tmpv));
				}
				bv_cfreeBv(tmpv);
			}
			switch (*q) {
				case ';': break;
				case ',': fprintf(fp, "\t"); break;
				case  0 : fprintf(fp, "\n"); break;
			}
		}
		free(tmp);
		p = bx_ffToNextParameter(p);
	}
}

void bf_readbs(b_scope * sc, char * c)
{
	char * tmp;
	b_varl * tmpv;
	b_varl * fnum;
	FILE * fp = 0;
	int first = 1;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	if (bx_statementDelim(*p)) be_throw(be_syntax);
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		tmp = substrim(p,q);
		if (first) {
			fnum = bx_eval(sc,tmp,0);
			if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); bv_cfreeBv(fnum); free(tmp); return; }
			fp = (FILE *)bv_getInt(fnum);
			bv_cfreeBv(fnum);
			first = 0;
		} else {
			if (*tmp) {
				tmpv = bx_eval(sc,tmp,1);
				if (!tmpv) {
					be_cthrow(be_undeclared);
				} else {
					switch (tmpv->type) {
					case bt_integer:
						fread(&tmpv->ival, sizeof(signed long int), 1, fp);
						break;
					case bt_float:
						fread(&tmpv->fval, sizeof(double), 1, fp);
						break;
					case bt_string: {
						char buf[65536];
						char * p = buf;
						char last;
						do {
							fread(&last, sizeof(char), 1, fp);
							(*p) = last;
							p++;
						} while (last && (p < (buf+65530)));
						bv_setString(tmpv, buf);
						} break;
					case bt_object:
						be_throw(be_cast);
						break;
					case bt_exception:
						fread(&tmpv->eval, sizeof(b_exception), 1, fp);
						break;
					}
				}
				bv_cfreeBv(tmpv);
			}
		}
		free(tmp);
		p = bx_ffToNextParameter(p);
	}
}

void bf_readm3(b_scope * sc, b_varl * fnum, b_varl * addr, b_varl * len)
{
	if (!fnum || fnum->dyna || fnum->type != bt_integer) { be_throw(be_cast); return; }
	if (!( bv_assertNumeric(addr) && bv_assertNumeric(len) )) { be_throw(be_cast); return; }
	FILE * fp = (FILE *)bv_getInt(fnum);
	fread((void *)bv_getInt(addr), 1, bv_getInt(len), fp);
}

void bf_create1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_throw(be_cast); return; }
	FILE * fp;
	if (!( fp = fopen(bv_getString(a),"w") )) be_throw(be_fileerr());
	else fclose(fp);
}

void bf_delete1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_throw(be_cast); return; }
	if (remove(bv_getString(a))) be_throw(be_fileerr());
}

void bf_rename2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertString(b) )) { be_throw(be_cast); return; }
	if (rename(bv_getString(a),bv_getString(b))) be_throw(be_fileerr());
}

void bf_chdir1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_throw(be_cast); return; }
	if (chdir(bv_getString(a))) be_throw(be_fileerr());
}

b_varl * bf_cwdir0(b_scope * sc)
{
	char buf[65536];
	if (!getcwd(buf,65535)) { be_throw(be_fileerr()); return 0; }
	return bv_stringToBv(buf);
}

void bf_chmod2(b_scope * sc, b_varl * a, b_varl * mode)
{
	if (!( bv_assertString(a) && bv_assertNumeric(mode) )) { be_throw(be_cast); return; }
	if (chmod(bv_getString(a),bv_getInt(mode))) be_throw(be_fileerr());
}

void bf_chown3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_throw(be_cast); return; }
	if (chown(bv_getString(a),bv_getInt(b),bv_getInt(c))) be_throw(be_fileerr());
}

b_varl * bf_dir1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_throw(be_cast); return 0; }
	struct direct ** files;
	int count;
	int i;
	char * cat = (char *)malloc(1); cat[0] = 0;
	b_varl * v = 0;
	count = scandir(bv_getString(a),&files,0,0);
	if (count<0) { be_throw(be_fileerr()); return 0; }
	for (i=0; i<count; i++) {
		cat = (char *)realloc(cat, strlen(cat)+strlen(files[i]->d_name)+2);
		if (!cat) { be_throw(be_memory); break; }
		strcat(cat, files[i]->d_name);
		if ((i+1)<count) strcat(cat, "\n");
	}
	if (cat) {
		v = bv_stringToBv(cat);
		free(cat);
	}
	return v;
}

b_varl * bf_dir0(b_scope * sc)
{
	b_varl * a = bv_stringToBv(".");
	b_varl * d = bf_dir1(sc,a);
	bv_cfreeBv(a);
	return d;
}

void bf_cdir1(b_scope * sc, b_varl * a)
{
	b_varl * d = bf_dir1(sc,a);
	if (d) {
		printf("%s\n", bv_getString(d));
	}
	bv_cfreeBv(d);
}

void bf_cdir0(b_scope * sc)
{
	b_varl * d = bf_dir0(sc);
	if (d) {
		printf("%s\n", bv_getString(d));
	}
	bv_cfreeBv(d);
}

void bf_mkdir1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_throw(be_cast); return; }
	if (mkdir(bv_getString(a),0755)) be_throw(be_fileerr());
}

void bf_rmdir1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_throw(be_cast); return; }
	if (rmdir(bv_getString(a))) be_throw(be_fileerr());
}

void bf_link2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertString(b) )) { be_throw(be_cast); return; }
	if (link(bv_getString(a),bv_getString(b))) be_throw(be_fileerr());
}

void bf_symlink2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertString(b) )) { be_throw(be_cast); return; }
	if (symlink(bv_getString(a),bv_getString(b))) be_throw(be_fileerr());
}

void bf_system1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_throw(be_cast); return; }
	if (system(bv_getString(a))) be_throw(be_fileerr());
}
