/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "btype.h"
#include "vars.h"
#include "expr.h"
#include <math.h>
#include "mathx.h"
#include <string.h>
#include "stringx.h"
#include <stdlib.h>

b_varl * bf_maxs(b_scope * s, char * c)
{
	char * a;
	b_varl * av;
	double mymax = 0;
	int firstOne = 1;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		a = substr(p,q);
		av = bx_eval(s,a,0);
		if (!av) {
			free(a);
			return 0;
		}
		if (firstOne) {
			mymax = bv_getDouble(av);
			firstOne = 0;
		} else if (bv_getDouble(av) > mymax) {
			mymax = bv_getDouble(av);
		}
		free(a);
		bv_cfreeBv(av);
		p = bx_ffToNextParameter(p);
	}
	return bv_doubleToBv(mymax);
}

b_varl * bf_mins(b_scope * s, char * c)
{
	char * a;
	b_varl * av;
	double mymin = 0;
	int firstOne = 1;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		a = substr(p,q);
		av = bx_eval(s,a,0);
		if (!av) {
			free(a);
			return 0;
		}
		if (firstOne) {
			mymin = bv_getDouble(av);
			firstOne = 0;
		} else if (bv_getDouble(av) < mymin) {
			mymin = bv_getDouble(av);
		}
		free(a);
		bv_cfreeBv(av);
		p = bx_ffToNextParameter(p);
	}
	return bv_doubleToBv(mymin);
}

b_varl * bf_sums(b_scope * s, char * c)
{
	char * a;
	b_varl * av;
	double mysum = 0;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		a = substr(p,q);
		av = bx_eval(s,a,0);
		if (!av) {
			free(a);
			return 0;
		}
		mysum += bv_getDouble(av);
		free(a);
		bv_cfreeBv(av);
		p = bx_ffToNextParameter(p);
	}
	return bv_doubleToBv(mysum);
}

b_varl * bf_avgs(b_scope * s, char * c)
{
	char * a;
	b_varl * av;
	double mysum = 0;
	double n = 0;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		a = substr(p,q);
		av = bx_eval(s,a,0);
		if (!av) {
			free(a);
			return 0;
		}
		mysum += bv_getDouble(av);
		n += 1.0;
		free(a);
		bv_cfreeBv(av);
		p = bx_ffToNextParameter(p);
	}
	return bv_doubleToBv(mysum/n);
}

b_varl * bf_prods(b_scope * s, char * c)
{
	char * a;
	b_varl * av;
	double myprod = 1;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		a = substr(p,q);
		av = bx_eval(s,a,0);
		if (!av) {
			free(a);
			return 0;
		}
		myprod *= bv_getDouble(av);
		free(a);
		bv_cfreeBv(av);
		p = bx_ffToNextParameter(p);
	}
	return bv_doubleToBv(myprod);
}

b_varl * bf_geoms(b_scope * s, char * c)
{
	char * a;
	b_varl * av;
	double myprod = 1;
	double n = 0;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		a = substr(p,q);
		av = bx_eval(s,a,0);
		if (!av) {
			free(a);
			return 0;
		}
		myprod *= bv_getDouble(av);
		n += 1.0;
		free(a);
		bv_cfreeBv(av);
		p = bx_ffToNextParameter(p);
	}
	return bv_doubleToBv(pow(myprod,1.0/n));
}

b_varl * bf_hypots(b_scope * s, char * c)
{
	char * a;
	b_varl * av;
	double mysum = 0;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		a = substr(p,q);
		av = bx_eval(s,a,0);
		if (!av) {
			free(a);
			return 0;
		}
		double term = bv_getDouble(av);
		mysum += term*term;
		free(a);
		bv_cfreeBv(av);
		p = bx_ffToNextParameter(p);
	}
	return bv_doubleToBv(sqrt(mysum));
}

b_varl * bf_rsrs(b_scope * s, char * c)
{
	char * a;
	b_varl * av;
	double mysum = 0;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		a = substr(p,q);
		av = bx_eval(s,a,0);
		if (!av) {
			free(a);
			return 0;
		}
		mysum += 1.0/bv_getDouble(av);
		free(a);
		bv_cfreeBv(av);
		p = bx_ffToNextParameter(p);
	}
	return bv_doubleToBv(1.0/mysum);
}

b_varl * bf_ceil1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( ceil(bv_getDouble(a)) );
}

b_varl * bf_floor1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( floor(bv_getDouble(a)) );
}

b_varl * bf_round1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( round(bv_getDouble(a)) );
}

b_varl * bf_trunc1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( trunc(bv_getDouble(a)) );
}

b_varl * bf_abs1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( fabs(bv_getDouble(a)) );
}

b_varl * bf_sgn1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	double d = bv_getDouble(a);
	return bv_intToBv( ((d<0)?(-1):((d>0)?(1):(0))) );
}

b_varl * bf_sqrt1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( sqrt(bv_getDouble(a)) );
}

b_varl * bf_cbrt1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( cbrt(bv_getDouble(a)) );
}

b_varl * bf_ln1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( log(bv_getDouble(a)) );
}

b_varl * bf_log1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( log10(bv_getDouble(a)) );
}

b_varl * bf_log2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_doubleToBv( log(bv_getDouble(a))/log(bv_getDouble(b)) );
}

b_varl * bf_exp1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( exp(bv_getDouble(a)) );
}

b_varl * bf_sin1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( sin(bv_getDouble(a)) );
}

b_varl * bf_cos1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( cos(bv_getDouble(a)) );
}

b_varl * bf_tan1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( tan(bv_getDouble(a)) );
}

b_varl * bf_cot1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( cot(bv_getDouble(a)) );
}

b_varl * bf_sec1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( sec(bv_getDouble(a)) );
}

b_varl * bf_csc1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( csc(bv_getDouble(a)) );
}

b_varl * bf_asin1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( asin(bv_getDouble(a)) );
}

b_varl * bf_acos1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( acos(bv_getDouble(a)) );
}

b_varl * bf_atan1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( atan(bv_getDouble(a)) );
}

b_varl * bf_atan2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_doubleToBv( atan2(bv_getDouble(a), bv_getDouble(b)) );
}

b_varl * bf_acot1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( acot(bv_getDouble(a)) );
}

b_varl * bf_asec1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( asec(bv_getDouble(a)) );
}

b_varl * bf_acsc1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( acsc(bv_getDouble(a)) );
}

b_varl * bf_sinh1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( sinh(bv_getDouble(a)) );
}

b_varl * bf_cosh1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( cosh(bv_getDouble(a)) );
}

b_varl * bf_tanh1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( tanh(bv_getDouble(a)) );
}

b_varl * bf_coth1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( coth(bv_getDouble(a)) );
}

b_varl * bf_sech1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( sech(bv_getDouble(a)) );
}

b_varl * bf_csch1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( csch(bv_getDouble(a)) );
}

b_varl * bf_asinh1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( asinh(bv_getDouble(a)) );
}

b_varl * bf_acosh1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( acosh(bv_getDouble(a)) );
}

b_varl * bf_atanh1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( atanh(bv_getDouble(a)) );
}

b_varl * bf_acoth1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( acoth(bv_getDouble(a)) );
}

b_varl * bf_asech1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( asech(bv_getDouble(a)) );
}

b_varl * bf_acsch1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( acsch(bv_getDouble(a)) );
}

b_varl * bf_gamma1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( gamma(bv_getDouble(a)) );
}

b_varl * bf_pick2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_doubleToBv( pick(bv_getInt(a),bv_getInt(b)) );
}

b_varl * bf_choose2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_doubleToBv( choose(bv_getInt(a),bv_getInt(b)) );
}

b_varl * bf_gcd2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv( gcdl(bv_getInt(a),bv_getInt(b)) );
}

b_varl * bf_lcm2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv( lcml(bv_getInt(a),bv_getInt(b)) );
}

void bf_randomize1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return; }
	srand(bv_getInt(a));
}

b_varl * bf_rnd0(b_scope * sc)
{
	return bv_doubleToBv( (float)rand() / (float)RAND_MAX );
}

b_varl * bf_rnd1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv( (float)rand() / (float)RAND_MAX * bv_getDouble(a) );
}

b_varl * bf_rnd2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_doubleToBv( bv_getDouble(a) + (float)rand() / (float)RAND_MAX * (bv_getDouble(b) - bv_getDouble(a)) );
}

