/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "btype.h"
#include "vars.h"
#include "exceptions.h"
#include "memx.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

b_varl * bf_peek1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv(peek(bv_getInt(a)));
}

b_varl * bf_peek2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	switch (bv_getInt(b)) {
		case 1: return bv_intToBv(peek(bv_getInt(a))); break;
		case 2: return bv_intToBv(peekw(bv_getInt(a))); break;
		case 4: return bv_intToBv(peeki(bv_getInt(a))); break;
		case 8: return bv_intToBv(peekl(bv_getInt(a))); break;
		default: be_throw(be_range); return 0; break;
	}
}

b_varl * bf_peekf1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_doubleToBv(peekf(bv_getInt(a)));
}

b_varl * bf_peekf2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	switch (bv_getInt(b)) {
		case 4: return bv_doubleToBv(peekf(bv_getInt(a))); break;
		case 8: return bv_doubleToBv(peekd(bv_getInt(a))); break;
		default: be_throw(be_range); return 0; break;
	}
}

b_varl * bf_peeks1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_stringToBv((char *)bv_getInt(a));
}

b_varl * bf_peeks2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	int n = bv_getInt(b);
	char * tmp = (char *)malloc(n+1);
	b_varl * v;
	if (!tmp) { be_throw(be_memory); return 0; } else {
		strncpy(tmp, (char *)bv_getInt(a), n);
		tmp[n] = 0;
		v = bv_stringToBv(tmp);
		free(tmp);
		return v;
	}
}

void bf_poke2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_throw(be_cast); return; }
	poke(bv_getInt(a), bv_getInt(b));
}

void bf_poke3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_throw(be_cast); return; }
	switch (bv_getInt(c)) {
		case 1: poke(bv_getInt(a), bv_getInt(b)); break;
		case 2: pokew(bv_getInt(a), bv_getInt(b)); break;
		case 4: pokei(bv_getInt(a), bv_getInt(b)); break;
		case 8: pokel(bv_getInt(a), bv_getInt(b)); break;
		default: be_throw(be_range); break;
	}
}

void bf_pokef2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_throw(be_cast); return; }
	pokef(bv_getInt(a), bv_getDouble(b));
}

void bf_pokef3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_throw(be_cast); return; }
	switch (bv_getInt(c)) {
		case 4: pokef(bv_getInt(a), bv_getDouble(b)); break;
		case 8: poked(bv_getInt(a), bv_getDouble(b)); break;
		default: be_throw(be_range); break;
	}
}

void bf_pokes2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertString(b) )) { be_throw(be_cast); return; }
	strcpy((char *)bv_getInt(a), bv_getString(b));
}

void bf_pokes3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertString(b) && bv_assertNumeric(c) )) { be_throw(be_cast); return; }
	strncpy((char *)bv_getInt(a), bv_getString(b), bv_getInt(c));
}

void bf_pokem2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_throw(be_cast); return; }
	memcpy((char *)bv_getInt(a), (char *)bv_getInt(b), 1);
}

void bf_pokem3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_throw(be_cast); return; }
	memcpy((char *)bv_getInt(a), (char *)bv_getInt(b), bv_getInt(c));
}

void bf_call1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_throw(be_cast); return; }
	((void (*)())bv_getInt(a))();
}

void bf_call2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_throw(be_cast); return; }
	((void (*)(int))bv_getInt(a))((int)bv_getInt(b));
}

void bf_call3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_throw(be_cast); return; }
	((void (*)(int,int))bv_getInt(a))((int)bv_getInt(b),(int)bv_getInt(c));
}

void bf_call4(b_scope * sc, b_varl * a, b_varl * b, b_varl * c, b_varl * d)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) && bv_assertNumeric(d) )) { be_throw(be_cast); return; }
	((void (*)(int,int,int))bv_getInt(a))((int)bv_getInt(b),(int)bv_getInt(c),(int)bv_getInt(d));
}

b_varl * bf_usr1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv(  ((int (*)())bv_getInt(a))()  );
}

b_varl * bf_usr2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv(  ((int (*)(int))bv_getInt(a))((int)bv_getInt(b))  );
}

b_varl * bf_usr3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { return 0; }
	return bv_intToBv(  ((int (*)(int,int))bv_getInt(a))((int)bv_getInt(b),(int)bv_getInt(c))  );
}

b_varl * bf_usr4(b_scope * sc, b_varl * a, b_varl * b, b_varl * c, b_varl * d)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) && bv_assertNumeric(d) )) { return 0; }
	return bv_intToBv(  ((int (*)(int,int,int))bv_getInt(a))((int)bv_getInt(b),(int)bv_getInt(c),(int)bv_getInt(d))  );
}

/* uggh - what a mess */

void bf_wait2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_throw(be_cast); return; }
	int addr = bv_getInt(a);
	int and = bv_getInt(b);
	while (!( peek(addr) & and )); /* this is very dangerous, I wouldn't use it */
}

void bf_wait3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_throw(be_cast); return; }
	int addr = bv_getInt(a);
	int and = bv_getInt(b);
	int xor = bv_getInt(c);
	while (!( (peek(addr) ^ xor) & and )); /* this is very dangerous, I wouldn't use it */
}

void bf_wait4(b_scope * sc, b_varl * a, b_varl * b, b_varl * c, b_varl * d)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_throw(be_cast); return; }
	int addr = bv_getInt(a);
	int and = bv_getInt(b);
	int xor = bv_getInt(c);
	switch (bv_getInt(d)) {
		case 1: while (!( (peek(addr) ^ xor) & and )); break;
		case 2: while (!( (peekw(addr) ^ xor) & and )); break;
		case 4: while (!( (peeki(addr) ^ xor) & and )); break;
		case 8: while (!( (peekl(addr) ^ xor) & and )); break;
		default: be_throw(be_range); break;
	}
}

b_varl * bf_malloc1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv((int)malloc(bv_getInt(a)));
}

b_varl * bf_calloc1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv((int)calloc(1,bv_getInt(a)));
}

void bf_free1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_throw(be_cast); return; }
	free((void *)bv_getInt(a));
}

void bf_bsave3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_throw(be_cast); return; }
	FILE * fp = fopen(bv_getString(a),"wb");
	if (!fp) {
		be_throw(be_fileerr());
	} else {
		fwrite((void *)bv_getInt(b), 1, bv_getInt(c), fp);
		fclose(fp);
	}
}

b_varl * bf_bload1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_throw(be_cast); return 0; }
	FILE * fp = fopen(bv_getString(a),"rb");
	size_t len;
	if (!fp) {
		be_throw(be_fileerr());
		return 0;
	} else {
		void * p;
		fseek(fp, 0, SEEK_END);
		len = ftell(fp);
		fseek(fp, 0, SEEK_SET);
		p = malloc(len);
		if (!p) {
			be_throw(be_memory);
		} else {
			fread(p, 1, len, fp);
		}
		fclose(fp);
		return bv_intToBv((signed long int)p);
	}
}

b_varl * bf_bload2f(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) )) { be_throw(be_cast); return 0; }
	FILE * fp = fopen(bv_getString(a),"rb");
	size_t len = bv_getInt(b);
	if (!fp) {
		be_throw(be_fileerr());
		return 0;
	} else {
		void * p = malloc(len);
		if (!p) {
			be_throw(be_memory);
		} else {
			fread(p, 1, len, fp);
		}
		fclose(fp);
		return bv_intToBv((signed long int)p);
	}
}

void bf_bload2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) )) { be_throw(be_cast); return; }
	FILE * fp = fopen(bv_getString(a),"rb");
	size_t len;
	if (!fp) {
		be_throw(be_fileerr());
	} else {
		fseek(fp, 0, SEEK_END);
		len = ftell(fp);
		fseek(fp, 0, SEEK_SET);
		fread((void *)bv_getInt(b), 1, len, fp);
		fclose(fp);
	}
}

void bf_bload3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_throw(be_cast); return; }
	FILE * fp = fopen(bv_getString(a),"rb");
	if (!fp) {
		be_throw(be_fileerr());
	} else {
		fread((void *)bv_getInt(b), 1, bv_getInt(c), fp);
		fclose(fp);
	}
}

