/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "btype.h"
#include "vars.h"
#include "expr.h"
#include <stdio.h>
#include <string.h>
#include "stringx.h"
#include <stdlib.h>

b_varl * bf_trim1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	char * s = trim(bv_getString(a));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_ltrim1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	char * s = ltrim(bv_getString(a));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_rtrim1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	char * s = rtrim(bv_getString(a));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_left2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) )) { return 0; }
	char * s = left(bv_getString(a), bv_getInt(b));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_mid2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) )) { return 0; }
	char * s = mid(bv_getString(a), bv_getInt(b)-1);
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_mid3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { return 0; }
	char * s = mid2(bv_getString(a), bv_getInt(b)-1, bv_getInt(c));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_right2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) )) { return 0; }
	char * s = right(bv_getString(a), bv_getInt(b));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_instr2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertString(b) )) { return 0; }
	int i = instr(bv_getString(a), bv_getString(b))+1;
	b_varl * v = bv_intToBv(i);
	return v;
}

b_varl * bf_rinstr2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertString(b) )) { return 0; }
	int i = rinstr(bv_getString(a), bv_getString(b))+1;
	b_varl * v = bv_intToBv(i);
	return v;
}

b_varl * bf_ucase1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	char * s = ucase(bv_getString(a));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_lcase1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	char * s = lcase(bv_getString(a));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_tcase1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	char * s = tcase(bv_getString(a));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_len1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	int i = strlen(bv_getString(a));
	b_varl * v = bv_intToBv(i);
	return v;
}

b_varl * bf_val1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	double d = bv_getDouble(a);
	b_varl * v = bv_doubleToBv(d);
	return v;
}

b_varl * bf_eval1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	char * s = bv_getString(a);
	b_varl * v = bx_eval(sc, s, 0);
	return v;
}

b_varl * bf_str1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	char * s = bv_getString(a);
	b_varl * v = bv_stringToBv(s);
	return v;
}

b_varl * bf_asc1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	int i = asc(bv_getString(a));
	b_varl * v = bv_intToBv(i);
	return v;
}

b_varl * bf_chr1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	char * s = chr(bv_getInt(a));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_uni1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	int i = uni(bv_getString(a));
	b_varl * v = bv_intToBv(i);
	return v;
}

b_varl * bf_uchr1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	char * s = uchr(bv_getInt(a));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_spc1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	int i;
	int n = bv_getInt(a);
	char * s = malloc(n+1);
	char * sp = s;
	b_varl * v;
	if (!s) return 0;
	for (i=0; i<n; i++) {
		(*sp++) = ' ';
	}
	(*sp) = 0;
	v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_countfields2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertString(b) )) { return 0; }
	int i = countFields(bv_getString(a),bv_getString(b));
	b_varl * v = bv_intToBv(i);
	return v;
}

b_varl * bf_nthfield3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertString(a) && bv_assertString(b) && bv_assertNumeric(c) )) { return 0; }
	char * s = nthField(bv_getString(a),bv_getString(b),bv_getInt(c));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_replace3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertString(a) && bv_assertString(b) && bv_assertString(c) )) { return 0; }
	char * s = replace(bv_getString(a),bv_getString(b),bv_getString(c));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_replaceall3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertString(a) && bv_assertString(b) && bv_assertString(c) )) { return 0; }
	char * s = replaceAll(bv_getString(a),bv_getString(b),bv_getString(c));
	b_varl * v = bv_stringToBv(s);
	free(s);
	return v;
}

b_varl * bf_hex1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	char s[256];
	snprintf(s,255,"%lX",bv_getInt(a));
	b_varl * v = bv_stringToBv(s);
	return v;
}

b_varl * bf_oct1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	char s[256];
	snprintf(s,255,"%lo",bv_getInt(a));
	b_varl * v = bv_stringToBv(s);
	return v;
}

b_varl * bf_bin1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	char s[256];
	char * p = &s[255];
	unsigned long int b = bv_getInt(a);
	(*p) = 0;
	while (b) {
		p--;
		if (b & 1) { (*p) = '1'; }
		else { (*p) = '0'; }
		b >>= 1;
	}
	b_varl * v = bv_stringToBv(p);
	return v;
}

