/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "btype.h"
#include "vars.h"
#include "expr.h"
#include "exceptions.h"
#include <stdio.h>
#include "stdiox.h"
#include <string.h>
#include "stringx.h"
#include <stdlib.h>
#include <sys/ioctl.h>
#ifdef CUSTOM_FGETS
extern int CUSTOM_FGETS(char * b, int c, FILE * fp);
#endif

void bf_prints(b_scope * s, char * c)
{
	char * tmp;
	b_varl * tmpv;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	if (bx_statementDelim(*p)) printf("\n");
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		tmp = substrim(p,q);
		if (*tmp) {
			tmpv = bx_eval(s,tmp,0);
			if (!tmpv) {
				be_cthrow(be_undeclared);
			} else {
				printf("%s", bv_getString(tmpv));
			}
			bv_cfreeBv(tmpv);
		}
		free(tmp);
		switch (*q) {
			case ';': break;
			case ',': printf("\t"); break;
			case  0 : printf("\n"); break;
		}
		p = bx_ffToNextParameter(p);
	}
	fflush(stdout); /* if we don't do this we don't get output until the next newline or the program exits */
}

void bf_input1(b_scope * s, b_varl * a)
{
	if (!a) { be_cthrow(be_undeclared); return; }
	char buf[32768];
#ifndef CUSTOM_FGETS
	fgets(buf, 32760, stdin);
#else
	CUSTOM_FGETS(buf, 32760, stdin);
#endif
	buf[strlen(buf)-1] = 0;
	switch (a->type) {
		case bt_integer: {
			int i;
			sscanf(buf, "%d", &i);
			bv_setInt(a,i);
		} break;
		case bt_float: {
			double d;
			sscanf(buf, "%lf", &d);
			bv_setDouble(a,d);
		} break;
		case bt_string: {
			bv_setString(a,buf);
		} break;
		default: {
			be_throw(be_type);
		} break;
	}
}

void bf_input2(b_scope * s, b_varl * a, b_varl * b)
{
	printf("%s", bv_getString(a));
	bf_input1(s,b);
}

void bf_inputs(b_scope * s, char * c)
{
	char * tmp;
	b_varl * tmpv;
	char * p = bx_ffToFirstStatement(c);
	char * q;
	int first = 1;
	if (bx_statementDelim(*p)) {
		be_throw(be_syntax);
		return;
	}
	while (!bx_statementDelim(*p)) {
		q = bx_ffToEndOfParameter(p);
		tmp = substrim(p,q);
		if (*tmp) {
			tmpv = bx_eval(s,tmp,1);
			if (!tmpv) {
				be_cthrow(be_undeclared);
			} else {
				switch (*q) {
					case ';': printf("%s?", bv_getString(tmpv)); break;
					case ',': printf("%s", bv_getString(tmpv)); break;
					case  0 : if (first) printf("?"); bf_input1(s,tmpv); break;
				}
			}
			bv_cfreeBv(tmpv);
		}
		free(tmp);
		p = bx_ffToNextParameter(p);
		first = 0;
	}
}

void bf_get1(b_scope * s, b_varl * a)
{
	char buf[2] = {0,0};
	if (!a) {
		be_cthrow(be_undeclared);
		return;
	}
	buf[0] = tty_readchar();
	bv_setString(a,buf);
}

void bf_normal0(b_scope * s)
{
	printf("\x1B[0m");
}

void bf_bold0(b_scope * s)
{
	printf("\x1B[1m");
}

void bf_lowintensity0(b_scope * s)
{
	printf("\x1B[2m");
}

void bf_underline0(b_scope * s)
{
	printf("\x1B[4m");
}

void bf_flash0(b_scope * s)
{
	printf("\x1B[5m");
}

void bf_inverse0(b_scope * s)
{
	printf("\x1B[7m");
}

void bf_invisible0(b_scope * s)
{
	printf("\x1B[8m");
}

void bf_home0(b_scope * s)
{
	printf("\x1B[2J\x1B[H");
}

void bf_vtab1(b_scope * s, b_varl * a)
{
	if (!bv_assertNumeric(a)) { be_cthrow(be_type); return; }
	printf("\x1B[%ld;f", bv_getInt(a));
}

void bf_htab1(b_scope * s, b_varl * a)
{
	if (!bv_assertNumeric(a)) { be_cthrow(be_type); return; }
	printf("\x1B[;%ldf", bv_getInt(a));
}

void bf_gotoxy2(b_scope * s, b_varl * a, b_varl * b)
{
	if (!(bv_assertNumeric(a) && bv_assertNumeric(b))) { be_cthrow(be_type); return; }
	printf("\x1B[%ld;%ldf", bv_getInt(b), bv_getInt(a));
}

b_varl * bf_vpos0(b_scope * s)
{
	char * buf = tty_readesc("\x1B[6n");
	int v=0, h=0;
	if (buf) {
		sscanf(buf, "\x1B[%d;%dR", &v, &h);
		free(buf);
	}
	return bv_intToBv(v);
}

b_varl * bf_hpos0(b_scope * s)
{
	char * buf = tty_readesc("\x1B[6n");
	int v=0, h=0;
	if (buf) {
		sscanf(buf, "\x1B[%d;%dR", &v, &h);
		free(buf);
	}
	return bv_intToBv(h);
}

b_varl * bf_tab1(b_scope * s, b_varl * a)
{
	if (!bv_assertNumeric(a)) { return 0; }
	char * buf = tty_readesc("\x1B[6n");
	int v=0, h=0;
	if (buf) {
		sscanf(buf, "\x1B[%d;%dR", &v, &h);
		free(buf);
	}
	h = bv_getInt(a)-h;
	if (h>0) {
		char * p;
		b_varl * v;
		buf = malloc(h+1);
		p = buf;
		while (h) {
			(*p++) = ' ';
			h--;
		}
		(*p) = 0;
		v = bv_stringToBv(buf);
		free(buf);
		return v;
	} else {
		return bv_stringToBv("");
	}
}

b_varl * bf_rows0(b_scope * s)
{
	struct winsize win;
	ioctl(1, TIOCGWINSZ, &win);
	return bv_intToBv(win.ws_row);
}

b_varl * bf_cols0(b_scope * s)
{
	struct winsize win;
	ioctl(1, TIOCGWINSZ, &win);
	return bv_intToBv(win.ws_col);
}

void bf_fgcolor1(b_scope * s, b_varl * a)
{
	if (!bv_assertNumeric(a)) { be_cthrow(be_type); return; }
	printf("\x1B[3%ldm", bv_getInt(a));
}

void bf_bgcolor1(b_scope * s, b_varl * a)
{
	if (!bv_assertNumeric(a)) { be_cthrow(be_type); return; }
	printf("\x1B[4%ldm", bv_getInt(a));
}

void bf_echos(b_scope * s, char * ch)
{
	char * c = trim(ch);
	if (!strcasecmp(c,"on")) {
		tty_echo();
	} else if (!strcasecmp(c,"off")) {
		tty_noecho();
	} else {
		be_throw(be_syntax);
	}
	free(c);
}

