/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "btype.h"
#include <stdarg.h>
#include <string.h>

#include "f_basic.h"
#include "f_textio.h"
#include "f_string.h"
#include "f_math.h"
#include "f_fileio.h"
#include "f_mem.h"
#include "f_time.h"

#define MAX_COMMANDS 1024
#define MAX_FUNCTIONS 2048

static b_command bf_Commands[MAX_COMMANDS];
static b_function bf_Functions[MAX_FUNCTIONS];
static int bf_NumCommands = 0;
static int bf_NumFunctions = 0;

int bf_AddCommands(int n, ...)
{
	int i;
	b_command cmd;
	va_list ap;
	va_start(ap, n);
	for (i=0; i<n; i++) {
		if (bf_NumCommands >= MAX_COMMANDS) break;
		cmd = va_arg(ap, b_command);
		bf_Commands[bf_NumCommands++] = cmd;
	}
	va_end(ap);
	return i;
}

int bf_AddFunctions(int n, ...)
{
	int i;
	b_function fnc;
	va_list ap;
	va_start(ap, n);
	for (i=0; i<n; i++) {
		if (bf_NumFunctions >= MAX_FUNCTIONS) break;
		fnc = va_arg(ap, b_function);
		bf_Functions[bf_NumFunctions++] = fnc;
	}
	va_end(ap);
	return i;
}

int bf_CommandCount()
{
	return bf_NumCommands;
}

int bf_FunctionCount()
{
	return bf_NumFunctions;
}

const char * bf_CommandName(int n)
{
	return bf_Commands[n].name;
}

const char * bf_FunctionName(int n)
{
	return bf_Functions[n].name;
}

const b_command * bf_FindCommand(char * n)
{
	int i;
	for (i=0; i < bf_NumCommands; i++) {
		if (!strcasecmp(bf_Commands[i].name, n)) return &bf_Commands[i];
	}
	return 0;
}

const b_function * bf_FindFunction(char * n)
{
	int i;
	for (i=0; i < bf_NumFunctions; i++) {
		if (!strcasecmp(bf_Functions[i].name, n)) return &bf_Functions[i];
	}
	return 0;
}

void bf_init()
{
	bf_NumCommands = 0;
	bf_NumFunctions = 0;
	
	bf_AddCommands(15,
		bf_include, bf_chain, bf_subrun, bf_run,
		bf_newc, bf_load, bf_save, bf_list,
		bf_dim, bf_redim, bf_let, bf_const,
		bf_swap, bf_del, bf_clear
	);
	bf_AddCommands(19,
		bf__, bf_print, bf_input, bf_get,
		bf_home, bf_cls, bf_htab, bf_vtab, bf_gotoxy,
		bf_normal, bf_bold, bf_lowintensity, bf_underline, bf_flash, bf_inverse, bf_invisible,
		bf_fgcolor, bf_bgcolor,
		bf_echo
	);
	bf_AddCommands(1,
		bf_randomize
	);
	bf_AddCommands(23,
		bf_open, bf_close, bf_flush, bf_rewind, bf_seek,
		bf_write, bf_writeb, bf_writem, bf_read, bf_readb, bf_readm,
		bf_rename, bf_delete, bf_create, bf_chdir, bf_chmod, bf_chown,
		bf_cdir, bf_mkdir, bf_rmdir, bf_link, bf_symlink, bf_system
	);
	bf_AddCommands(9,
		bf_poke, bf_pokef, bf_pokes, bf_pokem, bf_free,
		bf_call, bf_wait,
		bf_bsave, bf_bload
	);
	bf_AddCommands(3,
		bf_throw,
		bf_includepath,
		bf_aboutc
	);
	
	bf_AddFunctions(6,
		bf___, bf_if_, bf_if,
		bf_ubound, bf_new, bf_clone
	);
	bf_AddFunctions(59,
		bf_ceil, bf_floor, bf_round, bf_int, bf_trunc,
		bf_abs, bf_sgn,
		bf_sqr, bf_sqrt, bf_cbrt,
		bf_ln, bf_log, bf_log10, bf_exp,
		bf_sin, bf_cos, bf_tan, bf_cot, bf_sec, bf_csc,
		bf_asn, bf_asin, bf_acs, bf_acos, bf_atn, bf_atan, bf_atantwo, bf_acot, bf_asec, bf_acsc,
		bf_sinh, bf_cosh, bf_tanh, bf_coth, bf_sech, bf_csch,
		bf_asinh, bf_acosh, bf_atanh, bf_acoth, bf_asech, bf_acsch,
		bf_max, bf_min, bf_sum, bf_avg, bf_average, bf_prod, bf_product, bf_geom, bf_geomean,
		bf_rnd, bf_gamma, bf_pick, bf_choose,
		bf_gcd, bf_lcm, bf_hypot, bf_rsr
	);
	bf_AddFunctions(7,
		bf_spc, bf_tab,
		bf_pos, bf_hpos, bf_vpos, bf_rows, bf_cols
	);
	bf_AddFunctions(45,
		bf_len,
		bf_trim_, bf_trim, bf_ltrim_, bf_ltrim, bf_rtrim_, bf_rtrim,
		bf_left_, bf_left, bf_mid_, bf_mid, bf_right_, bf_right,
		bf_instr, bf_rinstr,
		bf_ucase_, bf_ucase, bf_lcase_, bf_lcase, bf_tcase_, bf_tcase,
		bf_val, bf_eval_, bf_eval, bf_str_, bf_str,
		bf_asc, bf_chr_, bf_chr,
		bf_uni, bf_uchr_, bf_uchr,
		bf_hex_, bf_hex, bf_oct_, bf_oct, bf_bin_, bf_bin,
		bf_countfields, bf_nthfield_, bf_nthfield,
		bf_replace_, bf_replace, bf_replaceall_, bf_replaceall
	);
	bf_AddFunctions(6,
		bf_eof, bf_tell,
		bf_cwdir_, bf_cwdir, bf_dir_, bf_dir
	);
	bf_AddFunctions(7,
		bf_peek, bf_peekf, bf_peeks, bf_malloc, bf_calloc,
		bf_usr,
		bf_bloadf
	);
	bf_AddFunctions(5,
		bf_time_, bf_time, bf_gmtime_, bf_gmtime, bf_ticks
	);
	bf_AddFunctions(10,
		bf_errstr_, bf_errstr,
		bf_includepathf, bf_includepath_,
		bf_commands_, bf_commands, bf_functions_, bf_functions,
		bf_about_, bf_about
	);
}

