/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "pz.h"
#include "btype.h"
#include "vars.h"
#include "expr.h"
#include "functions.h"
#include "exceptions.h"
#include "apiglue.h"
#include "module.h"

ttk_color rgbtottk(int rgb)
{
	return ttk_makecol( (rgb >> 16) & 0xFF, (rgb >> 8) & 0xFF, (rgb) & 0xFF );
}

int ttktorgb(ttk_color ttk)
{
	int r,g,b;
	ttk_unmakecol(ttk, &r, &g, &b);
	return ( (r << 16) | (g << 8) | b );
}

int basic_checkerr()
{
	char buf[1024];
	if (be_checkerr()) {
		be_sprinterr(buf);
		be_clearerr();
		pz_error("%s",buf);
		return 1;
	} else {
		return 0;
	}
}

void bf_pzrun1(b_scope * sc, b_varl * a)
{
	if (!a) { be_throw(be_undeclared); } else if (!bv_assertString(a)) { be_throw(be_type); } else {
		new_basic_graphical_window_with(bv_getString(a));
	}
}

void bf_termrun0(b_scope * sc)
{
	new_basic_terminal_window();
}

void bf_termrun1(b_scope * sc, b_varl * a)
{
	if (!a) { be_throw(be_undeclared); } else if (!bv_assertString(a)) { be_throw(be_type); } else {
		new_basic_terminal_window_with(bv_getString(a));
	}
}

void bf_exit0(b_scope * sc)
{
	destroy_basic_graphical_window_with(sc);
}

b_varl * bf_ttkpodtype1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 0: return bv_intToBv(TTK_POD_X11);
		case 1: return bv_intToBv(TTK_POD_1G);
		case 2: return bv_intToBv(TTK_POD_2G);
		case 3: return bv_intToBv(TTK_POD_3G);
		case 4: return bv_intToBv(TTK_POD_4G);
		case 5: return bv_intToBv(TTK_POD_VIDEO);
		case 10: return bv_intToBv(TTK_POD_MINI);
		case 11: return bv_intToBv(TTK_POD_MINI_1G);
		case 12: return bv_intToBv(TTK_POD_MINI_2G);
		case 13: return bv_intToBv(TTK_POD_PHOTO);
		case 14: return bv_intToBv(TTK_POD_NANO);
		case 5002: return bv_intToBv(TTK_POD_PP5002);
		case 5020: return bv_intToBv(TTK_POD_PP5020);
		case 5022: return bv_intToBv(TTK_POD_PP5022);
		case 5029: return bv_intToBv(TTK_POD_PP502X);
	}
	return bv_intToBv(-1);
}

b_varl * bf_ttkbutton1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 1: return bv_intToBv(TTK_BUTTON_ACTION);
		case 2: return bv_intToBv(TTK_BUTTON_PREVIOUS);
		case 3: return bv_intToBv(TTK_BUTTON_NEXT);
		case 4: return bv_intToBv(TTK_BUTTON_MENU);
		case 5: return bv_intToBv(TTK_BUTTON_PLAY);
		case 6: return bv_intToBv(TTK_BUTTON_HOLD);
	}
	return bv_intToBV(-1);
}

b_varl * bf_ttkmovewherefrom1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 1: return bv_intToBv(TTK_MOVE_ABS);
		case 2: return bv_intToBv(TTK_MOVE_REL);
		case 3: return bv_intToBv(TTK_MOVE_END);
	}
	return bv_intToBv(-1);
}

b_varl * bf_ttkdirtyness1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 1: return bv_intToBv(TTK_DIRTY_HEADER);
		case 2: return bv_intToBv(TTK_DIRTY_WINDOWAREA);
		case 3: return bv_intToBv(TTK_FILTHY);
	}
	return bv_intToBv(-1);
}

b_varl * bf_ttkevreturn1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 1: return bv_intToBv(TTK_EV_CLICK);
		case 2: return bv_intToBv(TTK_EV_UNUSED);
		case 3: return bv_intToBv(TTK_EV_DONE);
	}
	return bv_intToBv(-1);
}

b_varl * bf_ttkevretcustom1(b_scope * sc, b_varl * a)
{
	return bv_intToBv(TTK_EV_RET(bv_getInt(a)));
}

b_varl * bf_ttkinputkey1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 1: return bv_intToBv(TTK_INPUT_ENTER);
		case 2: return bv_intToBv(TTK_INPUT_BKSP);
		case 3: return bv_intToBv(TTK_INPUT_LEFT);
		case 4: return bv_intToBv(TTK_INPUT_RIGHT);
		case 5: return bv_intToBv(TTK_INPUT_END);
	}
	return bv_intToBv(-1);
}

b_varl * bf_ttkbwcolor1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a) & 0x3) {
		case 1: return bv_intToBv(MAKERGB(WHITE));
		case 2: return bv_intToBv(MAKERGB(GREY));
		case 3: return bv_intToBv(MAKERGB(DKGREY));
		case 0: return bv_intToBv(MAKERGB(BLACK));
	}
	return bv_intToBv(-1);
}

void bf_ondraws(b_scope * sc, char * s)
{
	bv_sets(sc, ":: DRAW ::", bv_stringToBv(s));
}

void bf_onscrolls(b_scope * sc, char * s)
{
	bv_sets(sc, ":: SCROLL ::", bv_stringToBv(s));
}

void bf_onstaps(b_scope * sc, char * s)
{
	bv_sets(sc, ":: STAP ::", bv_stringToBv(s));
}

void bf_onbuttons(b_scope * sc, char * s)
{
	bv_sets(sc, ":: BUTTON ::", bv_stringToBv(s));
}

void bf_ondowns(b_scope * sc, char * s)
{
	bv_sets(sc, ":: DOWN ::", bv_stringToBv(s));
}

void bf_onhelds(b_scope * sc, char * s)
{
	bv_sets(sc, ":: HELD ::", bv_stringToBv(s));
}

void bf_oninputs(b_scope * sc, char * s)
{
	bv_sets(sc, ":: INPUT ::", bv_stringToBv(s));
}

void bf_onframes(b_scope * sc, char * s)
{
	bv_sets(sc, ":: FRAME ::", bv_stringToBv(s));
}

void bf_ontimers(b_scope * sc, char * s)
{
	bv_sets(sc, ":: TIMER ::", bv_stringToBv(s));
}

void bf_ondestroys(b_scope * sc, char * s)
{
	bv_sets(sc, ":: DESTROY ::", bv_stringToBv(s));
}

void bf_ontexts(b_scope * sc, char * s)
{
	bv_sets(sc, ":: TICALLBACK ::", bv_stringToBv(s));
}

void basic_draw(TWidget * wid, ttk_surface srf)
{
	b_scope * sc = (b_scope *)(wid->data);
	b_varl * en = bv_find(sc, ":: DRAW ::");
	if (en) {
		char * ens = bv_getString(en);
		if (ens[0]) {
			char buf[100];
			snprintf(buf, 100, "%ld,%ld", (signed long int)wid, (signed long int)srf);
			bx_callUsub(bv_finduscope(sc,ens),bv_findus(sc,ens),buf);
			basic_checkerr();
		}
		bv_cfreeBv(en);
	}
}

int basic_scroll(TWidget * wid, int dir)
{
	b_scope * sc = (b_scope *)(wid->data);
	b_varl * en = bv_find(sc, ":: SCROLL ::");
	b_varl * re;
	int rei = TTK_EV_UNUSED;
	if (en) {
		char * ens = bv_getString(en);
		if (ens[0]) {
			char buf[100];
			snprintf(buf, 100, "%ld,%d", (signed long int)wid, dir);
			re = bx_callUfunc(bv_finduscope(sc,ens),bv_finduf(sc,ens),buf,0);
			basic_checkerr();
			if (re) {
				rei = bv_getInt(re);
				bv_cfreeBv(re);
			}
		}
		bv_cfreeBv(en);
	}
	return rei;
}

int basic_stap(TWidget * wid, int where)
{
	b_scope * sc = (b_scope *)(wid->data);
	b_varl * en = bv_find(sc, ":: STAP ::");
	b_varl * re;
	int rei = TTK_EV_UNUSED;
	if (en) {
		char * ens = bv_getString(en);
		if (ens[0]) {
			char buf[100];
			snprintf(buf, 100, "%ld,%d", (signed long int)wid, where);
			re = bx_callUfunc(bv_finduscope(sc,ens),bv_finduf(sc,ens),buf,0);
			basic_checkerr();
			if (re) {
				rei = bv_getInt(re);
				bv_cfreeBv(re);
			}
		}
		bv_cfreeBv(en);
	}
	return rei;
}

int basic_button(TWidget * wid, int button, int time)
{
	b_scope * sc = (b_scope *)(wid->data);
	b_varl * en = bv_find(sc, ":: BUTTON ::");
	b_varl * re;
	int rei = TTK_EV_UNUSED;
	if (en) {
		char * ens = bv_getString(en);
		if (ens[0]) {
			char buf[100];
			snprintf(buf, 100, "%ld,%d,%d", (signed long int)wid, button, time);
			re = bx_callUfunc(bv_finduscope(sc,ens),bv_finduf(sc,ens),buf,0);
			basic_checkerr();
			if (re) {
				rei = bv_getInt(re);
				bv_cfreeBv(re);
			}
		}
		bv_cfreeBv(en);
	}
	return rei;
}

int basic_down(TWidget * wid, int button)
{
	b_scope * sc = (b_scope *)(wid->data);
	b_varl * en = bv_find(sc, ":: DOWN ::");
	b_varl * re;
	int rei = TTK_EV_UNUSED;
	if (en) {
		char * ens = bv_getString(en);
		if (ens[0]) {
			char buf[100];
			snprintf(buf, 100, "%ld,%d", (signed long int)wid, button);
			re = bx_callUfunc(bv_finduscope(sc,ens),bv_finduf(sc,ens),buf,0);
			basic_checkerr();
			if (re) {
				rei = bv_getInt(re);
				bv_cfreeBv(re);
			}
		}
		bv_cfreeBv(en);
	}
	return rei;
}

int basic_held(TWidget * wid, int button)
{
	b_scope * sc = (b_scope *)(wid->data);
	b_varl * en = bv_find(sc, ":: HELD ::");
	b_varl * re;
	int rei = TTK_EV_UNUSED;
	if (en) {
		char * ens = bv_getString(en);
		if (ens[0]) {
			char buf[100];
			snprintf(buf, 100, "%ld,%d", (signed long int)wid, button);
			re = bx_callUfunc(bv_finduscope(sc,ens),bv_finduf(sc,ens),buf,0);
			basic_checkerr();
			if (re) {
				rei = bv_getInt(re);
				bv_cfreeBv(re);
			}
		}
		bv_cfreeBv(en);
	}
	return rei;
}

int basic_input(TWidget * wid, int ch)
{
	b_scope * sc = (b_scope *)(wid->data);
	b_varl * en = bv_find(sc, ":: INPUT ::");
	b_varl * re;
	int rei = TTK_EV_UNUSED;
	if (en) {
		char * ens = bv_getString(en);
		if (ens[0]) {
			char buf[100];
			snprintf(buf, 100, "%ld,%d", (signed long int)wid, ch);
			re = bx_callUfunc(bv_finduscope(sc,ens),bv_finduf(sc,ens),buf,0);
			basic_checkerr();
			if (re) {
				rei = bv_getInt(re);
				bv_cfreeBv(re);
			}
		}
		bv_cfreeBv(en);
	}
	return rei;
}

int basic_frame(TWidget * wid)
{
	b_scope * sc = (b_scope *)(wid->data);
	b_varl * en = bv_find(sc, ":: FRAME ::");
	b_varl * re;
	int rei = TTK_EV_UNUSED;
	if (en) {
		char * ens = bv_getString(en);
		if (ens[0]) {
			char buf[100];
			snprintf(buf, 100, "%ld", (signed long int)wid);
			re = bx_callUfunc(bv_finduscope(sc,ens),bv_finduf(sc,ens),buf,0);
			basic_checkerr();
			if (re) {
				rei = bv_getInt(re);
				bv_cfreeBv(re);
			}
		}
		bv_cfreeBv(en);
	}
	return rei;
}

int basic_timer(TWidget * wid)
{
	b_scope * sc = (b_scope *)(wid->data);
	b_varl * en = bv_find(sc, ":: TIMER ::");
	b_varl * re;
	int rei = TTK_EV_UNUSED;
	if (en) {
		char * ens = bv_getString(en);
		if (ens[0]) {
			char buf[100];
			snprintf(buf, 100, "%ld", (signed long int)wid);
			re = bx_callUfunc(bv_finduscope(sc,ens),bv_finduf(sc,ens),buf,0);
			basic_checkerr();
			if (re) {
				rei = bv_getInt(re);
				bv_cfreeBv(re);
			}
		}
		bv_cfreeBv(en);
	}
	return rei;
}

void basic_destroy(TWidget * wid)
{
	b_scope * sc = (b_scope *)(wid->data);
	b_varl * en = bv_find(sc, ":: DESTROY ::");
	if (en) {
		char * ens = bv_getString(en);
		if (ens[0]) {
			char buf[100];
			snprintf(buf, 100, "%ld", (signed long int)wid);
			bx_callUsub(bv_finduscope(sc,ens),bv_findus(sc,ens),buf);
			basic_checkerr();
		}
		bv_cfreeBv(en);
	}
}

int basic_ti_callback(TWidget * wid, char * text)
{
	b_scope * sc = (b_scope *)( ((TiBuffer *)wid->data)->data );
	b_varl * en = bv_find(sc, ":: TICALLBACK ::");
	b_varl * re;
	int rei = 0;
	if (en) {
		char * ens = bv_getString(en);
		if (ens[0]) {
			char buf[100];
			b_varl * stuff = bv_stringToBv(text);
			snprintf(buf, 100, "%ld,*(%ld)", (signed long int)wid, (signed long int)&stuff);
			re = bx_callUfunc(bv_finduscope(sc,ens),bv_finduf(sc,ens),buf,0);
			basic_checkerr();
			bv_cfreeBv(stuff);
			if (re) {
				rei = bv_getInt(re);
				bv_cfreeBv(re);
			}
		}
		bv_cfreeBv(en);
	}
	return rei;
}

void basic_widget_init(TWidget * wid, b_scope * sc)
{
	wid->data = (void *)sc;
	wid->draw = basic_draw;
	wid->scroll = basic_scroll;
	wid->stap = basic_stap;
	wid->button = basic_button;
	wid->down = basic_down;
	wid->held = basic_held;
	wid->input = basic_input;
	wid->frame = basic_frame;
	wid->timer = basic_timer;
	wid->destroy = basic_destroy;
}

void basic_tiwidget_init(TWidget * wid, b_scope * sc)
{
	((TiBuffer *)wid->data)->data = (void *)sc;
	((TiBuffer *)wid->data)->callback = basic_ti_callback;
}

void bf_init_apiglue()
{
	bf_AddCommands(14, bf_pzrun, bf_termrun, bf_exit, bf_ondraw, bf_onscroll, bf_onstap, bf_onbutton, bf_ondown, bf_onheld, bf_oninput, bf_onframe, bf_ontimer, bf_ondestroy, bf_ontext);
	bf_AddFunctions(8, bf_ttkpodtype, bf_ttkbutton, bf_ttkmovewherefrom, bf_ttkdirtyness, bf_ttkevreturn, bf_ttkevretcustom, bf_ttkinputkey, bf_ttkbwcolor);
}

