/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef NO_LIBIPOD
#include "ipod.h"
#endif
#ifndef NO_PZ2TTK
#include "pz.h"
#include "ttk.h"
#endif

#include <string.h>
#include "btype.h"
#include "vars.h"
#include "exceptions.h"
#include "functions.h"

#include "f_ipod.h"
#ifndef NO_PZ2TTK
#include "apiglue.h"
#include "f_pz2.h"
#include "f_ttk.h"
#endif

static char * fontNames[] = {
	"Fixed 6x13",
	"Fixed 6x12",
	"Fixed 6x10",
	"Fixed 6x9",
	"Fixed 5x8",
	"Fixed 5x7",
	"Fixed 4x6",
	"Fixed 7x13",
	"Fixed 7x14",
	"Fixed 8x13",
	"Fixed 9x15",
	"Fixed 9x18",
	"Fixed 10x20",
	"Unifont",
	"Unifont",
	"Unifont",
	"Apple II 40 Column",
	"Apple II 80 Column",
	"Topaz",
	"Topaz Doubled",
	"Topaz Sans",
	"Topaz Sans Doubled",
	"Opal",
	"Opal Double",
	"Diamond",
	"Onyx",
	"Peridot",
	"Sabine Doscbthm",
	"Sabine Doscbthm Black",
	"Sabine Doscbthm Script",
	"Sabine Doscbthm Serif",
	"Sabine Doscbthm Smallcaps",
	0
};

void bf_backlight0(b_scope * s)
{
#ifndef NO_LIBIPOD
	ipod_set_backlight(!ipod_get_backlight());
#endif
}

void bf_backlight1(b_scope * s, b_varl * a)
{
#ifndef NO_LIBIPOD
	if (bv_assertNumeric(a)) {
		ipod_set_backlight(bv_getInt(a));
	} else if (bv_assertString(a)) {
		char * b = bv_getString(a);
		if (strcasecmp(b, "on") == 0) { ipod_set_backlight(1); }
		else if (strcasecmp(b, "off") == 0) { ipod_set_backlight(0); }
		else if (strcasecmp(b, "toggle") == 0) { ipod_set_backlight(!ipod_get_backlight()); }
		else { be_cthrow(be_syntax); }
	} else {
		be_cthrow(be_syntax);
		return;
	}
#endif
}

b_varl * bf_backlightf0(b_scope * s)
{
#ifndef NO_LIBIPOD
	return bv_intToBv(ipod_get_backlight());
#else
	return bv_intToBv(0);
#endif
}

void bf_contrast1(b_scope * s, b_varl * a)
{
#ifndef NO_LIBIPOD
	if (!bv_assertNumeric(a)) { be_cthrow(be_syntax); return; }
	ipod_set_contrast(bv_getInt(a));
#endif
}

b_varl * bf_contrastf0(b_scope * s)
{
#ifndef NO_LIBIPOD
	return bv_intToBv(ipod_get_contrast());
#else
	return bv_intToBv(0);
#endif
}

b_varl * bf_fonts0(b_scope * s)
{
	int i=0;
	while (fontNames[i]) i++;
	return bv_intToBv(i);
}

b_varl * bf_fonts1(b_scope * s, b_varl * a)
{
	if (bv_assertNumeric(a)) {
		int i = 0;
		int j = bv_getInt(a);
		while (fontNames[i]) i++;
		if (j<0 || j>=i) return bv_stringToBv("");
		else return bv_stringToBv(fontNames[j]);
	} else if (bv_assertString(a)) {
		char * b = bv_getString(a);
		int i = 0;
		while (fontNames[i]) {
			if (strcasecmp(fontNames[i],b) == 0)
				return bv_intToBv(i);
			i++;
		}
		return bv_intToBv(-1);
	} else {
		return 0;
	}
}

b_varl * bf_fonts2(b_scope * s, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	int i = 0;
	char fn[50];
	snprintf(fn, 50, "Fixed %ldx%ld", bv_getInt(a), bv_getInt(b));
	while (fontNames[i]) {
		if (!strcasecmp(fontNames[i],fn))
			return bv_intToBv(i);
		i++;
	}
	return bv_intToBv(-1);
}

void bf_font1(b_scope * s, b_varl * a)
{
	int which = -1;
	if (bv_assertNumeric(a)) {
		which = bv_getInt(a);
	} else if (bv_assertString(a)) {
		char * b = bv_getString(a);
		int i = 0;
		while (fontNames[i]) {
			if (strcasecmp(fontNames[i],b) == 0) {
				which = i;
				break;
			}
			i++;
		}
	} else {
		be_cthrow(be_syntax);
		return;
	}
	if (which >= 0) printf("\x1B[%dF", which);
}

void bf_font2(b_scope * s, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_syntax); return; }
	int i = 0;
	int which = -1;
	char fn[50];
	snprintf(fn, 50, "Fixed %ldx%ld", bv_getInt(a), bv_getInt(b));
	while (fontNames[i]) {
		if (!strcasecmp(fontNames[i],fn)) {
			which = i;
			break;
		}
		i++;
	}
	if (which >= 0) printf("\x1B[%dF", which);
}

void bf_init_ipod()
{
	bf_AddCommands(3, bf_backlight, bf_contrast, bf_font);
	bf_AddFunctions(4, bf_backlightf, bf_contrastf, bf_fonts_, bf_fonts);
#ifndef NO_PZ2TTK
	bf_init_pz2();
	bf_init_ttk();
	bf_init_apiglue();
#endif
}

