/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "pz.h"
#include "btype.h"
#include "vars.h"
#include "functions.h"
#include "exceptions.h"
#include "apiglue.h"
#include "module.h"
#include "f_pz2.h"

extern TApItem * ti_ap_getx(int i);
extern ttk_color ti_ap_get(int i);
extern TWidget * ti_create_tim_widget(int ht, int wd);
extern int ti_supports_method(int id);
extern int ti_select(int id);
extern int ti_current_method();
extern int ti_use_method(int id);
extern int ti_unuse_method();
extern TWindow * ti_ttkselect(ttk_menu_item * item);
extern int ti_register(TWidget * (* create)(), TWidget * (* ncreate)(), char * name, int id);

extern TiBuffer * ti_create_buffer(int size);
extern void ti_destroy_buffer(TiBuffer * buf);
extern int ti_expand_buffer(TiBuffer * buf, int exp);
extern int ti_set_buffer(TiBuffer * buf, char * s);
extern TiBuffer * ti_create_buffer_with(char * s);
extern char * ti_get_buffer(TiBuffer * buf);
extern void ti_buffer_insert_byte(TiBuffer * buf, int pos, char ch);
extern void ti_buffer_remove_byte(TiBuffer * buf, int pos);
extern void ti_buffer_insert_char(TiBuffer * buf, int pos, int ch);
extern void ti_buffer_remove_char(TiBuffer * buf, int pos);
extern void ti_buffer_cmove(TiBuffer * buf, int m);
extern void ti_buffer_cset(TiBuffer * buf, int c);
extern int ti_buffer_cget(TiBuffer * buf);
extern void ti_buffer_input(TiBuffer * buf, int ch);
extern void ti_multiline_text(ttk_surface srf, ttk_font fnt, int x, int y, int w, int h, ttk_color col, const char *t, int cursorpos, int scroll, int * lc, int * sl, int * cb);
extern TWidget * ti_new_text_widget(int x, int y, int w, int h, int absheight, char * dt, int (*callback)(TWidget *, char *),
	void (*draw)(TWidget *, ttk_surface), int (*input)(TWidget *, int), int numeric);
extern TWidget * ti_new_standard_text_widget(int x, int y, int w, int h, int absheight, char * dt, int (*callback)(TWidget *, char *));
extern TWidget * ti_new_numeric_text_widget(int x, int y, int w, int h, int absheight, char * dt, int (*callback)(TWidget *, char *));
extern TWidget * ti_new_password_text_widget(int x, int y, int w, int h, int absheight, char * dt, int (*callback)(TWidget *, char *));
extern TWidget * ti_new_multiline_text_widget(int x, int y, int w, int h, int absheight, char * dt, int (*callback)(TWidget *, char *));

b_varl * bf_pzsettingtype1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 1: return bv_intToBv(PZ_SETTING_INT);
		case 2: return bv_intToBv(PZ_SETTING_STRING);
		case 3: return bv_intToBv(PZ_SETTING_FLOAT);
		case 4: return bv_intToBv(PZ_SETTING_ILIST);
		case 5: return bv_intToBv(PZ_SETTING_SLIST);
		case 6: return bv_intToBv(PZ_SETTING_BLOB);
	}
	return bv_intToBv(-1);
}

b_varl * bf_pzmenuaction1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 1: return bv_intToBv(PZ_MENU_DONOTHING);
		case 2: return bv_intToBv(PZ_MENU_UPONE);
		case 3: return bv_intToBv(PZ_MENU_UPALL);
		case 4: return bv_intToBv(PZ_MENU_ALREADYDONE);
		case 5: return bv_intToBv(PZ_MENU_QUIT);
		case 6: return bv_intToBv(PZ_MENU_REPLACE);
		case 7: return bv_intToBv(PZ_MENU_DESC_MAX);
	}
	return bv_intToBv(-1);
}

b_varl * bf_pzeventtype1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 1: return bv_intToBv(PZ_EVENT_SCROLL);
		case 2: return bv_intToBv(PZ_EVENT_STAP);
		case 3: return bv_intToBv(PZ_EVENT_BUTTON_UP);
		case 4: return bv_intToBv(PZ_EVENT_BUTTON_DOWN);
		case 5: return bv_intToBv(PZ_EVENT_BUTTON_HELD);
		case 6: return bv_intToBv(PZ_EVENT_INPUT);
		case 7: return bv_intToBv(PZ_EVENT_FRAME);
		case 8: return bv_intToBv(PZ_EVENT_TIMER);
		case 9: return bv_intToBv(PZ_EVENT_DESTROY);
	}
	return bv_intToBv(-1);
}

b_varl * bf_pzbutton1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 1: return bv_intToBv(PZ_BUTTON_MENU);
		case 2: return bv_intToBv(PZ_BUTTON_PREVIOUS);
		case 3: return bv_intToBv(PZ_BUTTON_NEXT);
		case 4: return bv_intToBv(PZ_BUTTON_PLAY);
		case 5: return bv_intToBv(PZ_BUTTON_HOLD);
		case 6: return bv_intToBv(PZ_BUTTON_ACTION);
	}
	return bv_intToBv(-1);
}

b_varl * bf_pzwindowgeom1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 1: return bv_intToBv(PZ_WINDOW_NORMAL);
		case 2: return bv_intToBv(PZ_WINDOW_FULLSCREEN);
		case 3: return bv_intToBv(PZ_WINDOW_POPUP);
		case 4: return bv_intToBv(PZ_WINDOW_XYWH);
	}
	return bv_intToBv(-1);
}

b_varl * bf_pzdspsource1(b_scope * sc, b_varl * a)
{
	switch (bv_getInt(a)) {
		case 1: return bv_intToBv(PZ_DSP_LINEIN);
		case 2: return bv_intToBv(PZ_DSP_LINEOUT);
		case 3: return bv_intToBv(PZ_DSP_MIC);
		case 4: return bv_intToBv(PZ_DSP_MAX_VOL);
	}
	return bv_intToBv(-1);
}

b_varl * bf_pz_basic_module0(b_scope * sc)
{
	return bf_intToBv((signed long int)basic_get_pzmodule());
}

b_varl * bf_pz_register_module1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	return bf_intToBv((signed long int)pz_register_module(bv_getString(a), 0));
}

b_varl * bf_pz_register_module2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) )) { return 0; }
	return bf_intToBv((signed long int)pz_register_module(bv_getString(a), (void (*)())bv_getInt(b)));
}

b_varl * bf_pz_module_get_cfgpath1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	return bf_stringToBv(pz_module_get_cfgpath(basic_get_pzmodule(), bv_getString(a)));
}

b_varl * bf_pz_module_get_cfgpath2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertString(b) )) { return 0; }
	return bf_stringToBv(pz_module_get_cfgpath((PzModule *)bv_getInt(a), bv_getString(b)));
}

b_varl * bf_pz_module_get_datapath1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	return bf_stringToBv(pz_module_get_datapath(basic_get_pzmodule(), bv_getString(a)));
}

b_varl * bf_pz_module_get_datapath2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertString(b) )) { return 0; }
	return bf_stringToBv(pz_module_get_datapath((PzModule *)bv_getInt(a), bv_getString(b)));
}

b_varl * bf_pz_module_softdep2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertString(b) )) { return 0; }
	return bv_intToBv((signed long int)pz_module_softdep(bv_getString(a), bv_getString(b)));
}

b_varl * bf_pz_load_config1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	return bf_intToBv((signed long int)pz_load_config(bv_getString(a)));
}

void bf_pz_save_config1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_save_config((PzConfig *)bv_getInt(a));
}

void bf_pz_blast_config1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_blast_config((PzConfig *)bv_getInt(a));
}

void bf_pz_free_config1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_free_config((PzConfig *)bv_getInt(a));
}

b_varl * bf_pz_get_setting2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv((signed long int)pz_get_setting((PzConfig *)bv_getInt(a), bv_getInt(b)));
}

b_varl * bf_pz_get_int_setting2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv(pz_get_int_setting((PzConfig *)bv_getInt(a), bv_getInt(b)));
}

b_varl * bf_pz_get_string_setting2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_stringToBv(pz_get_string_setting((PzConfig *)bv_getInt(a), bv_getInt(b)));
}

void bf_pz_set_int_setting3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_cthrow(be_cast); return; }
	pz_set_int_setting((PzConfig *)bv_getInt(a), bv_getInt(b), bv_getInt(c));
}

void bf_pz_set_string_setting3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertString(c) )) { be_cthrow(be_cast); return; }
	pz_set_int_setting((PzConfig *)bv_getInt(a), bv_getInt(b), bv_getString(c));
}

void bf_pz_set_float_setting3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_cthrow(be_cast); return; }
	pz_set_int_setting((PzConfig *)bv_getInt(a), bv_getInt(b), bv_getDouble(c));
}

void bf_pz_set_ilist_setting4(b_scope * sc, b_varl * a, b_varl * b, b_varl * c, b_varl * d)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) && bv_assertNumeric(d) )) { be_cthrow(be_cast); return; }
	pz_set_ilist_setting((PzConfig *)bv_getInt(a), bv_getInt(b), (int *)bv_getInt(c), bv_getInt(d));
}

void bf_pz_set_slist_setting4(b_scope * sc, b_varl * a, b_varl * b, b_varl * c, b_varl * d)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) && bv_assertNumeric(d) )) { be_cthrow(be_cast); return; }
	pz_set_slist_setting((PzConfig *)bv_getInt(a), bv_getInt(b), (char **)bv_getInt(c), bv_getInt(d));
}

void bf_pz_set_blob_setting4(b_scope * sc, b_varl * a, b_varl * b, b_varl * c, b_varl * d)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) && bv_assertNumeric(d) )) { be_cthrow(be_cast); return; }
	pz_set_blob_setting((PzConfig *)bv_getInt(a), bv_getInt(b), (void *)bv_getInt(c), bv_getInt(d));
}

void bf_pz_unset_setting2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	pz_unset_setting((PzConfig *)bv_getInt(a), bv_getInt(b));
}

b_varl * bf_pz_new_menu_window1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv((signed long int)pz_new_menu_window((TWidget *)bv_getInt(a)));
}

b_varl * bf_pz_get_menu_item1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	return bv_intToBv((signed long int)pz_get_menu_item(bv_getString(a)));
}

b_varl * bf_pz_mh_sub1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv((signed long int)pz_mh_sub((ttk_menu_item *)bv_getInt(a)));
}

b_varl * bf_pz_menu_add_ttkh3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { return 0; }
	return bv_intToBv((signed long int)pz_menu_add_ttkh(bv_getString(a), (TWindow *(*)())bv_getInt(b), (void *)bv_getInt(c)));
}

b_varl * bf_pz_menu_add_stub1(b_scope * sc)
{
	if (!( bv_assertString(a) )) { return 0; }
	return bv_intToBv((signed long int)pz_menu_add_stub(bv_getString(a)));
}

b_varl * bf_pz_menu_add_after3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) && bv_assertString(c) )) { return 0; }
	return bv_intToBv((signed long int)pz_menu_add_after(bv_getString(a), (TWindow *(*)())bv_getInt(b), bv_getString(c)));
}

b_varl * bf_pz_menu_add_top2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv((signed long int)pz_menu_add_top(bv_getString(a), (TWindow *(*)())bv_getInt(b)));
}

b_varl * bf_pz_menu_add_action2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv((signed long int)pz_menu_add_action(bv_getString(a), (TWindow *(*)())bv_getInt(b)));
}

b_varl * bf_pz_menu_add_option2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv((signed long int)pz_menu_add_option(bv_getString(a), (char **)bv_getInt(b)));
}

b_varl * bf_pz_menu_get_option1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	return bv_intToBv(pz_menu_get_option(bv_getString(a)));
}

void bf_pz_menu_set_option2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	pz_menu_set_option(bv_getString(a), bv_getInt(b));
}

b_varl * bf_pz_menu_add_setting4(b_scope * sc, b_varl * a, b_varl * b, b_varl * c, b_varl * d)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b), bv_assertNumeric(c), bv_assertNumeric(d) )) { return 0; }
	return bv_intToBv((signed long int)pz_menu_add_setting(bv_getString(a), bv_getInt(b), (PzConfig *)bv_getInt(c), (char **)bv_getInt(d)));
}

void bf_pz_menu_sort1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_cthrow(be_cast); }
	pz_menu_sort(bv_getString(a));
}

void bf_pz_menu_remove1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_cthrow(be_cast); }
	pz_menu_remove(bv_getString(a));
}

b_varl * bf_pz_new_window2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv((signed long int)pz_new_window(bv_getString(a), bv_getInt(b)));
}

b_varl * bf_pz_new_window3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { return 0; }
	return bv_intToBv((signed long int)pz_new_window(bv_getString(a), PZ_WINDOW_XYWH, bv_getInt(b)>>16, bv_getInt(b) & 0xFFFF, bv_getInt(c)>>16, bv_getInt(c) & 0xFFFF));
}

b_varl * bf_pz_finish_window1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv((signed long int)pz_finish_window(bv_getInt(a)));
}

void bf_pz_resize_widget3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_cthrow(be_cast); return; }
	pz_resize_widget((PzWidget *)bv_getInt(a), bv_getInt(b), bv_getInt(c));
}

void bf_pz_widget_set_timer2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	pz_widget_set_timer((PzWidget *)bv_getInt(a), bv_getInt(b));
}

void bf_pz_hide_window1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_hide_window((PzWindow *)bv_getInt(a));
}

void bf_pz_close_window1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_close_window((PzWindow *)bv_getInt(a));
}

void bf_pz_show_window1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_show_window((PzWindow *)bv_getInt(a));
}

void bf_pz_hwid_put_left1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_hwid_put_left((TWidget *)bv_getInt(a));
}

void bf_pz_hwid_pack_left1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_hwid_pack_left((TWidget *)bv_getInt(a));
}

void bf_pz_hwid_pack_right1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_hwid_pack_right((TWidget *)bv_getInt(a));
}

void bf_pz_hwid_unpack1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_hwid_unpack((TWidget *)bv_getInt(a));
}

void bf_pz_header_set_local2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	pz_header_set_local((TWidget *)bv_getInt(a), (TWidget *)bv_getInt(b));
}

void bf_pz_header_unset_local0(b_scope * sc)
{
	pz_header_unset_local();
}

b_varl * bf_pz_do_dialogv(b_scope * sc, b_varl ** argv, int argc)
{
	if (!( argc == 7 && bv_assertString(argv[0]) && bv_assertString(argv[1]) && bv_assertString(argv[2])
	 && bv_assertString(argv[3]) && bv_assertString(argv[4]) && bv_assertNumeric(argv[5]) && bv_assertNumeric(argv[6]) )) { return 0; }
	return bv_intToBv(pz_do_dialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getString(argv[2]),bv_getString(argv[3]),bv_getString(argv[4]),bv_getInt(argv[5]),bv_getInt(argv[6])));
}

b_varl * bf_pz_dialogv(b_scope * sc, b_varl ** argv, int argc)
{
	if (!( argc >= 4 && bv_assertString(argv[0]) && bv_assertString(argv[1]) && bv_assertNumeric(argv[2]) && bv_assertNumeric(argv[3]) )) { return 0; }
	switch (argc) {
	case 4:
		return bv_intToBv(pz_dialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3])));
		break;
	case 5:
		return bv_intToBv(pz_dialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4])));
		break;
	case 6:
		return bv_intToBv(pz_dialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4]),bv_getString(argv[5])));
		break;
	case 7:
		return bv_intToBv(pz_dialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4]),bv_getString(argv[5]),bv_getString(argv[6])));
		break;
	case 8:
		return bv_intToBv(pz_dialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4]),bv_getString(argv[5]),bv_getString(argv[6]),bv_getString(argv[7])));
		break;
	case 9:
		return bv_intToBv(pz_dialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4]),bv_getString(argv[5]),bv_getString(argv[6]),bv_getString(argv[7]),bv_getString(argv[8])));
		break;
	case 10:
		return bv_intToBv(pz_dialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4]),bv_getString(argv[5]),bv_getString(argv[6]),bv_getString(argv[7]),bv_getString(argv[8]),bv_getString(argv[9])));
		break;
	}
	return 0;
}

b_varl * bf_pz_errdialogv(b_scope * sc, b_varl ** argv, int argc)
{
	if (!( argc >= 4 && bv_assertString(argv[0]) && bv_assertString(argv[1]) && bv_assertNumeric(argv[2]) && bv_assertNumeric(argv[3]) )) { return 0; }
	switch (argc) {
	case 4:
		return bv_intToBv(pz_errdialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3])));
		break;
	case 5:
		return bv_intToBv(pz_errdialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4])));
		break;
	case 6:
		return bv_intToBv(pz_errdialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4]),bv_getString(argv[5])));
		break;
	case 7:
		return bv_intToBv(pz_errdialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4]),bv_getString(argv[5]),bv_getString(argv[6])));
		break;
	case 8:
		return bv_intToBv(pz_errdialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4]),bv_getString(argv[5]),bv_getString(argv[6]),bv_getString(argv[7])));
		break;
	case 9:
		return bv_intToBv(pz_errdialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4]),bv_getString(argv[5]),bv_getString(argv[6]),bv_getString(argv[7]),bv_getString(argv[8])));
		break;
	case 10:
		return bv_intToBv(pz_errdialog(bv_getString(argv[0]),bv_getString(argv[1]),bv_getInt(argv[2]),bv_getInt(argv[3]),bv_getString(argv[4]),bv_getString(argv[5]),bv_getString(argv[6]),bv_getString(argv[7]),bv_getString(argv[8]),bv_getString(argv[9])));
		break;
	}
	return 0;
}

void bf_pz_message_title2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertString(a) && bv_assertString(b) )) { be_cthrow(be_cast); return; }
	pz_message_title(bv_getString(a), bv_getString(b));
}

void bf_pz_message1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_cthrow(be_cast); return; }
	pz_message(bv_getString(a));
}

void bf_pz_warning1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_cthrow(be_cast); return; }
	pz_warning("%s", bv_getString(a));
}

void bf_pz_error1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_cthrow(be_cast); return; }
	pz_error("%s", bv_getString(a));
}

void bf_pz_perror1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { be_cthrow(be_cast); return; }
	pz_perror(bv_getString(a));
}

void bf_pz_vector_stringv(b_scope * sc, b_varl ** argv, int argc)
{
	if (!( argc == 8 && assertNumeric(argv[0]) && assertString(argv[1]) && assertNumeric(argv[2]) && assertNumeric(argv[3])
	 && assertNumeric(argv[4]) && assertNumeric(argv[5]) && assertNumeric(argv[6]) && assertNumeric(argv[7]) )) { be_cthrow(be_cast); return; }
	pz_vector_string((ttk_surface)bv_getInt(argv[0]), bv_getString(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]),
	 bv_getInt(argv[4]), bv_getInt(argv[5]), bv_getInt(argv[6]), rgbtottk(bv_getInt(argv[7])));
}

void bf_pz_vector_string_centerv(b_scope * sc, b_varl ** argv, int argc)
{
	if (!( argc == 8 && assertNumeric(argv[0]) && assertString(argv[1]) && assertNumeric(argv[2]) && assertNumeric(argv[3])
	 && assertNumeric(argv[4]) && assertNumeric(argv[5]) && assertNumeric(argv[6]) && assertNumeric(argv[7]) )) { be_cthrow(be_cast); return; }
	pz_vector_string_center((ttk_surface)bv_getInt(argv[0]), bv_getString(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]),
	 bv_getInt(argv[4]), bv_getInt(argv[5]), bv_getInt(argv[6]), rgbtottk(bv_getInt(argv[7])));
}

b_varl * bf_pz_vector_width4(b_scope * sc, b_varl * a, b_varl * b, b_varl * c, b_varl * d)
{
	if (!( bv_assertString(a) && bv_assertNumeric(b) && bv_assertNumeric(c) && bv_assertNumeric(d) )) { return 0; }
	return bv_intToBv(pz_vector_width(bv_getString(a),bv_getInt(b),bv_getInt(c),bv_getInt(d)));
}

void bf_pz_ipod_set2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	pz_ipod_set(bv_getInt(a), bv_getInt(b));
}

void bf_pz_ipod_fix_setting2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	pz_ipod_fix_setting(bv_getInt(a), bv_getInt(b));
}

void bf_pz_ipod_fix_settings1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_ipod_fix_settings((PzConfig *)bv_getInt(a));
}

b_varl * bf_pz_ipod_get_battery_level0(b_scope * sc)
{
	return bv_intToBv(pz_ipod_get_battery_level());
}

b_varl * bf_pz_ipod_is_charging0(b_scope * sc)
{
	return bv_intToBv(pz_ipod_is_charging());
}

b_varl * bf_pz_ipod_get_hw_version0(b_scope * sc)
{
	return bv_intToBv(pz_ipod_get_hw_version());
}

void bf_pz_ipod_go_to_diskmode0(b_scope * sc)
{
	pz_ipod_go_to_diskmode();
}

void bf_pz_ipod_reboot0(b_scope * sc)
{
	pz_ipod_reboot();
}

void bf_pz_ipod_powerdown0(b_scope * sc)
{
	pz_ipod_powerdown();
}

b_varl * bf_pz_ipod_usb_is_connected0(b_scope * sc)
{
	return bv_intToBv(pz_ipod_usb_is_connected());
}

b_varl * bf_pz_ipod_fw_is_connected0(b_scope * sc)
{
	return bv_intToBv(pz_ipod_fw_is_connected());
}

void bf_pz_load_font4(b_scope * sc, b_varl * a, b_varl * b, b_varl * c, b_varl * d)
{
	if (!( bv_assertNumeric(a) && bv_assertString(b) && bv_assertNumeric(c) && bv_assertNumeric(d) )) { be_cthrow(be_cast); return; }
	pz_load_font((ttk_font *)bv_getInt(a), bv_getString(b), bv_getInt(c), (PzConfig *)bv_getInt(d));
}

b_varl * bf_pz_select_font1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv((signed long int)pz_select_font((ttk_menu_item *)bv_getInt(a)));
}

b_varl * bf_pz_dsp_vol_change2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv(pz_dsp_vol_change((pz_dsp_st *)bv_getInt(a), bv_getInt(b)));
}

b_varl * bf_pz_dsp_vol_up1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv(pz_dsp_vol_up((pz_dsp_st *)bv_getInt(a)));
}

b_varl * bf_pz_dsp_vol_down1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv(pz_dsp_vol_down((pz_dsp_st *)bv_getInt(a)));
}

b_varl * bf_pz_dsp_get_volume1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv(pz_dsp_get_volume((pz_dsp_st *)bv_getInt(a)));
}

b_varl * bf_pz_dsp_setup3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { return 0; }
	return bv_intToBv(pz_dsp_setup((pz_dsp_st *)bv_getInt(a), bv_getInt(b), bv_getInt(c)));
}

b_varl * bf_pz_dsp_read3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { return 0; }
	return bv_intToBv(pz_dsp_read((pz_dsp_st *)bv_getInt(a), (void *)bv_getInt(b), bv_getInt(c)));
}

void bf_pz_dsp_write3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_cthrow(be_cast); return; }
	pz_dsp_write((pz_dsp_st *)bv_getInt(a), (void *)bv_getInt(b), bv_getInt(c));
}

b_varl * bf_pz_dsp_open2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv(pz_dsp_open((pz_dsp_st *)bv_getInt(a), bv_getInt(b)));
}

void bf_pz_dsp_close1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_dsp_close((pz_dsp_st *)bv_getInt(a));
}

b_varl * bf_pz_browser_open1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	return bv_intToBv((signed long int)pz_browser_open(bv_getString(a)));
}

b_varl * bf_pz_browser_get_actions1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	return bv_intToBv((signed long int)pz_browser_get_actions(bv_getString(a)));
}

void bf_pz_browser_set_handler2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	pz_browser_set_handler((int (*)(const char *))bv_getInt(a), (TWindow *(*)())bv_getInt(b));
}

void bf_pz_browser_remove_handler1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_browser_remove_handler((int (*)(const char *))bv_getInt(a));
}

void bf_pz_browser_add_action2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	pz_browser_add_action((int (*)(const char *))bv_getInt(a), (ttk_menu_item *)bv_getInt(b));
}

void bf_pz_browser_remove_action1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_browser_remove_action((int (*)(const char *))bv_getInt(a));
}

void bf_pz_register_input_method1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_register_input_method((TWidget *(*)())bv_getInt(a));
}

void bf_pz_register_input_method_n1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_register_input_method_n((TWidget *(*)())bv_getInt(a));
}

b_varl * bf_pz_start_input0(b_scope * sc)
{
	return bv_intToBv(pz_start_input());
}

b_varl * bf_pz_start_input_n0(b_scope * sc)
{
	return bv_intToBv(pz_start_input_n());
}

void bf_pz_register_global_hold_button3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_cthrow(be_cast); return; }
	pz_register_global_hold_button(bv_getInt(a), bv_getInt(b), (void (*)())bv_getInt(c));
}

void bf_pz_register_global_unused_handler2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	pz_register_global_unused_handler(bv_getInt(a), (int (*)(int, int))bv_getInt(b));
}

void bf_pz_unregister_global_hold_button1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_unregister_global_hold_button(bv_getInt(a));
}

void bf_pz_unregister_global_unused_handler1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_unregister_global_unused_handler(bv_getInt(a));
}

void bf_pz_handled_hold1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	pz_handled_hold(bv_getInt(a));
}

b_varl * bf_ti_ap_getx1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv((signed long int)ti_ap_getx(bv_getInt(a)));
}

b_varl * bf_ti_ap_get1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv(ttktorgb(ti_ap_get(bv_getInt(a))));
}

b_varl * bf_ti_create_tim_widget2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv((signed long int)ti_create_tim_widget(bv_getInt(a),bv_getInt(b)));
}

b_varl * bf_ti_supports_method1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv(ti_supports_method(bv_getInt(a)));
}

b_varl * bf_ti_select1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv(ti_select(bv_getInt(a)));
}

b_varl * bf_ti_current_method0(b_scope * sc)
{
	return bv_intToBv(ti_current_method());
}

b_varl * bf_ti_use_method1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv(ti_use_method(bv_getInt(a)));
}

b_varl * bf_ti_unuse_method0(b_scope * sc)
{
	return bv_intToBv(ti_unuse_method());
}

b_varl * bf_ti_ttkselect1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv((signed long int)ti_ttkselect((ttk_menu_item *)bv_getInt(a)));
}

b_varl * bf_ti_register4(b_scope * sc, b_varl * a, b_varl * b, b_varl * c, b_varl * d)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertString(c) && bv_assertNumeric(d) )) { return 0; }
	return bv_intToBv(ti_register((TWidget *(*)())bv_getInt(a),(TWidget *(*)())bv_getInt(b),bv_getString(c),bv_getInt(d)));
}

b_varl * bf_ti_create_buffer0(b_scope * sc)
{
	return bv_intToBv((signed long int)ti_create_buffer(1024));
}

b_varl * bf_ti_create_buffer1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv((signed long int)ti_create_buffer(bv_getInt(a)));
}

void bf_ti_destroy_buffer1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { be_cthrow(be_cast); return; }
	ti_destroy_buffer((TiBuffer *)bv_getInt(a));
}

b_varl * bf_ti_expand_buffer2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { return 0; }
	return bv_intToBv(ti_expand_buffer((TiBuffer *)bv_getInt(a), bv_getInt(b)));
}

b_varl * bf_ti_set_buffer2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertString(b) )) { return 0; }
	return bv_intToBv(ti_set_buffer((TiBuffer *)bv_getInt(a), bv_getString(b)));
}

b_varl * bf_ti_create_buffer_with1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertString(a) )) { return 0; }
	return bv_intToBv((signed long int)ti_create_buffer_with(bv_getString(a)));
}

b_varl * bf_ti_get_buffer1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_stringToBv(ti_get_buffer((TiBuffer *)bv_getInt(a)));
}

void bf_ti_buffer_insert_byte3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_cthrow(be_cast); return; }
	ti_buffer_insert_byte((TiBuffer *)bv_getInt(a), bv_getInt(b), bv_getInt(c));
}

void bf_ti_buffer_remove_byte2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	ti_buffer_remove_byte((TiBuffer *)bv_getInt(a), bv_getInt(b));
}

void bf_ti_buffer_insert_char3(b_scope * sc, b_varl * a, b_varl * b, b_varl * c)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) && bv_assertNumeric(c) )) { be_cthrow(be_cast); return; }
	ti_buffer_insert_char((TiBuffer *)bv_getInt(a), bv_getInt(b), bv_getInt(c));
}

void bf_ti_buffer_remove_char2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	ti_buffer_remove_char((TiBuffer *)bv_getInt(a), bv_getInt(b));
}

void bf_ti_buffer_cmove2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	ti_buffer_cmove((TiBuffer *)bv_getInt(a), bv_getInt(b));
}

void bf_ti_buffer_cset2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	ti_buffer_cset((TiBuffer *)bv_getInt(a), bv_getInt(b));
}

b_varl * bf_ti_buffer_cget1(b_scope * sc, b_varl * a)
{
	if (!( bv_assertNumeric(a) )) { return 0; }
	return bv_intToBv(ti_buffer_cget((TiBuffer *)bv_getInt(a)));
}

void bf_ti_buffer_input2(b_scope * sc, b_varl * a, b_varl * b)
{
	if (!( bv_assertNumeric(a) && bv_assertNumeric(b) )) { be_cthrow(be_cast); return; }
	ti_buffer_input((TiBuffer *)bv_getInt(a), bv_getInt(b));
}

void bf_ti_multiline_textv(b_scope * sc, b_varl ** argv, int argc)
{
	int lc, sl, cb;
	if (!( argc >= 10 && argc <= 13 && bv_assertNumeric(argv[0]) && bv_assertNumeric(argv[1]) && bv_assertNumeric(argv[2]) && bv_assertNumeric(argv[3]) && bv_assertNumeric(argv[4]) && bv_assertNumeric(argv[5])
	 && bv_assertNumeric(argv[6]) && bv_assertString(argv[7]) && bv_assertNumeric(argv[8]) && bv_assertNumeric(argv[9]) )) { be_cthrow(be_cast); return; }
	ti_multiline_text((ttk_surface)bv_getInt(argv[0]), *((ttk_font *)bv_getInt(argv[1])), bv_getInt(argv[2]), bv_getInt(argv[3]), bv_getInt(argv[4]), bv_getInt(argv[5])
	 rgbtottk(bv_getInt(argv[6])), bv_getString(argv[7]), bv_getInt(argv[8]), bv_getInt(argv[9]), &lc, &sl, &cb);
	if (argc > 10 && argv[10]) bv_sets(argv[10], bv_intToBv(lc));
	if (argc > 11 && argv[11]) bv_sets(argv[11], bv_intToBv(sl));
	if (argc > 12 && argv[12]) bv_sets(argv[12], bv_intToBv(cb));
}

b_varl * bf_ti_new_text_widgetv(b_scope * sc, b_varl ** argv, int argc)
{
	if (!( ((argc == 7 && bv_assertNumeric(argv[6]) || (argc == 10 && bv_assertNumeric(argv[6]) && bv_assertNumeric(argv[7]) && bv_assertNumeric(argv[8]) && bv_assertNumeric(argv[9])))
	 && bv_assertNumeric(argv[0]) && bv_assertNumeric(argv[1]) && bv_assertNumeric(argv[2]) && bv_assertNumeric(argv[3])
	 && bv_assertNumeric(argv[4]) && bv_assertString(argv[5]) )) { return 0; }
	if (argc == 6) {
		return bv_intToBv((signed long int)ti_new_text_widget(bv_getInt(argv[0]), bv_getInt(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]), bv_getInt(argv[4]), bv_getString(argv[5]), basic_ti_callback, basic_draw, basic_input, bv_getInt(argv[6])));
	} else {
		return bv_intToBv((signed long int)ti_new_text_widget(bv_getInt(argv[0]), bv_getInt(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]), bv_getInt(argv[4]), bv_getString(argv[5]), (int (*)(TWidget *, char *))bv_getInt(argv[6]), (void (*)(TWidget *, ttk_surface))bv_getInt(argv[7]), (int (*)(TWidget *, int))bv_getInt(argv[8]), bv_getInt(argv[9])));
	}
}

b_varl * bf_ti_new_standard_text_widgetv(b_scope * sc, b_varl ** argv, int argc)
{
	if (!( ((argc == 6) || (argc == 7 && bv_assertNumeric(argv[6]))) && bv_assertNumeric(argv[0]) && bv_assertNumeric(argv[1]) && bv_assertNumeric(argv[2]) && bv_assertNumeric(argv[3])
	 && bv_assertNumeric(argv[4]) && bv_assertString(argv[5]) )) { return 0; }
	if (argc == 6) {
		return bv_intToBv((signed long int)ti_new_standard_text_widget(bv_getInt(argv[0]), bv_getInt(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]), bv_getInt(argv[4]), bv_getString(argv[5]), basic_ti_callback));
	} else {
		return bv_intToBv((signed long int)ti_new_standard_text_widget(bv_getInt(argv[0]), bv_getInt(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]), bv_getInt(argv[4]), bv_getString(argv[5]), (int (*)(TWidget *, char *))bv_getInt(argv[6])));
	}
}

b_varl * bf_ti_new_numeric_text_widgetv(b_scope * sc, b_varl ** argv, int argc)
{
	if (!( ((argc == 6) || (argc == 7 && bv_assertNumeric(argv[6]))) && bv_assertNumeric(argv[0]) && bv_assertNumeric(argv[1]) && bv_assertNumeric(argv[2]) && bv_assertNumeric(argv[3])
	 && bv_assertNumeric(argv[4]) && bv_assertString(argv[5]) )) { return 0; }
	if (argc == 6) {
		return bv_intToBv((signed long int)ti_new_numeric_text_widget(bv_getInt(argv[0]), bv_getInt(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]), bv_getInt(argv[4]), bv_getString(argv[5]), basic_ti_callback));
	} else {
		return bv_intToBv((signed long int)ti_new_numeric_text_widget(bv_getInt(argv[0]), bv_getInt(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]), bv_getInt(argv[4]), bv_getString(argv[5]), (int (*)(TWidget *, char *))bv_getInt(argv[6])));
	}
}

b_varl * bf_ti_new_password_text_widgetv(b_scope * sc, b_varl ** argv, int argc)
{
	if (!( ((argc == 6) || (argc == 7 && bv_assertNumeric(argv[6]))) && bv_assertNumeric(argv[0]) && bv_assertNumeric(argv[1]) && bv_assertNumeric(argv[2]) && bv_assertNumeric(argv[3])
	 && bv_assertNumeric(argv[4]) && bv_assertString(argv[5]) )) { return 0; }
	if (argc == 6) {
		return bv_intToBv((signed long int)ti_new_password_text_widget(bv_getInt(argv[0]), bv_getInt(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]), bv_getInt(argv[4]), bv_getString(argv[5]), basic_ti_callback));
	} else {
		return bv_intToBv((signed long int)ti_new_password_text_widget(bv_getInt(argv[0]), bv_getInt(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]), bv_getInt(argv[4]), bv_getString(argv[5]), (int (*)(TWidget *, char *))bv_getInt(argv[6])));
	}
}

b_varl * bf_ti_new_multiline_text_widgetv(b_scope * sc, b_varl ** argv, int argc)
{
	if (!( ((argc == 6) || (argc == 7 && bv_assertNumeric(argv[6]))) && bv_assertNumeric(argv[0]) && bv_assertNumeric(argv[1]) && bv_assertNumeric(argv[2]) && bv_assertNumeric(argv[3])
	 && bv_assertNumeric(argv[4]) && bv_assertString(argv[5]) )) { return 0; }
	if (argc == 6) {
		return bv_intToBv((signed long int)ti_new_multiline_text_widget(bv_getInt(argv[0]), bv_getInt(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]), bv_getInt(argv[4]), bv_getString(argv[5]), basic_ti_callback));
	} else {
		return bv_intToBv((signed long int)ti_new_multiline_text_widget(bv_getInt(argv[0]), bv_getInt(argv[1]), bv_getInt(argv[2]), bv_getInt(argv[3]), bv_getInt(argv[4]), bv_getString(argv[5]), (int (*)(TWidget *, char *))bv_getInt(argv[6])));
	}
}

void bf_init_pz2()
{
	bf_AddCommands(60,
	bf_pz_save_config, bf_pz_blast_config, bf_pz_free_config, bf_pz_set_int_setting,
	bf_pz_set_string_setting, bf_pz_set_float_setting, bf_pz_set_ilist_setting,
	bf_pz_set_slist_setting, bf_pz_set_blob_setting, bf_pz_unset_setting,
	bf_pz_menu_set_option, bf_pz_menu_sort, bf_pz_menu_remove, bf_pz_resize_widget,
	bf_pz_widget_set_timer, bf_pz_hide_window, bf_pz_close_window, bf_pz_show_window,
	bf_pz_hwid_put_left, bf_pz_hwid_pack_left, bf_pz_hwid_pack_right, bf_pz_hwid_unpack,
	bf_pz_header_set_local, bf_pz_header_unset_local, bf_pz_message_title,
	bf_pz_message, bf_pz_warning, bf_pz_error, bf_pz_perror, bf_pz_vector_string,
	bf_pz_vector_string_center, bf_pz_ipod_set, bf_pz_ipod_fix_setting,
	bf_pz_ipod_fix_settings, bf_pz_ipod_go_to_diskmode, bf_pz_ipod_reboot,
	bf_pz_ipod_powerdown, bf_pz_load_font, bf_pz_dsp_write, bf_pz_dsp_close,
	bf_pz_browser_set_handler, bf_pz_browser_remove_handler, bf_pz_browser_add_action,
	bf_pz_browser_remove_action, bf_pz_register_input_method, bf_pz_register_input_method_n,
	bf_pz_register_global_hold_button, bf_pz_register_global_unused_handler,
	bf_pz_unregister_global_hold_button, bf_pz_unregister_global_unused_handler,
	bf_pz_handled_hold, bf_ti_destroy_buffer, bf_ti_buffer_insert_byte,
	bf_ti_buffer_remove_byte, bf_ti_buffer_insert_char, bf_ti_buffer_remove_char,
	bf_ti_buffer_cmove, bf_ti_buffer_cset, bf_ti_buffer_input, bf_ti_multiline_text);
	
	bf_AddFunctions(70,
	bf_pzsettingtype, bf_pzmenuaction, bf_pzeventtype, bf_pzbutton, bf_pzwindowgeom, bf_pzdspsource,
	bf_pz_basic_module, bf_pz_register_module, bf_pz_module_get_cfgpath,
	bf_pz_module_get_datapath, bf_pz_module_softdep, bf_pz_load_config,
	bf_pz_get_setting, bf_pz_get_int_setting, bf_pz_get_string_setting,
	bf_pz_new_menu_window, bf_pz_get_menu_item, bf_pz_mh_sub, bf_pz_menu_add_ttkh,
	bf_pz_menu_add_stub, bf_pz_menu_add_after, bf_pz_menu_add_top, bf_pz_menu_add_action,
	bf_pz_menu_add_option, bf_pz_menu_get_option, bf_pz_menu_add_setting,
	bf_pz_new_window, bf_pz_finish_window, bf_pz_do_dialog, bf_pz_dialog,
	bf_pz_errdialog, bf_pz_vector_width, bf_pz_ipod_get_battery_level,
	bf_pz_ipod_is_charging, bf_pz_ipod_get_hw_version, bf_pz_ipod_usb_is_connected,
	bf_pz_ipod_fw_is_connected, bf_pz_select_font, bf_pz_dsp_vol_change,
	bf_pz_dsp_vol_up, bf_pz_dsp_vol_down, bf_pz_dsp_get_volume, bf_pz_dsp_setup,
	bf_pz_dsp_read, bf_pz_dsp_open, bf_pz_browser_open, bf_pz_browser_get_actions,
	bf_pz_start_input, bf_pz_start_input_n, bf_ti_ap_getx, bf_ti_ap_get,
	bf_ti_create_tim_widget, bf_ti_supports_method, bf_ti_select, bf_ti_current_method,
	bf_ti_use_method, bf_ti_unuse_method, bf_ti_ttkselect, bf_ti_register,
	bf_ti_create_buffer, bf_ti_expand_buffer, bf_ti_set_buffer,
	bf_ti_create_buffer_with, bf_ti_get_buffer, bf_ti_buffer_cget, bf_ti_new_text_widget,
	bf_ti_new_standard_text_widget, bf_ti_new_numeric_text_widget,
	bf_ti_new_password_text_widget, bf_ti_new_multiline_text_widget);
}

