/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "pz.h"
#include "basic.h"
#include "f_ipod.h"
#include <string.h>
#include <sys/stat.h>

extern PzWindow * new_terminal_window_with(char * path, char * argv0, char * argv1);

#define MAX_BASICS_AT_ONCE 16

static PzModule * module;
static ttk_menu_item basic_gmi;
static ttk_menu_item basic_tmi;
static b_scope * basic_scopestack[MAX_BASICS_AT_ONCE];
static int basic_scopestackptr = 0;

PzModule * basic_get_pzmodule()
{
	return module;
}

PzWindow * destroy_basic_graphical_window_with(b_scope * sc)
{
	if (basic_scopestackptr < 1) {
		pz_error("No BASIC programs are running.");
		return TTK_MENU_DONOTHING;
	}
	do {
		bv_freeScope(basic_scopestack[--basic_scopestackptr], 1);
	} while (basic_scopestackptr > 0 && basic_scopestack[basic_scopestackptr] != sc);
	return TTK_MENU_DONOTHING;
}

PzWindow * destroy_basic_graphical_window()
{
	if (basic_scopestackptr < 1) {
		pz_error("No BASIC programs are running.");
		return TTK_MENU_DONOTHING;
	}
	bv_freeScope(basic_scopestack[--basic_scopestackptr], 1);
	return TTK_MENU_DONOTHING;
}

PzWindow * new_basic_graphical_window_with(char * filename)
{
	b_scope * s;
	b_varl * v;
	if (basic_scopestackptr >= MAX_BASICS_AT_ONCE) {
		pz_error("You cannot have more than %d BASIC programs running at once.", MAX_BASICS_AT_ONCE);
		return TTK_MENU_DONOTHING;
	}
	s = basic_scopestack[basic_scopestackptr++] = bv_newScope();
	bx_set_include_path((char *)pz_module_get_datapath(module, ""));
	be_init();
	bf_init();
	bf_init_ipod();
	v = bv_stringToBv(filename);
	bf_run1(s,v);
	bv_cfreeBv(v);
	if (be_checkerr()) {
		char buf[1024];
		be_sprinterr(buf);
		be_clearerr();
		pz_error("Could not load BASIC program: %s",buf);
		destroy_basic_graphical_window_with(s);
	}
	return TTK_MENU_DONOTHING;
}

PzWindow * new_basic_terminal_window_with(char * filename)
{
	return new_terminal_window_with((char *)pz_module_get_datapath(module, "basic"), "basic", filename);
}

PzWindow * new_basic_terminal_window()
{
	return new_terminal_window_with((char *)pz_module_get_datapath(module, "basic"), "basic", 0);
}

static int basic_openable(const char * filename)
{
	char ext[4];
	struct stat st;
	stat(filename, &st);
	if (S_ISDIR(st.st_mode)) return 0;
	strncpy(ext, &filename[strlen(filename)-4], 4);
	if (strncasecmp(ext,".bas",4)) return 0;
	return 1;
}

static PzWindow * basic_graphical_handler(ttk_menu_item * item)
{
	return new_basic_graphical_window_with(item->data);
}

static PzWindow * basic_terminal_handler(ttk_menu_item * item)
{
	return new_basic_terminal_window_with(item->data);
}

static void basic_mod_init()
{
	module = pz_register_module("basic", 0);
	pz_menu_add_action("/Extras/Utilities/BASIC", new_basic_terminal_window);
	
	basic_gmi.name = _("Run with Graphical LET BASIC");
	basic_gmi.makesub = basic_graphical_handler;
	pz_browser_add_action (basic_openable, &basic_gmi);
	basic_tmi.name = _("Run with Terminal LET BASIC");
	basic_tmi.makesub = basic_terminal_handler;
	pz_browser_add_action (basic_openable, &basic_tmi);
}

PZ_MOD_INIT(basic_mod_init)
