/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "basic.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef CUSTOM_INIT
extern void CUSTOM_INIT();
#endif
#ifdef CUSTOM_FGETS
extern int CUSTOM_FGETS(char * b, int c, FILE * fp);
#endif

int main(int argc, char * argv[])
{
	char buf[65536];
	int printVersion = 0;
	int printAbout = 1;
	int promptMode = 1;
	int resetTerm = 1;
	char prompt = ']';
	b_scope * s = bv_newScope();
	be_init();
	bf_init();
#ifdef CUSTOM_INIT
	CUSTOM_INIT();
#endif
	if (argc>1) {
		int i,j;
		b_varl * v;
		int runMode = 0;
		int noBreak = 0;
		int noEnd = 0;
		printAbout = 0;
		promptMode = 0;
		resetTerm = 0;
		for (i=1; i<argc; i++) {
			if (argv[i][0] == '-') {
				
#define USAGE "[-bnvapPt] [-k char] [[-rsciI] file ...] [-e text ...]"
				if (argv[i][1] == '-') {
					if (!strcmp(&argv[i][2], "help")) {
						printf("Usage: %s "USAGE"\n",argv[0]);
						printf("\n");
						printf("Options:\n");
						printf("  -r file   Run a BASIC program in its own variable scope (default).\n");
						printf("  -s file   Run a BASIC program in a child of the global scope.\n");
						printf("  -c file   Run a BASIC program in the global scope.\n");
						printf("  -i file   Run a BASIC include file in the global scope.\n");
						printf("  -I path   Set the path to check for include files (must come before -i).\n");
						printf("  -e text   Execute the BASIC instructions passed.\n");
						printf("  -b        Continue processing parameters after an error occurs.\n");
						printf("  -n        Continue processing parameters after an END instruction occurs.\n");
						printf("  -v        Output version number after processing parameters.\n");
						printf("  -a        Output about text after processing parameters.\n");
						printf("  -p        Show BASIC prompt after processing parameters.\n");
						printf("  -P        Same as -p -k \"\". Use for piping input to the interpreter.\n");
						printf("  -k char   Change prompt character. Use -k \"\" for no prompt character.\n");
						printf("  -t        Reset terminal display attributes upon exiting.\n");
						printf("  --help    Show this information.\n");
						printf("  --version Same as -a.\n");
						printf("\n");
						printf("No parameters is equivalent to %s -apt\n",argv[0]);
					} else if (!strcmp(&argv[i][2], "version")) {
						printAbout = 1;
					} else {
						printf("%s: illegal option -- %s\n", argv[0], &argv[i][2]);
						printf("usage: %s "USAGE"\n", argv[0]);
					}
				} else for (j=1; j<strlen(argv[i]); j++) {
					switch (argv[i][j]) {
					case 'r': runMode = 0; break;
					case 's': runMode = 1; break;
					case 'c': runMode = 2; break;
					case 'i': runMode = 3; break;
					case 'e': runMode = 16; break;
					case 'k': runMode = 32; break;
					case 'I': runMode = 33; break;
					case 'b': noBreak = 1; break;
					case 'n': noEnd = 1; break;
					case 'v': printVersion = 1; break;
					case 'a': printAbout = 1; break;
					case 'p': promptMode = 1; break;
					case 'P': promptMode = 1; prompt = 0; break;
					case 't': resetTerm = 1; break;
					default:
						printf("%s: illegal option -- %c\n", argv[0], argv[i][j]);
						printf("usage: %s "USAGE"\n", argv[0]);
						j = strlen(argv[i]);
						break;
					}
				}
				
			} else {
				switch (runMode) {
				case 0:
					v = bv_stringToBv(argv[i]);
					bf_run1(s,v);
					bv_freeBv(v);
					break;
				case 1:
					v = bv_stringToBv(argv[i]);
					bf_subrun1(s,v);
					bv_freeBv(v);
					break;
				case 2:
					v = bv_stringToBv(argv[i]);
					bf_chain1(s,v);
					bv_freeBv(v);
					break;
				case 3:
					v = bv_stringToBv(argv[i]);
					bf_include1(s,v);
					bv_freeBv(v);
					break;
				case 16:
					bv_cfreeBv(bx_exec(s,argv[i]));
					break;
				case 32:
					prompt = argv[i][0];
					break;
				case 33:
					bx_set_include_path(argv[i]);
					break;
				}
				if (be_checkerr()) {
					be_printerr();
					be_clearerr();
					if (!noBreak) break;
				}
				if (be_closed() && !noEnd) {
					break;
				}
				runMode = 0;
			}
		}
	}
	if (printVersion) {
		b_varl * n = bv_intToBv(1);
		b_varl * st = bf_about1(s,n);
		if (st->sval && st->sval[0]) {
			printf("%s\n", st->sval+8);
		}
		bv_cfreeBv(st);
		bv_cfreeBv(n);
	}
	if (printAbout) {
		bf_about0(s);
	}
	if (promptMode) {
		while (1) {
			if (prompt > 32) printf("%c",prompt);
#ifndef CUSTOM_FGETS
			if (fgets(buf, 65530, stdin)) {
#else
			if (CUSTOM_FGETS(buf, 65530, stdin)) {
#endif
				bv_cfreeBv(bx_exec(s, buf));
				if (be_checkerr()) {
					be_printerr();
					be_clearerr();
				}
				if (be_closed()) {
					break;
				}
			}
		}
	}
	if (resetTerm) {
		printf("\x1B[0m");
	}
	bv_freeScope(s,1);
	return 0;
}
