/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

/**
 * Turn off local echo for the terminal.
 */
void tty_noecho()
{
	system("stty -echo &> /dev/null");
}

/**
 * Turn on local echo for the terminal.
 */
void tty_echo()
{
	system("stty echo &> /dev/null");
}

/**
 * Read a single character from the terminal,
 * without waiting for a newline first,
 * with echo if the terminal has it on by default,
 * blocking.
 * This is used by LET BASIC's GET command.
 */
int tty_readchar()
{
	char ch;
	system("stty cbreak &> /dev/null");
	ch = getchar();
	system("stty -cbreak &> /dev/null");
	return ch;
}

/**
 * Turn echo off, enter raw mode so we don't have to wait for newlines,
 * send the escape sequence passed in the parameter, gather the terminal's
 * responding escape sequence, resume normal operations (echo, line mode),
 * and return the response the terminal gave us.
 * This is used by LET BASIC's functions for getting the cursor position.
 */
char * tty_readesc(char * seq)
{
	char buf[256];
	char * p = buf;
	char last;
	system("stty cbreak -echo &> /dev/null");
	if (seq) printf("%s",seq);
	do {
		last = *p = getchar();
		p++;
	} while (!isalpha(last));
	*p = 0;
	system("stty -cbreak echo &> /dev/null");
	return strdup(buf);
}
