/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "btype.h"
#include "expr.h"
#include "exceptions.h"
#include "vars.h"
#include <stdio.h>
#include <string.h>
#include "stringx.h"
#include <stdlib.h>

b_object * bv_createObj(char * className, b_scope * class)
{
	b_usub * us;
	b_object * o = (b_object *)calloc(1, sizeof(b_object));
	if (!o) return 0;
	o->className = strdup(className);
	o->useCount = 1;
	o->classScope = class;
	o->theObject = bv_copyScope(class,1);
	us = bv_findus(o->theObject, "created");
	if (us) {
		bx_callUsub(o->theObject, us, "");
	} else {
		be_clearerr();
	}
	return o;
}

void bv_freeObj(b_object * o)
{
	if (o) {
		b_usub * us = bv_findus(o->theObject, "destroyed");
		if (us) {
			bx_callUsub(o->theObject, us, "");
		} else {
			be_clearerr();
		}
		if (o->className) free(o->className);
		if (o->theObject) bv_freeScope(o->theObject,1);
		free(o);
	}
}

b_object * bv_copyObj(b_object * o)
{
	if (o) {
		b_usub * us;
		b_object * no = (b_object *)calloc(1, sizeof(b_object));
		no->className = strdup(o->className);
		no->useCount = 1;
		no->classScope = o->classScope;
		no->theObject = bv_copyScope(o->theObject,1);
		us = bv_findus(no->theObject, "cloned");
		if (us) {
			bx_callUsub(no->theObject, us, "");
		} else {
			be_clearerr();
		}
		return no;
	} else return 0;
}

b_object * bv_useObj(b_object * o)
{
	if (o) o->useCount++;
	return o;
}

b_object * bv_unuseObj(b_object * o)
{
	if (o) {
		o->useCount--;
		if (!o->useCount) {
			bv_freeObj(o);
			return 0;
		}
	}
	return o;
}

b_varl * bv_createBv(b_type type, b_scope * otype, double d, signed long int i, char * s, b_object * o, b_exception e)
{
	b_varl * v = (b_varl *)calloc(1, sizeof(b_varl));
	if (!v) return 0;
	v->type = type;
	v->otyp = otype;
	v->dyna = 1;
	v->readOnly = 0;
	v->fval = d;
	v->ival = i;
	v->sval = (s?strdup(s):0);
	v->oval = (o?bv_useObj(o):0);
	v->eval = e;
	return v;
}

void bv_freeBv(b_varl * v)
{
	if (v) {
		if (v->sval) free(v->sval);
		if (v->oval) bv_unuseObj(v->oval);
		free(v);
	}
}

void bv_cfreeBv(b_varl * v)
{
	if (v && v->dyna) bv_freeBv(v);
}

b_varl * bv_copyBv(b_varl * v)
{
	if (v) {
		b_varl * nv = (b_varl *)calloc(1, sizeof(b_varl));
		if (!nv) return 0;
		nv->type = v->type;
		nv->otyp = v->otyp;
		nv->dyna = 1;
		nv->readOnly = 0;
		nv->fval = v->fval;
		nv->ival = v->ival;
		nv->sval = ((v->sval)?strdup(v->sval):0);
		nv->oval = ((v->oval)?bv_useObj(v->oval):0);
		nv->eval = v->eval;
		return nv;
	} else return 0;
}

b_varl * bv_makeStatic(b_varl * v)
{
	if (v) v->dyna = 0;
	return v;
}

b_varl * bv_makeDynamic(b_varl * v)
{
	if (v) v->dyna = 1;
	return v;
}

b_array * bv_createArray(b_type t, int n, int a, int b, int c)
{
	b_array * arr = (b_array *)calloc(1, sizeof(b_array));
	if (!arr) return 0;
	if (n<0) n=0; if (n>3) n=3;
	if (a<1) a=1; if (b<1) b=1; if (c<1) c=1;
	switch (n) {
		case 0: a=1;
		case 1: b=1;
		case 2: c=1;
	}
	arr->type = t;
	arr->dim = n;
	arr->dim1 = a;
	arr->dim2 = b;
	arr->dim3 = c;
	arr->elem = (b_varl *)calloc(a*b*c, sizeof(b_varl));
	if (arr->elem) {
		int i;
		for (i=a*b*c-1; i>=0; i--) {
			arr->elem[i].type = t;
			arr->elem[i].dyna = 0;
			arr->elem[i].readOnly = 0;
		}
	}
	return arr;
}

b_array * bv_redimArray(b_array * arr, int n, int a, int b, int c)
{
	int i, j, k, im, jm, km, ai, nai;
	b_array * narr;
	if (!arr) return 0;
	if (n<0) n=0; if (n>3) n=3;
	if (a<1) a=1; if (b<1) b=1; if (c<1) c=1;
	switch (n) {
		case 0: a=1;
		case 1: b=1;
		case 2: c=1;
	}
	narr = bv_createArray(arr->type, n, a, b, c);
	if (!narr) return 0;
	if (narr->elem && arr->elem) {
		im = (a < arr->dim1)?(a):(arr->dim1);
		jm = (b < arr->dim2)?(b):(arr->dim2);
		km = (c < arr->dim3)?(c):(arr->dim3);
		for (i=0; i < arr->dim1; i++) {
			for (j=0; j < arr->dim2; j++) {
				for (k=0; k < arr->dim3; k++) {
					ai = (i*arr->dim2*arr->dim3 + j*arr->dim3 + k);
					if (i<im && j<jm && k<km) {
						nai = (i*b*c + j*c + k);
						narr->elem[nai] = arr->elem[ai];
					} else {
						if (arr->elem[ai].sval) free(arr->elem[ai].sval);
						if (arr->elem[ai].oval) bv_unuseObj(arr->elem[ai].oval);
					}
				}
			}
		}
	}
	if (arr->elem) free(arr->elem);
	arr->dim = n;
	arr->dim1 = a;
	arr->dim2 = b;
	arr->dim3 = c;
	arr->elem = narr->elem;
	free(narr);
	return arr;
}

void bv_freeArray(b_array * arr)
{
	if (arr) {
		if (arr->elem) {
			int i;
			for (i = arr->dim1 * arr->dim2 * arr->dim3 - 1; i>=0; i--) {
				if (arr->elem[i].sval) free(arr->elem[i].sval);
				if (arr->elem[i].oval) bv_unuseObj(arr->elem[i].oval);
			}
			free(arr->elem);
		}
		free(arr);
	}
}

b_array * bv_copyArray(b_array * arr)
{
	if (arr) {
		b_array * narr = (b_array *)calloc(1, sizeof(b_array));
		if (!narr) return 0;
		narr->type = arr->type;
		narr->dim = arr->dim;
		narr->dim1 = arr->dim1;
		narr->dim2 = arr->dim2;
		narr->dim3 = arr->dim3;
		if (arr->elem) {
			int m = arr->dim1 * arr->dim2 * arr->dim3;
			narr->elem = (b_varl *)calloc(m, sizeof(b_varl));
			if (narr->elem) {
				int i;
				memcpy(narr->elem, arr->elem, m * sizeof(b_varl));
				for (i=m-1; i>=0; i--) {
					if (arr->elem[i].sval) narr->elem[i].sval = strdup(arr->elem[i].sval);
					if (arr->elem[i].oval) narr->elem[i].oval = bv_useObj(arr->elem[i].oval);
				}
			}
		}
		return narr;
	} else return 0;
}

b_usub * bv_createUsub(int n, char * pl, char * a, b_type at, char * b, b_type bt, char * c, b_type ct, char * d, b_type dt, char * code)
{
	b_usub * us = (b_usub *)calloc(1, sizeof(b_usub));
	if (!us) return 0;
	us->paramCount = n;
	us->paramNames[0] = (a?strdup(a):0);
	us->paramNames[1] = (b?strdup(b):0);
	us->paramNames[2] = (c?strdup(c):0);
	us->paramNames[3] = (d?strdup(d):0);
	us->paramTypes[0] = at;
	us->paramTypes[1] = bt;
	us->paramTypes[2] = ct;
	us->paramTypes[3] = dt;
	us->paramList = (pl?strdup(pl):0);
	us->code = (code?strdup(code):0);
	return us;
}

void bv_freeUsub(b_usub * us)
{
	if (us) {
		if (us->code) free(us->code);
		if (us->paramNames[0]) free(us->paramNames[0]);
		if (us->paramNames[1]) free(us->paramNames[1]);
		if (us->paramNames[2]) free(us->paramNames[2]);
		if (us->paramNames[3]) free(us->paramNames[3]);
		if (us->paramList) free(us->paramList);
		free(us);
	}
}

b_usub * bv_copyUsub(b_usub * us)
{
	if (us) {
		b_usub * nus = (b_usub *)calloc(1, sizeof(b_usub));
		if (!nus) return 0;
		nus->paramCount = us->paramCount;
		nus->paramNames[0] = ((us->paramNames[0])?strdup(us->paramNames[0]):0);
		nus->paramNames[1] = ((us->paramNames[1])?strdup(us->paramNames[1]):0);
		nus->paramNames[2] = ((us->paramNames[2])?strdup(us->paramNames[2]):0);
		nus->paramNames[3] = ((us->paramNames[3])?strdup(us->paramNames[3]):0);
		nus->paramTypes[0] = us->paramTypes[0];
		nus->paramTypes[1] = us->paramTypes[1];
		nus->paramTypes[2] = us->paramTypes[2];
		nus->paramTypes[3] = us->paramTypes[3];
		nus->paramList = (us->paramList ? strdup(us->paramList) : 0);
		nus->code = (us->code ? strdup(us->code) : 0);
		return nus;
	} else return 0;
}

b_ufunc * bv_createUfunc(b_type ret, int n, char * pl, char * a, b_type at, char * b, b_type bt, char * c, b_type ct, char * d, b_type dt, char * code)
{
	b_ufunc * uf = (b_ufunc *)calloc(1, sizeof(b_ufunc));
	if (!uf) return 0;
	uf->returnType = ret;
	uf->paramCount = n;
	uf->paramNames[0] = (a?strdup(a):0);
	uf->paramNames[1] = (b?strdup(b):0);
	uf->paramNames[2] = (c?strdup(c):0);
	uf->paramNames[3] = (d?strdup(d):0);
	uf->paramTypes[0] = at;
	uf->paramTypes[1] = bt;
	uf->paramTypes[2] = ct;
	uf->paramTypes[3] = dt;
	uf->paramList = (pl?strdup(pl):0);
	uf->code = (code?strdup(code):0);
	return uf;
}

void bv_freeUfunc(b_ufunc * uf)
{
	if (uf) {
		if (uf->code) free(uf->code);
		if (uf->paramNames[0]) free(uf->paramNames[0]);
		if (uf->paramNames[1]) free(uf->paramNames[1]);
		if (uf->paramNames[2]) free(uf->paramNames[2]);
		if (uf->paramNames[3]) free(uf->paramNames[3]);
		if (uf->paramList) free(uf->paramList);
		free(uf);
	}
}

b_ufunc * bv_copyUfunc(b_ufunc * uf)
{
	if (uf) {
		b_ufunc * nuf = (b_ufunc *)calloc(1, sizeof(b_ufunc));
		if (!nuf) return 0;
		nuf->returnType = uf->returnType;
		nuf->paramCount = uf->paramCount;
		nuf->paramNames[0] = ((uf->paramNames[0])?strdup(uf->paramNames[0]):0);
		nuf->paramNames[1] = ((uf->paramNames[1])?strdup(uf->paramNames[1]):0);
		nuf->paramNames[2] = ((uf->paramNames[2])?strdup(uf->paramNames[2]):0);
		nuf->paramNames[3] = ((uf->paramNames[3])?strdup(uf->paramNames[3]):0);
		nuf->paramTypes[0] = uf->paramTypes[0];
		nuf->paramTypes[1] = uf->paramTypes[1];
		nuf->paramTypes[2] = uf->paramTypes[2];
		nuf->paramTypes[3] = uf->paramTypes[3];
		nuf->paramList = (uf->paramList ? strdup(uf->paramList) : 0);
		nuf->code = (uf->code ? strdup(uf->code) : 0);
		return nuf;
	} else return 0;
}

b_scope * bv_newScope()
{
	b_scope * sc = (b_scope *)calloc(1, sizeof(b_scope));
	return sc;
}

b_scope * bv_newScopeChild(b_scope * scope)
{
	b_scope * sc = (b_scope *)calloc(1, sizeof(b_scope));
	sc->fallback = scope;
	return sc;
}

b_scope * bv_newScopeSibling(b_scope * scope)
{
	b_scope * sc = (b_scope *)calloc(1, sizeof(b_scope));
	if (scope) sc->fallback = scope->fallback;
	return sc;
}

b_scope * bv_newScopeParent(b_scope * scope)
{
	b_scope * sc = (b_scope *)calloc(1, sizeof(b_scope));
	if (scope) {
		sc->fallback = scope->fallback;
		scope->fallback = sc;
	}
	return sc;
}

void bv_freeVA(b_scope * sc)
{
	int i;
	if (!sc) return;
	
	/* vars */
	if (sc->names) {
		for (i=0; i<sc->count; i++) {
			free(sc->names[i]);
		}
		free(sc->names);
		sc->names = 0;
	}
	if (sc->vars) {
		for (i=0; i<sc->count; i++) {
			bv_freeBv(sc->vars[i]);
		}
		free(sc->vars);
		sc->vars = 0;
	}
	sc->count = 0;
	
	/* arrays */
	if (sc->anames) {
		for (i=0; i<sc->acount; i++) {
			free(sc->anames[i]);
		}
		free(sc->anames);
		sc->anames = 0;
	}
	if (sc->arrays) {
		for (i=0; i<sc->acount; i++) {
			bv_freeArray(sc->arrays[i]);
		}
		free(sc->arrays);
		sc->arrays = 0;
	}
	sc->acount = 0;
}

void bv_freeScope(b_scope * sc, int deep)
{
	int i;
	if (!sc) return;
	
	if (deep && sc->fallback) bv_freeScope(sc->fallback, deep);
	
	/* vars & arrays */
	bv_freeVA(sc);
	
	/* user subs */
	if (sc->usnames) {
		for (i=0; i<sc->uscount; i++) {
			free(sc->usnames[i]);
		}
		free(sc->usnames);
	}
	if (sc->usubs) {
		for (i=0; i<sc->uscount; i++) {
			bv_freeUsub(sc->usubs[i]);
		}
		free(sc->usubs);
	}
	
	/* user funcs */
	if (sc->ufnames) {
		for (i=0; i<sc->ufcount; i++) {
			free(sc->ufnames[i]);
		}
		free(sc->ufnames);
	}
	if (sc->ufuncs) {
		for (i=0; i<sc->ufcount; i++) {
			bv_freeUfunc(sc->ufuncs[i]);
		}
		free(sc->ufuncs);
	}
	
	/* classes */
	if (sc->cnames) {
		for (i=0; i<sc->ccount; i++) {
			free(sc->cnames[i]);
		}
		free(sc->cnames);
	}
	if (sc->classes) {
		for (i=0; i<sc->ccount; i++) {
			bv_freeScope(sc->classes[i],0);
		}
		free(sc->classes);
	}
	
	free(sc);
}

b_scope * bv_copyScope(b_scope * sc, int deep)
{
	int i;
	if (!sc) return 0;
	b_scope * ns = (b_scope *)calloc(1, sizeof(b_scope));
	if (!ns) return 0;
	
	/* vars */
	if (sc->names) {
		ns->names = (char **)malloc(sc->count * sizeof(char *));
		for (i=0; i<sc->count; i++) {
			ns->names[i] = strdup(sc->names[i]);
		}
	}
	if (sc->vars) {
		ns->vars = (b_varl **)malloc(sc->count * sizeof(b_varl *));
		for (i=0; i<sc->count; i++) {
			ns->vars[i] = bv_makeStatic(bv_copyBv(sc->vars[i]));
		}
	}
	ns->count = sc->count;
	
	/* arrays */
	if (sc->anames) {
		ns->anames = (char **)malloc(sc->acount * sizeof(char *));
		for (i=0; i<sc->acount; i++) {
			ns->anames[i] = strdup(sc->anames[i]);
		}
	}
	if (sc->arrays) {
		ns->arrays = (b_array **)malloc(sc->acount * sizeof(b_array *));
		for (i=0; i<sc->acount; i++) {
			ns->arrays[i] = bv_copyArray(sc->arrays[i]);
		}
	}
	ns->acount = sc->acount;
	
	/* user subs */
	if (sc->usnames) {
		ns->usnames = (char **)malloc(sc->uscount * sizeof(char *));
		for (i=0; i<sc->uscount; i++) {
			ns->usnames[i] = strdup(sc->usnames[i]);
		}
	}
	if (sc->usubs) {
		ns->usubs = (b_usub **)malloc(sc->uscount * sizeof(b_usub *));
		for (i=0; i<sc->uscount; i++) {
			ns->usubs[i] = bv_copyUsub(sc->usubs[i]);
		}
	}
	ns->uscount = sc->uscount;
	
	/* user funcs */
	if (sc->ufnames) {
		ns->ufnames = (char **)malloc(sc->ufcount * sizeof(char *));
		for (i=0; i<sc->ufcount; i++) {
			ns->ufnames[i] = strdup(sc->ufnames[i]);
		}
	}
	if (sc->ufuncs) {
		ns->ufuncs = (b_ufunc **)malloc(sc->ufcount * sizeof(b_ufunc *));
		for (i=0; i<sc->ufcount; i++) {
			ns->ufuncs[i] = bv_copyUfunc(sc->ufuncs[i]);
		}
	}
	ns->ufcount = sc->ufcount;
	
	/* classes */
	if (sc->cnames) {
		ns->cnames = (char **)malloc(sc->ccount * sizeof(char *));
		for (i=0; i<sc->ccount; i++) {
			ns->cnames[i] = strdup(sc->cnames[i]);
		}
	}
	if (sc->classes) {
		ns->classes = (b_scope **)malloc(sc->ccount * sizeof(b_scope *));
		for (i=0; i<sc->ccount; i++) {
			ns->classes[i] = bv_copyScope(sc->classes[i],1);
		}
	}
	ns->ccount = sc->ccount;
	
	if (deep && sc->fallback) {
		ns->fallback = bv_copyScope(sc->fallback, deep);
	} else {
		ns->fallback = sc->fallback;
	}
	return ns;
}

int bv_scopeIsChild(b_scope * ocl, b_scope * cl)
{
	/* returns true if and only if OCL is CL or OCL is a child of CL */
	/* returns true if and only if OCL is CL or OCL is a subclass of CL */
	while (1) {
		if (!ocl) {
			return 0;
			break;
		} else if (ocl == cl) {
			return 1;
			break;
		} else {
			ocl = ocl->fallback;
		}
	}
}

double bv_getDouble(b_varl * v)
{
	if (!v) return 0.0;
	switch (v->type) {
	case bt_float:
		return v->fval;
		break;
	case bt_integer:
		return (double)v->ival;
		break;
	case bt_string:
		{
		double d;
		if (!v->sval) return 0.0;
		sscanf(v->sval, "%lf", &d);
		return d;
		}
		break;
	case bt_object:
		return 0;
		break;
	case bt_exception:
		return (double)v->eval;
		break;
	default:
		return 0.0;
		break;
	}
	return 0.0;
}

signed long int bv_getInt(b_varl * v)
{
	if (!v) return 0;
	switch (v->type) {
	case bt_float:
		return (signed long int)v->fval;
		break;
	case bt_integer:
		return v->ival;
		break;
	case bt_string:
		{
		signed long int d;
		if (!v->sval) return 0;
		sscanf(v->sval, "%ld", &d);
		return d;
		}
		break;
	case bt_object:
		return 0;
		break;
	case bt_exception:
		return v->eval;
		break;
	default:
		return 0;
		break;
	}
	return 0;
}

char * bv_getString(b_varl * v)
{
	static char buf[20];
	if (!v) return "";
	switch (v->type) {
	case bt_float:
		snprintf(buf, 20, "%lg", v->fval);
		return buf;
		break;
	case bt_integer:
		snprintf(buf, 20, "%ld", v->ival);
		return buf;
		break;
	case bt_string:
		return (v->sval)?(v->sval):("");
		break;
	case bt_object:
		return "";
		break;
	case bt_exception:
		snprintf(buf, 20, "%d", v->eval);
		return buf;
		break;
	default:
		return "";
		break;
	}
	return "";
}

b_object * bv_getObject(b_varl * v)
{
	if (!v) return 0;
	switch (v->type) {
	case bt_float:
		return 0;
		break;
	case bt_integer:
		return 0;
		break;
	case bt_string:
		return 0;
		break;
	case bt_object:
		return v->oval;
		break;
	case bt_exception:
		return 0;
		break;
	default:
		return 0;
		break;
	}
	return 0;
}

b_exception bv_getException(b_varl * v)
{
	if (!v) return be_none;
	switch (v->type) {
	case bt_float:
		return (int)v->fval;
		break;
	case bt_integer:
		return (int)v->ival;
		break;
	case bt_string:
		{
		int d;
		if (!v->sval) return 0;
		sscanf(v->sval, "%d", &d);
		return d;
		}
		break;
	case bt_object:
		return 0;
		break;
	case bt_exception:
		return v->eval;
		break;
	default:
		return 0;
		break;
	}
	return 0;
}

int bv_getBoolean(b_varl * v)
{
	if (!v) return 0;
	switch (v->type) {
	case bt_float:
		return (int)v->fval;
		break;
	case bt_integer:
		return v->ival;
		break;
	case bt_string:
		return (v->sval && v->sval[0]);
		break;
	case bt_object:
		return !!(v->oval);
		break;
	case bt_exception:
		return v->eval;
		break;
	default:
		return 0;
		break;
	}
	return 0;
}

int bv_assertNumeric(b_varl * v)
{
	return (v && (v->type == bt_integer || v->type == bt_float));
}

int bv_assertString(b_varl * v)
{
	return (v && (v->type == bt_string));
}

int bv_assertObject(b_varl * v)
{
	return (v && (v->type == bt_object));
}

void bv_setDouble(b_varl * v, double d)
{
	if (!v) { be_throw(be_undeclared); return; }
	if (v->readOnly) { be_throw(be_constant); return; }
	switch (v->type) {
	case bt_float:
		v->fval = d;
		break;
	case bt_integer:
		v->ival = (signed long int)d;
		break;
	case bt_string:
		be_throw(be_cast);
		break;
	case bt_object:
		be_throw(be_cast);
		break;
	case bt_exception:
		v->eval = (int)d;
		break;
	}
}

void bv_setInt(b_varl * v, signed long int i)
{
	if (!v) { be_throw(be_undeclared); return; }
	if (v->readOnly) { be_throw(be_constant); return; }
	switch (v->type) {
	case bt_float:
		v->fval = (double)i;
		break;
	case bt_integer:
		v->ival = i;
		break;
	case bt_string:
		be_throw(be_cast);
		break;
	case bt_object:
		be_throw(be_cast);
		break;
	case bt_exception:
		v->eval = i;
		break;
	}
}

void bv_setString(b_varl * v, char * s)
{
	if (!v) { be_throw(be_undeclared); return; }
	if (v->readOnly) { be_throw(be_constant); return; }
	switch (v->type) {
	case bt_float:
		be_throw(be_cast);
		break;
	case bt_integer:
		be_throw(be_cast);
		break;
	case bt_string:
		if (v->sval) free(v->sval);
		v->sval = strdup(s);
		break;
	case bt_object:
		be_throw(be_cast);
		break;
	case bt_exception:
		be_throw(be_cast);
		break;
	}
}

void bv_setObject(b_varl * v, b_object * o)
{
	if (!v) { be_throw(be_undeclared); return; }
	if (v->readOnly) { be_throw(be_constant); return; }
	switch (v->type) {
	case bt_float:
		be_throw(be_cast);
		break;
	case bt_integer:
		be_throw(be_cast);
		break;
	case bt_string:
		be_throw(be_cast);
		break;
	case bt_object:
		if (v->otyp) {
			if (!o || !bv_scopeIsChild(o->classScope, v->otyp)) {
				be_throw(be_cast);
			} else {
				bv_unuseObj(v->oval);
				v->oval = bv_useObj(o);
			}
		} else {
			v->oval = bv_useObj(o);
		}
		break;
	case bt_exception:
		be_throw(be_cast);
		break;
	}
}

void bv_setException(b_varl * v, b_exception e)
{
	if (!v) { be_throw(be_undeclared); return; }
	if (v->readOnly) { be_throw(be_constant); return; }
	switch (v->type) {
	case bt_float:
		v->fval = (double)e;
		break;
	case bt_integer:
		v->ival = (signed long int)e;
		break;
	case bt_string:
		be_throw(be_cast);
		break;
	case bt_object:
		be_throw(be_cast);
		break;
	case bt_exception:
		v->eval = e;
		break;
	}
}

b_varl * bv_doubleToBv(double d)
{
	return bv_createBv(bt_float, 0, d, (signed long int)d, 0, 0, (int)d);
}

b_varl * bv_intToBv(signed long int i)
{
	return bv_createBv(bt_integer, 0, (double)i, i, 0, 0, i);
}

b_varl * bv_stringToBv(char * s)
{
	return bv_createBv(bt_string, 0, 0, 0, s, 0, 0);
}

b_varl * bv_objectToBv(b_object * o, b_scope * ot)
{
	return bv_createBv(bt_object, ot, 0, 0, 0, o, 0);
}

b_varl * bv_exceptionToBv(b_exception e)
{
	return bv_createBv(bt_exception, 0, (double)e, (signed long int)e, 0, 0, e);
}

b_varl * bv_find(b_scope * sc, char * name)
{
	int i;
	char * n = bx_normalizeIdent(name);
	char * p = strchr(n,'.');
	if (p) {
		b_varl * ov = 0;
		b_object * o = 0;
		(*p) = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			free(n);
			if (!o) { be_throw(be_nullptr); return 0; }
			return bv_find(o->theObject, p);
		}
		free(n);
	} else {
		for (i=0; i<sc->count; i++) {
			if (!strcmp(n, sc->names[i])) {
				free(n);
				return sc->vars[i];
			}
		}
		free(n);
		if (sc->fallback) {
			return bv_find(sc->fallback, name);
		} else {
			be_throw(be_undeclared);
		}
	}
	return 0;
}

b_array * bv_finda(b_scope * sc, char * name)
{
	int i;
	char * n = bx_normalizeIdent(name);
	char * p = strchr(n,'.');
	if (p) {
		b_varl * ov = 0;
		b_object * o = 0;
		(*p) = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			free(n);
			if (!o) { be_throw(be_nullptr); return 0; }
			return bv_finda(o->theObject, p);
		}
		free(n);
	} else {
		for (i=0; i<sc->acount; i++) {
			if (!strcmp(n, sc->anames[i])) {
				free(n);
				return sc->arrays[i];
			}
		}
		free(n);
		if (sc->fallback) {
			return bv_finda(sc->fallback, name);
		} else {
			be_throw(be_undeclared);
		}
	}
	return 0;
}

b_usub * bv_findus(b_scope * sc, char * name)
{
	int i;
	char * n = bx_normalizeIdent(name);
	char * p = strchr(n,'.');
	if (p) {
		b_varl * ov = 0;
		b_object * o = 0;
		(*p) = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			free(n);
			if (!o) { be_throw(be_nullptr); return 0; }
			return bv_findus(o->theObject, p);
		}
		free(n);
	} else {
		for (i=0; i<sc->uscount; i++) {
			if (!strcmp(n, sc->usnames[i])) {
				free(n);
				return sc->usubs[i];
			}
		}
		free(n);
		if (sc->fallback) {
			return bv_findus(sc->fallback, name);
		} else {
			be_throw(be_syntax);
		}
	}
	return 0;
}

b_ufunc * bv_finduf(b_scope * sc, char * name)
{
	int i;
	char * n = bx_normalizeIdent(name);
	char * p = strchr(n,'.');
	if (p) {
		b_varl * ov = 0;
		b_object * o = 0;
		(*p) = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			free(n);
			if (!o) { be_throw(be_nullptr); return 0; }
			return bv_finduf(o->theObject, p);
		}
		free(n);
	} else {
		for (i=0; i<sc->ufcount; i++) {
			if (!strcmp(n, sc->ufnames[i])) {
				free(n);
				return sc->ufuncs[i];
			}
		}
		free(n);
		if (sc->fallback) {
			return bv_finduf(sc->fallback, name);
		} else {
			be_throw(be_syntax);
		}
	}
	return 0;
}

b_scope * bv_finduscope(b_scope * sc, char * name)
{
	char * n = bx_normalizeIdent(name);
	char * p = strchr(n,'.');
	if (p) {
		b_varl * ov = 0;
		b_object * o = 0;
		(*p) = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			free(n);
			if (!o) { be_throw(be_nullptr); return 0; }
			return bv_finduscope(o->theObject, p);
		}
		free(n);
	}
	return sc;
}

b_scope * bv_findc(b_scope * sc, char * name)
{
	int i;
	char * n = bx_normalizeIdent(name);
	char * p = strchr(n,'.');
	if (p) {
		b_varl * ov = 0;
		b_object * o = 0;
		(*p) = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			free(n);
			if (!o) { be_throw(be_nullptr); return 0; }
			return bv_findc(o->theObject, p);
		}
		free(n);
	} else {
		for (i=0; i<sc->ccount; i++) {
			if (!strcmp(n, sc->cnames[i])) {
				free(n);
				return sc->classes[i];
			}
		}
		free(n);
		if (sc->fallback) {
			return bv_findc(sc->fallback, name);
		} else {
			be_throw(be_undeclared);
		}
	}
	return 0;
}

b_varl * bv_afind(b_scope * sc, char * name, int a, int b, int c)
{
	int i;
	b_array * arr = bv_finda(sc, name);
	if (!arr) {
		be_throw(be_undeclared);
		return 0;
	} else if (!arr->elem) {
		be_throw(be_memory);
		return 0;
	}
	switch (arr->dim) {
		case 0: a=0;
		case 1: b=0;
		case 2: c=0;
	}
	if (a < 0 || b < 0 || c < 0 || a >= arr->dim1 || b >= arr->dim2 || c >= arr->dim3) {
		be_throw(be_subscript);
		return 0;
	}
	i = (a * arr->dim2 * arr->dim3) + (b * arr->dim3) + c;
	return &arr->elem[i];
}

void bv_del(b_scope * sc, char * name)
{
	int i,j;
	char * n = bx_normalizeIdent(name);
	char * p = strchr(n,'.');
	if (p) {
		b_varl * ov = 0;
		b_object * o = 0;
		(*p) = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			if (!o) { be_throw(be_nullptr); return; }
			bv_del(o->theObject, p);
		}
		free(n);
	} else {
		if (!(sc->vars && sc->names && sc->count)) {
			be_throw(be_undeclared);
			return;
		}
		for (i=0; i<sc->count; i++) {
			if (!strcmp(n, sc->names[i])) {
				free(n);
				bv_freeBv(sc->vars[i]);
				free(sc->names[i]);
				for (j=i+1; j<sc->count; j++) {
					sc->vars[j-1] = sc->vars[j];
					sc->names[j-1] = sc->names[j];
				}
				sc->vars = (b_varl **)realloc(sc->vars, (sc->count-1) * sizeof(b_varl *));
				sc->count--;
				return;
			}
		}
		free(n);
		be_throw(be_undeclared);
	}
}

void bv_dela(b_scope * sc, char * name)
{
	int i,j;
	char * n = bx_normalizeIdent(name);
	char * p = strchr(n,'.');
	if (p) {
		b_varl * ov = 0;
		b_object * o = 0;
		(*p) = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			if (!o) { be_throw(be_nullptr); return; }
			bv_dela(o->theObject, p);
		}
		free(n);
	} else {
		if (!(sc->arrays && sc->anames && sc->acount)) {
			be_throw(be_undeclared);
			return;
		}
		for (i=0; i<sc->acount; i++) {
			if (!strcmp(n, sc->anames[i])) {
				free(n);
				bv_freeArray(sc->arrays[i]);
				free(sc->anames[i]);
				for (j=i+1; j<sc->acount; j++) {
					sc->arrays[j-1] = sc->arrays[j];
					sc->anames[j-1] = sc->anames[j];
				}
				sc->arrays = (b_array **)realloc(sc->arrays, (sc->acount-1) * sizeof(b_array *));
				sc->acount--;
				return;
			}
		}
		free(n);
		be_throw(be_undeclared);
	}
}

void bv_add(b_scope * sc, char * name, b_varl * v)
{
	char * p = strchr(name,'.');
	if (p) {
		char * n = substr(name,p);
		b_varl * ov = 0;
		b_object * o = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			if (!o) { be_throw(be_nullptr); return; }
			bv_add(o->theObject, p, v);
		}
		free(n);
	} else {
		if (!v->dyna) v = bv_copyBv(v);
		v->dyna = 0;
		
		if (!sc->vars) {
			sc->vars = (b_varl **)calloc(1, sizeof(b_varl *));
			if (sc->vars) {
				sc->vars[0] = v;
			} else { be_throw(be_memory); }
		} else {
			b_varl ** tmp = (b_varl **)realloc(sc->vars, (sc->count+1) * sizeof(b_varl *));
			if (tmp) {
				sc->vars = tmp;
				sc->vars[sc->count] = v;
			} else { be_throw(be_memory); }
		}
		if (!sc->names) {
			sc->names = (char **)calloc(1, sizeof(char *));
			if (sc->names) {
				sc->names[0] = bx_normalizeIdent(name);
				sc->count = 1;
			} else { be_throw(be_memory); }
		} else {
			char ** tmp = (char **)realloc(sc->names, (sc->count+1) * sizeof(char *));
			if (tmp) {
				sc->names = tmp;
				sc->names[sc->count] = bx_normalizeIdent(name);
				sc->count++;
			} else { be_throw(be_memory); }
		}
	}
}

void bv_adda(b_scope * sc, char * name, b_array * a)
{
	char * p = strchr(name,'.');
	if (p) {
		char * n = substr(name,p);
		b_varl * ov = 0;
		b_object * o = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			if (!o) { be_throw(be_nullptr); return; }
			bv_adda(o->theObject, p, a);
		}
		free(n);
	} else {
		if (!sc->arrays) {
			sc->arrays = (b_array **)calloc(1, sizeof(b_array *));
			if (sc->arrays) {
				sc->arrays[0] = a;
			} else { be_throw(be_memory); }
		} else {
			b_array ** tmp = (b_array **)realloc(sc->arrays, (sc->acount+1) * sizeof(b_array *));
			if (tmp) {
				sc->arrays = tmp;
				sc->arrays[sc->acount] = a;
			} else { be_throw(be_memory); }
		}
		if (!sc->anames) {
			sc->anames = (char **)calloc(1, sizeof(char *));
			if (sc->anames) {
				sc->anames[0] = bx_normalizeIdent(name);
				sc->acount = 1;
			} else { be_throw(be_memory); }
		} else {
			char ** tmp = (char **)realloc(sc->anames, (sc->acount+1) * sizeof(char *));
			if (tmp) {
				sc->anames = tmp;
				sc->anames[sc->acount] = bx_normalizeIdent(name);
				sc->acount++;
			} else { be_throw(be_memory); }
		}
	}
}

void bv_addus(b_scope * sc, char * name, b_usub * us)
{
	char * p = strchr(name,'.');
	if (p) {
		char * n = substr(name,p);
		b_varl * ov = 0;
		b_object * o = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			if (!o) { be_throw(be_nullptr); return; }
			bv_addus(o->theObject, p, us);
		}
		free(n);
	} else {
		if (!sc->usubs) {
			sc->usubs = (b_usub **)calloc(1, sizeof(b_usub));
			if (sc->usubs) {
				sc->usubs[0] = us;
			} else { be_throw(be_memory); }
		} else {
			b_usub ** tmp = (b_usub **)realloc(sc->usubs, (sc->uscount+1) * sizeof(b_usub));
			if (tmp) {
				sc->usubs = tmp;
				sc->usubs[sc->uscount] = us;
			} else { be_throw(be_memory); }
		}
		if (!sc->usnames) {
			sc->usnames = (char **)calloc(1, sizeof(char *));
			if (sc->usnames) {
				sc->usnames[0] = bx_normalizeIdent(name);
				sc->uscount = 1;
			} else { be_throw(be_memory); }
		} else {
			char ** tmp = (char **)realloc(sc->usnames, (sc->uscount+1) * sizeof(char *));
			if (tmp) {
				sc->usnames = tmp;
				sc->usnames[sc->uscount] = bx_normalizeIdent(name);
				sc->uscount++;
			} else { be_throw(be_memory); }
		}
	}
}

void bv_adduf(b_scope * sc, char * name, b_ufunc * uf)
{
	char * p = strchr(name,'.');
	if (p) {
		char * n = substr(name,p);
		b_varl * ov = 0;
		b_object * o = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			if (!o) { be_throw(be_nullptr); return; }
			bv_adduf(o->theObject, p, uf);
		}
		free(n);
	} else {
		if (!sc->ufuncs) {
			sc->ufuncs = (b_ufunc **)calloc(1, sizeof(b_ufunc));
			if (sc->ufuncs) {
				sc->ufuncs[0] = uf;
			} else { be_throw(be_memory); }
		} else {
			b_ufunc ** tmp = (b_ufunc **)realloc(sc->ufuncs, (sc->ufcount+1) * sizeof(b_ufunc));
			if (tmp) {
				sc->ufuncs = tmp;
				sc->ufuncs[sc->ufcount] = uf;
			} else { be_throw(be_memory); }
		}
		if (!sc->ufnames) {
			sc->ufnames = (char **)calloc(1, sizeof(char *));
			if (sc->ufnames) {
				sc->ufnames[0] = bx_normalizeIdent(name);
				sc->ufcount = 1;
			} else { be_throw(be_memory); }
		} else {
			char ** tmp = (char **)realloc(sc->ufnames, (sc->ufcount+1) * sizeof(char *));
			if (tmp) {
				sc->ufnames = tmp;
				sc->ufnames[sc->ufcount] = bx_normalizeIdent(name);
				sc->ufcount++;
			} else { be_throw(be_memory); }
		}
	}
}

void bv_addc(b_scope * sc, char * name, b_scope * c)
{
	char * p = strchr(name,'.');
	if (p) {
		char * n = substr(name,p);
		b_varl * ov = 0;
		b_object * o = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			if (!o) { be_throw(be_nullptr); return; }
			bv_addc(o->theObject, p, c);
		}
		free(n);
	} else {
		if (!sc->classes) {
			sc->classes = (b_scope **)calloc(1, sizeof(b_scope *));
			if (sc->classes) {
				sc->classes[0] = c;
			} else { be_throw(be_memory); }
		} else {
			b_scope ** tmp = (b_scope **)realloc(sc->classes, (sc->ccount+1) * sizeof(b_scope *));
			if (tmp) {
				sc->classes = tmp;
				sc->classes[sc->ccount] = c;
			} else { be_throw(be_memory); }
		}
		if (!sc->cnames) {
			sc->cnames = (char **)calloc(1, sizeof(char *));
			if (sc->cnames) {
				sc->cnames[0] = bx_normalizeIdent(name);
				sc->ccount = 1;
			} else { be_throw(be_memory); }
		} else {
			char ** tmp = (char **)realloc(sc->cnames, (sc->ccount+1) * sizeof(char *));
			if (tmp) {
				sc->cnames = tmp;
				sc->cnames[sc->ccount] = bx_normalizeIdent(name);
				sc->ccount++;
			} else { be_throw(be_memory); }
		}
	}
}

void bv_aadd(b_scope * sc, char * name, b_type t, int n, int a, int b, int c)
{
	bv_adda(sc,name,bv_createArray(t,n,a,b,c));
}

void bv_setsp(b_varl * existing, b_varl * v)
{
	if (existing) {
		switch (v->type) {
		case bt_float:
			bv_setDouble(existing, v->fval);
			break;
		case bt_integer:
			bv_setInt(existing, v->ival);
			break;
		case bt_string:
			bv_setString(existing, v->sval);
			break;
		case bt_object:
			bv_setObject(existing, v->oval);
			break;
		case bt_exception:
			bv_setException(existing, v->eval);
			break;
		}
	}
}

static int bv__seth(b_scope * sc, char * name, b_varl * v)
{
	int i;
	char * n = bx_normalizeIdent(name);
	char * p = strchr(n,'.');
	if (p) {
		b_varl * ov = 0;
		b_object * o = 0;
		(*p) = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			free(n);
			if (!o) { be_throw(be_nullptr); return 1; }
			return bv__seth(o->theObject, p, v);
		}
		free(n);
	} else {
		for (i=0; i<sc->count; i++) {
			if (!strcmp(n, sc->names[i])) {
				if (sc->vars[i]->readOnly) {
					be_throw(be_constant);
					free(n);
					return 1;
				}
				bv_freeBv(sc->vars[i]);
				
				if (!v->dyna) v = bv_copyBv(v);
				v->dyna = 0;
				
				sc->vars[i] = v;
				free(n);
				return 1;
			}
		}
		free(n);
	}
	if (sc->fallback) {
		return bv__seth(sc->fallback, name, v);
	} else {
		return 0;
	}
}

void bv_seth(b_scope * sc, char * name, b_varl * v)
{
	if (!bv__seth(sc, name, v)) {
		bv_add(sc, name, v);
	}
}

static int bv__sets(b_scope * sc, char * name, b_varl * v)
{
	int i;
	char * n = bx_normalizeIdent(name);
	char * p = strchr(n,'.');
	if (p) {
		b_varl * ov = 0;
		b_object * o = 0;
		(*p) = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			free(n);
			if (!o) { be_throw(be_nullptr); return 1; }
			return bv__sets(o->theObject, p, v);
		}
		free(n);
	} else {
		for (i=0; i<sc->count; i++) {
			if (!strcmp(n, sc->names[i])) {
				bv_setsp(sc->vars[i], v);
				free(n);
				return 1;
			}
		}
		free(n);
	}
	if (sc->fallback) {
		return bv__sets(sc->fallback, name, v);
	} else {
		return 0;
	}
}

void bv_sets(b_scope * sc, char * name, b_varl * v)
{
	if (!bv__sets(sc, name, v)) {
		bv_add(sc, name, v);
	}
}

static int bv__seta(b_scope * sc, char * name, b_array * a)
{
	int i;
	char * n = bx_normalizeIdent(name);
	char * p = strchr(n,'.');
	if (p) {
		b_varl * ov = 0;
		b_object * o = 0;
		(*p) = 0;
		p++;
		ov = bv_find(sc,n);
		if (!ov) {
			be_throw(be_undeclared);
		} else if (!bv_assertObject(ov)) {
			be_throw(be_cast);
		} else {
			o = ov->oval;
			free(n);
			if (!o) { be_throw(be_nullptr); return 1; }
			return bv__seta(o->theObject, p, a);
		}
		free(n);
	} else {
		for (i=0; i<sc->acount; i++) {
			if (!strcmp(n, sc->anames[i])) {
				bv_freeArray(sc->arrays[i]);
				sc->arrays[i] = a;
				free(n);
				return 1;
			}
		}
	}
	free(n);
	if (sc->fallback) {
		return bv__seta(sc->fallback, name, a);
	} else {
		return 0;
	}
}

void bv_seta(b_scope * sc, char * name, b_array * a)
{
	if (!bv__seta(sc, name, a)) {
		bv_adda(sc, name, a);
	}
}

void bv_aseth(b_scope * sc, char * name, int a, int b, int c, b_varl * nv)
{
	b_varl * v = bv_afind(sc, name, a, b, c);
	if (v) {
		if (v->sval) free(v->sval);
		if (v->oval) bv_unuseObj(v->oval);
		
		if (!nv->dyna) nv = bv_copyBv(nv);
		nv->dyna = 0;
		
		(*v) = (*nv);
	} else {
		be_cthrow(be_undeclared);
	}
}

void bv_asets(b_scope * sc, char * name, int a, int b, int c, b_varl * nv)
{
	b_varl * v = bv_afind(sc, name, a, b, c);
	if (v) {
		bv_setsp(v, nv);
	} else {
		be_cthrow(be_undeclared);
	}
}

int bv_equal(b_varl * a, b_varl * b)
{
	if (!a && !b) {
		return 1;
	} else if (!a || !b) {
		return 0;
	} else if (a->type == bt_object && b->type == bt_object) {
		if (a->oval == b->oval) return 1;
		return 0;
	} else if (a->type == bt_object || b->type == bt_object) {
		return 0;
	} else if (a->type == bt_string || b->type == bt_string) {
		if (strcmp(bv_getString(a), bv_getString(b))) {
			return 0;
		} else {
			return 1;
		}
	} else if (a->type == bt_float || b->type == bt_float) {
		if (bv_getDouble(a) == bv_getDouble(b)) {
			return 1;
		} else {
			return 0;
		}
	} else {
		if (bv_getInt(a) == bv_getInt(b)) {
			return 1;
		} else {
			return 0;
		}
	}
}

double bv_compare(b_varl * a, b_varl * b)
{
	if (!a && !b) {
		return 0.0;
	} else if (!a || !b) {
		return 1.0;
	} else if (a->type == bt_object && b->type == bt_object) {
		if (a->oval == b->oval) return 0.0;
		return 1.0;
	} else if (a->type == bt_object || b->type == bt_object) {
		return 1.0;
	} else if (a->type == bt_string || b->type == bt_string) {
		return (double)strcmp(bv_getString(a), bv_getString(b));
	} else {
		return bv_getDouble(a)-bv_getDouble(b);
	}
}

b_exception bv_resolveType(b_scope * sc, char * typestr, b_type * type, b_scope ** clas)
{
	b_scope * c;
	     if (!strcasecmp(typestr, "int"))       (*type) = bt_integer;
	else if (!strcasecmp(typestr, "integer"))   (*type) = bt_integer;
	else if (!strcasecmp(typestr, "double"))    (*type) = bt_float;
	else if (!strcasecmp(typestr, "float"))     (*type) = bt_float;
	else if (!strcasecmp(typestr, "string"))    (*type) = bt_string;
	else if (!strcasecmp(typestr, "object"))  { (*type) = bt_object; (*clas) = 0; }
	else if (!strcasecmp(typestr, "exception")) (*type) = bt_exception;
	else if (!bx_verifyIdent(typestr))      return be_syntax;
	else if (!(c = bv_findc(sc,typestr))) { be_clearerr(); return be_type; }
	else { (*type) = bt_object; (*clas) = c; }
	return be_none;
}


