/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef ___B_VARS_H___
#define ___B_VARS_H___

#include "btype.h"

/* working with b_objects */
b_object * bv_createObj(char * className, b_scope * class);
void bv_freeObj(b_object * o);
b_object * bv_copyObj(b_object * o);
b_object * bv_useObj(b_object * o);
b_object * bv_unuseObj(b_object * o);

/* working with b_varls */
b_varl * bv_createBv(b_type type, b_scope * otype, double d, signed long int i, char * s, b_object * o, b_exception e);
void bv_freeBv(b_varl * v);
void bv_cfreeBv(b_varl * v);
b_varl * bv_copyBv(b_varl * v);
b_varl * bv_makeStatic(b_varl * v);
b_varl * bv_makeDynamic(b_varl * v);
int bv_scopeIsChild(b_scope * ocl, b_scope * cl);

/* working with b_arrays */
b_array * bv_createArray(b_type t, int n, int a, int b, int c);
b_array * bv_redimArray(b_array * arr, int n, int a, int b, int c);
void bv_freeArray(b_array * arr);
b_array * bv_copyArray(b_array * arr);

/* working with b_usubs */
b_usub * bv_createUsub(int n, char * pl, char * a, b_type at, char * b, b_type bt, char * c, b_type ct, char * d, b_type dt, char * code);
void bv_freeUsub(b_usub * us);
b_usub * bv_copyUsub(b_usub * us);

/* working with b_ufuncs */
b_ufunc * bv_createUfunc(b_type ret, int n, char * pl, char * a, b_type at, char * b, b_type bt, char * c, b_type ct, char * d, b_type dt, char * code);
void bv_freeUfunc(b_ufunc * uf);
b_ufunc * bv_copyUfunc(b_ufunc * uf);

/* working with b_scopes */
b_scope * bv_newScope();
b_scope * bv_newScopeChild(b_scope * scope);
b_scope * bv_newScopeSibling(b_scope * scope);
b_scope * bv_newScopeParent(b_scope * scope);
void bv_freeVA(b_scope * sc);
void bv_freeScope(b_scope * sc, int deep);
b_scope * bv_copyScope(b_scope * sc, int deep);

/* get values out of varls */
double bv_getDouble(b_varl * v);
signed long int bv_getInt(b_varl * v);
char * bv_getString(b_varl * v);
b_object * bv_getObject(b_varl * v);
b_exception bv_getException(b_varl * v);
int bv_getBoolean(b_varl * v);

/* check type of varls */
int bv_assertNumeric(b_varl * v);
int bv_assertString(b_varl * v);
int bv_assertObject(b_varl * v);

/* set values of varls */
void bv_setDouble(b_varl * v, double d);
void bv_setInt(b_varl * v, signed long int i);
void bv_setString(b_varl * v, char * s);
void bv_setObject(b_varl * v, b_object * o);
void bv_setException(b_varl * v, b_exception e);

/* create varl around value */
b_varl * bv_doubleToBv(double d);
b_varl * bv_intToBv(signed long int i);
b_varl * bv_stringToBv(char * s);
b_varl * bv_objectToBv(b_object * o, b_scope * ot);
b_varl * bv_exceptionToBv(b_exception e);

/* find the various things in a scope */
b_varl * bv_find(b_scope * sc, char * name);
b_array * bv_finda(b_scope * sc, char * name);
b_usub * bv_findus(b_scope * sc, char * name);
b_ufunc * bv_finduf(b_scope * sc, char * name);
b_scope * bv_finduscope(b_scope * sc, char * name);
b_scope * bv_findc(b_scope * sc, char * name);

/* find an element of an array */
b_varl * bv_afind(b_scope * sc, char * name, int a, int b, int c);

/* add various things to a scope */
void bv_add(b_scope * sc, char * name, b_varl * v);
void bv_adda(b_scope * sc, char * name, b_array * a);
void bv_addus(b_scope * sc, char * name, b_usub * us);
void bv_adduf(b_scope * sc, char * name, b_ufunc * uf);
void bv_addc(b_scope * sc, char * name, b_scope * c);

/* add a new array */
void bv_aadd(b_scope * sc, char * name, b_type t, int n, int a, int b, int c);

/* delete variable or array */
void bv_del(b_scope * sc, char * name);
void bv_dela(b_scope * sc, char * name);

/* set variable */
void bv_setsp(b_varl * existing, b_varl * v);
void bv_seth(b_scope * sc, char * name, b_varl * v);
void bv_sets(b_scope * sc, char * name, b_varl * v);
void bv_seta(b_scope * sc, char * name, b_array * a);
void bv_aseth(b_scope * sc, char * name, int a, int b, int c, b_varl * nv);
void bv_asets(b_scope * sc, char * name, int a, int b, int c, b_varl * nv);

int bv_equal(b_varl * a, b_varl * b);
double bv_compare(b_varl * a, b_varl * b);
b_exception bv_resolveType(b_scope * sc, char * typestr, b_type * type, b_scope ** clas);

#endif
